/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#define	LIBTEKLTI_EXPORT	(0)

#ifndef	_cplusplus
#define	_cplusplus
#endif	/* _cplusplus */
#include "teklti.h"

#include <errno.h>


#ifndef	TEKLTI_USE_BZIP2
#error You can not compile the BZIP2 interfaces unless TEKLTI_USE_BZIP2 is defined.
#endif	/* TEKLTI_USE_BZIP2 */


/*
  Our private flags.
*/

enum TekFileBZ2ASCII_Mode
{
	TekFileBZ2ASCII_Mode_WriteToFile = 0x0001,
	TekFileBZ2ASCII_Mode_EndOfStream = 0x0002,
	TekFileBZ2ASCII_Mode_UseLessMemory = 0x0004
};


MKTEKUUID(UTekFileBZ2ASCII, "6CC3729C-2D12-11DA-887B-000BDBC434D9")
MKTEKDEBUGSTRING(DBTekFileBZ2ASCII, "TekFileBZ2ASCII")



/**
Instantizer.
*/
TekFileBZ2ASCII::TekFileBZ2ASCII()
{
	TEKSTD_IMPLEMENT_BEGIN_MYSELF
	TEKSTD_IMPLEMENT_INTERFACE_MYSELF(UTekFile)
	TEKSTD_IMPLEMENT_INTERFACE_MYSELF(UTekFileASCII)
	TEKSTD_IMPLEMENT_INTERFACE_MYSELF(UTekFileBZ2)
	TEKSTD_IMPLEMENT_INTERFACE_MYSELF(UTekFileBZ2ASCII)
	TEKSTD_IMPLEMENT_END_MYSELF

	/* Set everything up. */
	this->PrivFileDesc = NULL;
	this->PrivBZ2FileDesc = NULL;
	this->PrivFlags = 0;
	this->PrivBlockSize = TEKLTI_USE_BZIP2_BLOCKSIZE;
	this->PrivBytesIn = this->PrivBytesOut = 0;
	this->PrivVerbosity = 0;
	this->PrivWorkFactor = 30;
	this->PrivFilename.uchar_t_ascii =
	  (char *)calloc(PATH_MAX,sizeof(char));
	this->PrivFilename.uchar_t_asciilen = 0;
}

/**
Destructor.
*/
TekFileBZ2ASCII::~TekFileBZ2ASCII()
{
	this->Close();

	if ( this->PrivFilename.uchar_t_ascii != NULL )
		free(this->PrivFilename.uchar_t_ascii);

	TEKSTD_IMPLEMENT_FREE
}


TEKSTD_SOURCE_FUNC(TekFileBZ2ASCII);


/**
    \fn TekFileBZ2ASCII::ReadLine(uchar_t** Buffer, unsigned long ReadMax)
    
    Reads up to ReadMax characters from a file and puts it in a
    uchar_t.  The Buffer can be an already-created uchar_t, or
    can be a pointer to a uchar_t* to store the new uchar_t in.
    
    NOTE: For optimization reasons, always let this function
    create the uchar_t for you.  However, the same uchar_t can
    be passed in to this function without any problems.
 */

TEKERR TekFileBZ2ASCII::ReadLine(uchar_t** Buffer, unsigned long ReadMax)
{
	char * buf;
	uchar_t * ubuffer;
	unsigned long l, in;
	TEKERR terr;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc == NULL )
		return TEKERR_SEQUENCE;
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */


	if ( ReadMax > TEKFILEASCII_STDBUFLEN )
		ReadMax = TEKFILEASCII_STDBUFLEN;

	/* Get ready to check the passed-in pointer. */
	ubuffer = *Buffer;

	if ( ubuffer == NULL )
	{
		/* Caller wants us to create a uchar_t. */
		ubuffer = (uchar_t *)malloc(sizeof(uchar_t));
		/* If malloc() didn't fail, try to allocate the string. */
		if ( ubuffer == NULL )
		{
			return TEKERR_MEMORY;
		} else {
			ubuffer->uchar_t_ascii = (char *)
				calloc(TEKFILEASCII_STDBUFLEN + 1, sizeof(char));
			if ( ubuffer->uchar_t_ascii == NULL )
			{
				/* If calloc() failed, there's nothing to do. */
				free(ubuffer);
				return TEKERR_MEMORY;
			}

			/* Set up the length value. */
			ubuffer->uchar_t_asciilen =
				(TEKFILEASCII_STDBUFLEN + 1) * sizeof(char);
		}

		*Buffer = ubuffer;
	}
#ifdef	TEKLTI_ENFORCE_PRIVACY
	else
	{
		/* Clear out the memory. */
		memset(
			ubuffer->uchar_t_ascii,
			'\0',
			(TEKFILEASCII_STDBUFLEN + 1) * sizeof(char)
		);
	}
#endif	/* TEKLTI_ENFORCE_PRIVACY */

	buf = ubuffer->uchar_t_ascii;

	for ( l=0; l < ReadMax; l++ )
	{
		terr = this->Read(buf, 1, &in);
		if ( terr != TEKERR_OK )
		{
			if (
			( terr == TEKERR_FILE_EOF ) &&
			( in != 0 )
			)
			{
				buf[1] = '\0';
			}
			
			return terr;
		}

		/* Detect for carriage return. */
		if ( buf[0] == '\r' )
		{
			/* Check for a line feed ahead of it. */
			buf++;
			terr = this->Read(buf, 1, &in);
			if ( terr != TEKERR_OK )
			{
				if (
				( terr == TEKERR_FILE_EOF ) &&
				( in != 0 )
				)
				{
					buf[0] = '\0';
				}
			
				return terr;
			}

			/* If yes, cover them with zeroes. */
			if ( buf[0] = '\n' )
			{
				buf--;
				buf[0] = '\0';
				buf[1] = '\0';
				return TEKERR_FILE_EOF;
			}

			/* If not, continue looking. */
			buf++;
			continue;
		} else {

		/* Detect for line feed. */
		if ( buf[0] == '\n' )
		{
			buf[0] = '\0';
			return TEKERR_FILE_EOF;
		}
		}

		buf++;
	}

	return TEKERR_OK;
}

/**
    \fn TekFileBZ2ASCII::WriteLine(const uchar_t* Buffer)
    
    Writes a ascii-part of a uchar_t to the file.
 */

TEKERR TekFileBZ2ASCII::WriteLine(const uchar_t* Buffer)
{
	unsigned long Wrote;
	const char * Buf;
	size_t RealLen;
	TEKERR terr;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc == NULL )
		return TEKERR_SEQUENCE;
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Get ready to look for true EOL. */
	Buf = Buffer->uchar_t_ascii;
	RealLen = 0;

	/* Look for EOL. */
	for ( RealLen=0; RealLen < Buffer->uchar_t_asciilen; RealLen++ )
	{
		if ( Buf[0] == '\0' )
			break;
	}

	/* Write the result to the stream. */
	terr = this->Write(Buffer->uchar_t_ascii, RealLen, &Wrote);
	if ( terr != TEKERR_OK )
		return terr;
	if ( Wrote < RealLen )
		return TEKERR_FILE_DISKFULL;

	/* Terminate the line. */
#ifdef	TEKLTI_WRITELINE_CRLF
	terr = this->Write((void *)"\r\n", 2, &Wrote);
	if ( terr != TEKERR_OK )
		return terr;
	if ( Wrote < 2 )
	{
#else	/* not TEKLTI_WRITELINE_CRLF */
	terr = this->Write((void *)"\n", 1, &Wrote);
	if ( terr != TEKERR_OK )
		return terr;
	if ( Wrote < 1 )
	{
#endif	/* not TEKLTI_WRITELINE_CRLF */
		return TEKERR_FILE_DISKFULL;
	}
	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_BytesIn(uint64_t * RetVal)
    
    Retrieves the amount of bytes bzip2 recieved. \n
    \n
    <b>NOTE:</b> You will get a TEKERR_UNAVAILABLE if you try calling
    this function while the BytesIn data is unavailable.
 */

TEKERR TekFileBZ2ASCII::get_BytesIn(uint64_t * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = this->PrivBytesIn;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_BytesOut(uint64_t * RetVal)
    
    Retrieves the amount of bytes bzip2 put out. \n
    \n
    <b>NOTE:</b> You will get a TEKERR_UNAVAILABLE if you try calling
    this function while the BytesIn data is unavailable.
 */
 

TEKERR TekFileBZ2ASCII::get_BytesOut(uint64_t * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = this->PrivBytesOut;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::put_UseLessMemory(unsigned int NewVal)
    
    Sets or resets the UseLessMemory flag. \n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened.
 */

TEKERR TekFileBZ2ASCII::put_UseLessMemory(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	if ( NewVal == 0 )
	{
		/* Reset the UseLessMemory flag. */
		if ( this->PrivFlags & TekFileBZ2ASCII_Mode_UseLessMemory )
			this->PrivFlags ^= TekFileBZ2ASCII_Mode_UseLessMemory;
	} else {
		/* Set the UseLessMemory flag. */
		this->PrivFlags |= TekFileBZ2ASCII_Mode_UseLessMemory;
	}

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_UseLessMemory(unsigned int * RetVal)
    
    Retrieves the UseLessMemory flag. \n
 */

TEKERR TekFileBZ2ASCII::get_UseLessMemory(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	if ( this->PrivFlags & TekFileBZ2ASCII_Mode_UseLessMemory )
		*RetVal = 1;
	else
		*RetVal = 0;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::put_WorkFactor(unsigned int NewVal)
    
    Sets the work factor number.\n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened. \n
    \n
    <b>WARNING:</b> It is best that you not set this yourself.
    The bzip2 library handles this on its own.
 */

TEKERR TekFileBZ2ASCII::put_WorkFactor(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	this->PrivWorkFactor = NewVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_WorkFactor(unsigned int * RetVal)
    
    Retrieves the work factor number.\n
 */

TEKERR TekFileBZ2ASCII::get_WorkFactor(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = (unsigned int)this->PrivWorkFactor;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::put_Verbosity(unsigned int NewVal)
    
    Sets the current verbosity value.\n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened.
 */

TEKERR TekFileBZ2ASCII::put_Verbosity(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	this->PrivVerbosity = (int)NewVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_Verbosity(unsigned int * RetVal)
    
    Retrieves the current verbosity value.
 */

TEKERR TekFileBZ2ASCII::get_Verbosity(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = (unsigned int)this->PrivVerbosity;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::put_BlockSize(unsigned int NewVal)
    
    Sets the block size to use for writing.  Allowed
    values are 1-9.\n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened.
 */
TEKERR TekFileBZ2ASCII::put_BlockSize(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	if ( ( NewVal < 1 ) || ( NewVal > 9 ) )
		return TEKERR_FAIL;

	this->PrivBlockSize = NewVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_BlockSize(unsigned int * RetVal)
    
    Retrieves the block size to use for writing.
 */
TEKERR TekFileBZ2ASCII::get_BlockSize(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = this->PrivBlockSize;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::CloseStream()
    
    Tells the BZIP2 library to close the stream currently
    open for the internally held FILE *\n
    \n
    <b>NOTE:</b> TekFileBZ2ASCII::Close() should be used instead.
    TekFileBZ2ASCII::CloseStream() will not close the internally
    held FILE *
 */
TEKERR TekFileBZ2ASCII::CloseStream()
{
	struct i64 bzIn, bzOut;
	int bzError;

#ifndef NO_INTERNAL_NULL_CHECKS
        if ( this->PrivBZ2FileDesc == NULL )
                return TEKERR_SEQUENCE;
#endif  /* NO_INTERNAL_NULL_CHECKS */

	/* Reset the EndOfStream flag. */
	if ( this->PrivFlags & TekFileBZ2ASCII_Mode_EndOfStream )
		this->PrivFlags ^= TekFileBZ2ASCII_Mode_EndOfStream;

	/* Determine if there is a BZ2 Descriptor open. */
	if ( this->PrivBZ2FileDesc != NULL )
	{
		/* How should we close the stream? */
		if ( this->PrivFlags & TekFileBZ2ASCII_Mode_WriteToFile )
		{
			/* Do a write close. */
			BZ2_bzWriteClose64(
				&bzError,
				this->PrivBZ2FileDesc,
				0,
				&bzIn.i64_a,
				&bzIn.i64_b,
				&bzOut.i64_a,
				&bzOut.i64_b
			);
			/* Convert our data into uint64_t's. */
			memcpy(&this->PrivBytesIn, &bzIn, 8);
			memcpy(&this->PrivBytesOut, &bzOut, 8);

			/* Handle the error, if necessary. */
			if ( bzError == BZ_IO_ERROR )
				return TEKERR_FILE_IO;
		} else {
			/* Do a read close. */
			BZ2_bzReadClose(
				&bzError,
				this->PrivBZ2FileDesc
			);
		}

		/* Handle any catastrophic errors. */
		if ( bzError != BZ_OK )
			return TEKERR_UNEXPECTED;

		/* NULLify the stream descriptor. */
		this->PrivBZ2FileDesc = NULL;
	}

	/* Return to caller. */
	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::OpenStream()
    
    Tells the BZIP2 library to open a stream for the internally
    held FILE *
 */
TEKERR TekFileBZ2ASCII::OpenStream()
{
	BZFILE * NewStream;
	int bzError;
	int UseSmallMemory;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* How should we open the stream? */
	if ( this->PrivFlags & TekFileBZ2ASCII_Mode_WriteToFile )
	{
		//* Open a stream. */
		NewStream = BZ2_bzWriteOpen(
			&bzError,
			this->PrivFileDesc,
			this->PrivBlockSize,
			this->PrivVerbosity,
			this->PrivWorkFactor
		);
	} else {
		/* Determine if SmallMemory is to be used. */
		if ( this->PrivFlags & TekFileBZ2ASCII_Mode_UseLessMemory )
			UseSmallMemory = 1;
		else
			UseSmallMemory = 0;

		/* Open a stream. */
		NewStream = BZ2_bzReadOpen(
			&bzError,
			this->PrivFileDesc,
			this->PrivVerbosity,
			UseSmallMemory,
			NULL,
			0
		);
	}

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_FILE(FILE ** FileDescriptor)
    
    Obtains the handle used to access the open file.\n
    \n
    <b>NOTE:</b>  If there is an open BZ2 stream in the instance, you will get a \n
    TEKERR_ACCESS returned to you.
 */
TEKERR TekFileBZ2ASCII::get_FILE(FILE ** FileDescriptor)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

#ifndef	NO_SIZE_T_CHECKS
	if ( FileDescriptor == (FILE **)NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	*FileDescriptor = this->PrivFileDesc;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::get_Filename(uchar_t ** Name)
    
    Obtains the filename in the form of a uchar.\n
    \n
    <b>NOTE:</b>  If a Filename has not yet been set, a
    TEKERR_ACCESS will be returned.
 */
TEKERR TekFileBZ2ASCII::get_Filename(uchar_t ** Name)
{
	uchar_t * RetVal;

#ifndef	NO_SIZE_T_CHECKS
	if ( Name == (uchar_t **)NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	/* Check to see if the constructor was able to allocate memory. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	if ( this->PrivFilename.uchar_t_ascii[0] == '\0' )
		return TEKERR_ACCESS;

	RetVal = char2uchar(this->PrivFilename.uchar_t_ascii);
	if ( RetVal == NULL )
		return TEKERR_MEMORY;

	*Name = RetVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::put_FILE(FILE * FileDescriptor)
    
    Sets the handle used to access a file.\n
    \n
    <b>NOTE:</b> If there is a descriptor open in the descriptor,
    and <i>FileDescriptor</i> is not NULL, a TEKERR_ACCESS will be returned. 
    However, if <i>FileDescriptor</i> is NULL, the file descriptor and its 
    associated stream will be closed.
    \n
    <b>WARNING!!!!!</b>  DO NOT USE THE FILE DESCRIPTOR YOU PASS IN 
    TO THIS FUNCTION UNLESS YOU HAVE RE-RETRIEVED IT USING 
    TekFileBZ2ASCII::get_FILE()!  Utilizing the descriptor in a direct fashion after a 
    successful call to put_FILE() will result in undefined results!!!!!
 */
TEKERR TekFileBZ2ASCII::put_FILE(FILE * FileDescriptor)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
	if ( ( this->PrivFileDesc != NULL ) && ( FileDescriptor != NULL ) )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	if ( ( FileDescriptor == NULL ) && ( this->PrivFileDesc != NULL ) )
	{
		if ( this->PrivBZ2FileDesc != NULL )
			this->CloseStream();
		this->Close();
	}

	this->PrivFileDesc = FileDescriptor;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2ASCII::put_Filename(uchar_t * Name)
    
    Sets the filename to access.\n
    \n
    <b>NOTE:</b> If there is a descriptor open in the descriptor,
    and "Name" is not NULL, a TEKERR_ACCESS will be returned.
 */
TEKERR TekFileBZ2ASCII::put_Filename(uchar_t * Name)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

#ifndef	NO_SIZE_T_CHECKS
	if ( Name == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	/* Determine the length. */
	this->PrivFilename.uchar_t_asciilen =
	  strlen(Name->uchar_t_ascii);

	/* Check to see if the constructor was able to allocate memory. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	/* Copy the Filename string. */
	strcpy(this->PrivFilename.uchar_t_ascii, Name->uchar_t_ascii);

	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::get_Type(TekFileType * FileTypePointer)
    
    Gets the file type.
 */
TEKERR TekFileBZ2ASCII::get_Type(TekFileType * FileTypePointer)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( FileTypePointer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*FileTypePointer = TekFileType_BZIP2_ASCII;

	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::Close()
    
    Closes the file and the related stream.
 */
TEKERR TekFileBZ2ASCII::Close()
{
	struct i64 bzIn, bzOut;
	int bzError;

	/* Determine if there is a BZ2 Descriptor open. */
	if ( this->PrivBZ2FileDesc != NULL )
	{
		/* How should we close the stream? */
		if ( this->PrivFlags & TekFileBZ2ASCII_Mode_WriteToFile )
		{
			/* Do a write close. */
			BZ2_bzWriteClose64(
				&bzError,
				this->PrivBZ2FileDesc,
				0,
				&bzIn.i64_a,
				&bzIn.i64_b,
				&bzOut.i64_a,
				&bzOut.i64_b
			);
			/* Convert our data into uint64_t's. */
			memcpy(&this->PrivBytesIn, &bzIn, 8);
			memcpy(&this->PrivBytesOut, &bzOut, 8);

			/* Handle the error, if necessary. */
			if ( bzError == BZ_IO_ERROR )
				return TEKERR_FILE_IO;
		} else {
			/* Do a read close. */
			BZ2_bzReadClose(
				&bzError,
				this->PrivBZ2FileDesc
			);
		}

		/* Handle any catastrophic errors. */
		if ( bzError != BZ_OK )
			return TEKERR_UNEXPECTED;

		/* NULLify the stream descriptor. */
		this->PrivBZ2FileDesc = NULL;

		if ( (this->PrivFlags & TekFileBZ2ASCII_Mode_EndOfStream) != 0 )
			this->PrivFlags ^= TekFileBZ2ASCII_Mode_EndOfStream;
	}

	/* Determine if there is a FILE descriptor open. */
	if ( this->PrivFileDesc != NULL )
	{
		/* Close the descriptor. */
		if ( fclose(this->PrivFileDesc) != 0 )
		{
			/* Handle the error. */
			if ( errno == ENOSPC )
				return TEKERR_FILE_DISKFULL;
			else
				return TEKERR_FILE_IO;
		}

		/* NULLify the FILE descriptor. */
		this->PrivFileDesc = NULL;
	}

	/* We're all done here. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::get_Mode(TekFileMode * ModeID)
    
    Retrieves the mode currently set.
 */
TEKERR TekFileBZ2ASCII::get_Mode(TekFileMode * ModeID)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( ModeID == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Determine if the WriteToFile bit is set. */
	if ( this->PrivFlags && TekFileBZ2ASCII_Mode_WriteToFile )
		*ModeID = TekFileMode_WriteOnly;
	else
		*ModeID = TekFileMode_ReadOnly;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::put_Mode(TekFileMode ModeID)
    
    Retrieves the mode currently set.\n
    \n
    <b>NOTE:</b>  This function will fail if a descriptor is open!
 */
TEKERR TekFileBZ2ASCII::put_Mode(TekFileMode ModeID)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Determine whether the flag needs to be set. */
	if ( ModeID != TekFileMode_ReadOnly )
	{
		this->PrivFlags |= TekFileBZ2ASCII_Mode_WriteToFile;
	}
	else
	{
		if ( this->PrivFlags && TekFileBZ2ASCII_Mode_WriteToFile )
			this->PrivFlags ^= TekFileBZ2ASCII_Mode_WriteToFile;
	}

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::Flush()
    
    This is not implemented.
 */
TEKERR TekFileBZ2ASCII::Flush()
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2ASCII::Open()
    
    Opens the file and a BZIP2 stream.
 */
TEKERR TekFileBZ2ASCII::Open()
{
	BZFILE * NewStream;
	FILE * NewDesc;
	int bzError;
	int UseSmallMemory;

	/* Make sure the internal buffer isn't NULL. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	/* How should we open the stream? */
	if ( this->PrivFlags && TekFileBZ2ASCII_Mode_WriteToFile )
	{
		/* Open the file for writing. */
		NewDesc = fopen(
			this->PrivFilename.uchar_t_ascii,
			"wb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		/* Open a stream. */
		NewStream = BZ2_bzWriteOpen(
			&bzError,
			NewDesc,
			this->PrivBlockSize,
			this->PrivVerbosity,
			this->PrivWorkFactor
		);
	} else {
		/* Open the file for reading. */
		NewDesc = fopen(
			this->PrivFilename.uchar_t_ascii,
			"rb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		if ( this->PrivFlags & TekFileBZ2ASCII_Mode_UseLessMemory )
			UseSmallMemory = 1;
		else
			UseSmallMemory = 0;

		/* Open a stream. */
		NewStream = BZ2_bzReadOpen(
			&bzError,
			NewDesc,
			this->PrivVerbosity,
			UseSmallMemory,
			NULL,
			0
		);
	}

	/* Handle errors. */
	if ( bzError != BZ_OK )
	{
		fclose(NewDesc);
		switch ( bzError )
		{
			case BZ_IO_ERROR:
				return TEKERR_FILE_IO;
			case BZ_MEM_ERROR:
				return TEKERR_MEMORY;
			case BZ_CONFIG_ERROR:
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	/* Set up our internal pointers. */
	this->PrivFileDesc = NewDesc;
	this->PrivBZ2FileDesc = NewStream;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::Open64()
    
    Opens the file and a BZIP2 stream using 64 bit semantics.
 */
TEKERR TekFileBZ2ASCII::Open64()
{
	BZFILE * NewStream;
	FILE * NewDesc;
	int bzError;
	int UseSmallMemory;

	/* Make sure the internal buffer isn't NULL. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	/* How should we open the stream? */
	if ( this->PrivFlags && TekFileBZ2ASCII_Mode_WriteToFile )
	{
		/* Open the file for writing. */
		NewDesc = fopen64(
			this->PrivFilename.uchar_t_ascii,
			"wb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		/* Open a stream. */
		NewStream = BZ2_bzWriteOpen(
			&bzError,
			NewDesc,
			this->PrivBlockSize,
			this->PrivVerbosity,
			this->PrivWorkFactor
		);
	} else {
		/* Open the file for reading. */
		NewDesc = fopen64(
			this->PrivFilename.uchar_t_ascii,
			"rb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		if ( this->PrivFlags & TekFileBZ2ASCII_Mode_UseLessMemory )
			UseSmallMemory = 1;
		else
			UseSmallMemory = 0;

		/* Open a stream. */
		NewStream = BZ2_bzReadOpen(
			&bzError,
			NewDesc,
			this->PrivVerbosity,
			UseSmallMemory,
			NULL,
			0
		);
	}

	/* Handle errors. */
	if ( bzError != BZ_OK )
	{
		fclose(NewDesc);
		switch ( bzError )
		{
			case BZ_IO_ERROR:
				return TEKERR_FILE_IO;
			case BZ_MEM_ERROR:
				return TEKERR_MEMORY;
			case BZ_CONFIG_ERROR:
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	/* Set up our internal pointers. */
	this->PrivFileDesc = NewDesc;
	this->PrivBZ2FileDesc = NewStream;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
    
    Read from the file using 64 bit semantics.
 */
TEKERR TekFileBZ2ASCII::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2ASCII::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
    
    Read from the file.
 */
TEKERR TekFileBZ2ASCII::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
{
	unsigned long BytesWeRead;
	int bzError;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Determine if we ended up with a buffer situation. */
	if ( this->PrivFlags & TekFileBZ2ASCII_Mode_EndOfStream )
	{
		/* Determine if we are at the end. */
		if ( BytesToRead >= (unsigned long)this->PrivUnusedLen )
		{
			memcpy(Buffer, this->PrivUnused, this->PrivUnusedLen);
			if ( BytesRead != NULL )
				*BytesRead = this->PrivUnusedLen;
			return TEKERR_FILE_EOF;
		} else {
			memcpy(Buffer, this->PrivUnused, BytesToRead);
			this->PrivUnused += BytesToRead;
			this->PrivUnusedLen -= BytesToRead;

			if ( BytesRead != NULL )
				*BytesRead = this->PrivUnusedLen;
		}
	} else {
		/* Read next area in the stream. */
		BytesWeRead = (unsigned long)BZ2_bzRead(
			&bzError,
			this->PrivBZ2FileDesc,
			Buffer,
			BytesToRead
		);
		if ( BytesRead != NULL )
			*BytesRead = BytesWeRead;

		/* Handle errors. */
		if ( bzError != BZ_OK )
		{
			switch ( bzError )
			{
			case BZ_STREAM_END:
				/* If we're here, we reached end of stream. */
				this->PrivFlags |= TekFileBZ2ASCII_Mode_EndOfStream;
				BZ2_bzReadGetUnused(
					&bzError,
					this->PrivBZ2FileDesc,
					(void**)&this->PrivUnused,
					&this->PrivUnusedLen
				);
				if ( bzError != BZ_OK )
					return TEKERR_UNEXPECTED;
				if ( this->PrivUnusedLen == 0 )
					return TEKERR_FILE_EOF;
				break;
			case BZ_MEM_ERROR:
				return TEKERR_MEMORY;
			case BZ_IO_ERROR:
				return TEKERR_FILE_IO;
			case BZ_DATA_ERROR:
			case BZ_DATA_ERROR_MAGIC:
			case BZ_UNEXPECTED_EOF:
			case BZ_SEQUENCE_ERROR:
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
			}
		}
	}

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
    
    This is not implemented.
 */
TEKERR TekFileBZ2ASCII::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2ASCII::Seek(TekFileSeek SeekWhere, signed long NewLocation)
    
    This is not implemented.
 */
TEKERR TekFileBZ2ASCII::Seek(TekFileSeek SeekWhere, signed long NewLocation)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2ASCII::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
    
    Write to the file using 64 bit semantics.
 */
TEKERR TekFileBZ2ASCII::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2ASCII::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
    
    Write to the file.
 */
TEKERR TekFileBZ2ASCII::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
{
	int bzError;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Write out the buffer. */
	BZ2_bzWrite(&bzError, this->PrivBZ2FileDesc, Buffer, BytesToWrite);
	/* Handle any errors. */
	if ( bzError != BZ_OK )
	{
		switch ( bzError )
		{
			case BZ_SEQUENCE_ERROR:
				return TEKERR_SEQUENCE;
			case BZ_IO_ERROR:
				if ( errno == ENOSPC )
					return TEKERR_FILE_DISKFULL;
				return TEKERR_FILE_IO;
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	/* Set *BytesWrote. */
	if ( BytesWrote != NULL )
		*BytesWrote = BytesToWrite;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2ASCII::get_Location(unsigned long * Location)

    Retrieves the location.\n
    \n
    <b>CAUTION!</b> This function may not be implemented due to
    optimization settings, so watch out for TEKERR_NOTIMPL
    return value.
 */

TEKERR TekFileBZ2ASCII::get_Location(unsigned long * Location)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2ASCII::get_Location64(uint64_t * Location)

    Retrieves the location and stores it in a 64-bit variable.\n
    \n
    <b>CAUTION!</b> This function may not be implemented due to
    optimization settings, so watch out for TEKERR_NOTIMPL
    return value.
 */

TEKERR TekFileBZ2ASCII::get_Location64(uint64_t * Location)
{
	return TEKERR_NOTIMPL;
}


