/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#define	LIBTEKLTI_EXPORT	(0)

#ifndef	_cplusplus
#define	_cplusplus
#endif	/* _cplusplus */
#include "teklti.h"


#include <errno.h>


#ifndef	TEKLTI_USE_BZIP2
#error You can not compile the BZIP2 interfaces unless TEKLTI_USE_BZIP2 is defined.
#endif	/* TEKLTI_USE_BZIP2 */


/*
  Our private flags.
*/

enum TekFileBZ2Binary_Mode
{
	TekFileBZ2Binary_Mode_WriteToFile = 0x0001,
	TekFileBZ2Binary_Mode_EndOfStream = 0x0002,
	TekFileBZ2Binary_Mode_UseLessMemory = 0x0004
};


MKTEKUUID(UTekFileBZ2Binary, "6E9A6C06-2D12-11DA-99DC-000BDBC434D9")
MKTEKDEBUGSTRING(DBTekFileBZ2Binary, "TekFileBZ2Binary")


/**
Instantizer.
*/
TekFileBZ2Binary::TekFileBZ2Binary()
{
	TEKSTD_IMPLEMENT_INTERFACE(UTekFile)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFileBZ2)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFileBZ2Binary)
	TEKSTD_IMPLEMENT_ENDBASE

	/* Set everything up. */
	this->PrivFileDesc = NULL;
	this->PrivBZ2FileDesc = NULL;
	this->PrivFlags = 0;
	this->PrivBlockSize = TEKLTI_USE_BZIP2_BLOCKSIZE;
	this->PrivBytesIn = this->PrivBytesOut = 0;
	this->PrivWorkFactor = 30;
	this->PrivVerbosity = 0;
	this->PrivFilename.uchar_t_ascii =
	  (char *)calloc(PATH_MAX,sizeof(char));
	this->PrivFilename.uchar_t_asciilen = 0;
}


/**
Destructor.
*/
TekFileBZ2Binary::~TekFileBZ2Binary()
{
	if ( this->PrivFilename.uchar_t_ascii != NULL )
		free(this->PrivFilename.uchar_t_ascii);

	this->Close();
}


//TEKSTD_SOURCE_FUNC(TekFileBZ2Binary)


/**
    \fn TekFileBZ2Binary::get_BytesIn(uint64_t * RetVal)
    
    Retrieves the amount of bytes bzip2 recieved. \n
    \n
    <b>NOTE:</b> You will get a TEKERR_UNAVAILABLE if you try calling
    this function while the BytesIn data is unavailable.
 */

TEKERR TekFileBZ2Binary::get_BytesIn(uint64_t * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = this->PrivBytesIn;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_BytesOut(uint64_t * RetVal)
    
    Retrieves the amount of bytes bzip2 put out. \n
    \n
    <b>NOTE:</b> You will get a TEKERR_UNAVAILABLE if you try calling
    this function while the BytesIn data is unavailable.
 */
 

TEKERR TekFileBZ2Binary::get_BytesOut(uint64_t * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = this->PrivBytesOut;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::put_UseLessMemory(unsigned int NewVal)
    
    Sets or resets the UseLessMemory flag. \n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened.
 */

TEKERR TekFileBZ2Binary::put_UseLessMemory(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	if ( NewVal == 0 )
	{
		/* Reset the UseLessMemory flag. */
		if ( this->PrivFlags & TekFileBZ2Binary_Mode_UseLessMemory )
			this->PrivFlags ^= TekFileBZ2Binary_Mode_UseLessMemory;
	} else {
		/* Set the UseLessMemory flag. */
		this->PrivFlags |= TekFileBZ2Binary_Mode_UseLessMemory;
	}

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_UseLessMemory(unsigned int * RetVal)
    
    Retrieves the UseLessMemory flag. \n
 */

TEKERR TekFileBZ2Binary::get_UseLessMemory(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	if ( this->PrivFlags & TekFileBZ2Binary_Mode_UseLessMemory )
		*RetVal = 1;
	else
		*RetVal = 0;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::put_WorkFactor(unsigned int NewVal)
    
    Sets the work factor number.\n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened. \n
    \n
    <b>WARNING:</b> It is best that you not set this yourself.
    The bzip2 library handles this on its own.
 */

TEKERR TekFileBZ2Binary::put_WorkFactor(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	this->PrivWorkFactor = NewVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_WorkFactor(unsigned int * RetVal)
    
    Retrieves the work factor number.\n
 */

TEKERR TekFileBZ2Binary::get_WorkFactor(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = (unsigned int)this->PrivWorkFactor;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::put_Verbosity(unsigned int NewVal)
    
    Sets the current verbosity value.\n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened.
 */

TEKERR TekFileBZ2Binary::put_Verbosity(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	this->PrivVerbosity = (int)NewVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_Verbosity(unsigned int * RetVal)
    
    Retrieves the current verbosity value.
 */

TEKERR TekFileBZ2Binary::get_Verbosity(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = (unsigned int)this->PrivVerbosity;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::put_BlockSize(unsigned int NewVal)
    
    Sets the block size to use for writing.  Allowed
    values are 1-9.\n
    \n
    <b>NOTE:</b> You will get a TEKERR_ACCESS if you try calling
    this function while the internally held BZFILE structure is
    opened.
 */
TEKERR TekFileBZ2Binary::put_BlockSize(unsigned int NewVal)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	if ( ( NewVal < 1 ) || ( NewVal > 9 ) )
		return TEKERR_FAIL;

	this->PrivBlockSize = NewVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_BlockSize(unsigned int * RetVal)
    
    Retrieves the block size to use for writing.
 */
TEKERR TekFileBZ2Binary::get_BlockSize(unsigned int * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*RetVal = this->PrivBlockSize;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::CloseStream()
    
    Tells the BZIP2 library to close the stream currently
    open for the internally held FILE *\n
    \n
    <b>NOTE:</b> TekFileBZ2Binary::Close() should be used instead.
    TekFileBZ2Binary::CloseStream() will not close the internally
    held FILE *
 */
TEKERR TekFileBZ2Binary::CloseStream()
{
	struct i64 bzIn, bzOut;
	int bzError;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	if ( this->PrivFlags & TekFileBZ2Binary_Mode_EndOfStream )
		this->PrivFlags ^= TekFileBZ2Binary_Mode_EndOfStream;

	/* Determine if there is a BZ2 Descriptor open. */
	if ( this->PrivBZ2FileDesc != NULL )
	{
		/* How should we close the stream? */
		if ( this->PrivFlags & TekFileBZ2Binary_Mode_WriteToFile )
		{
			/* Do a write close. */
			BZ2_bzWriteClose64(
				&bzError,
				this->PrivBZ2FileDesc,
				0,
				&bzIn.i64_a,
				&bzIn.i64_b,
				&bzOut.i64_a,
				&bzOut.i64_b
			);
			/* Convert our data into uint64_t's. */
			memcpy(&this->PrivBytesIn, &bzIn, 8);
			memcpy(&this->PrivBytesOut, &bzOut, 8);

			/* Handle the error, if necessary. */
			if ( bzError == BZ_IO_ERROR )
				return TEKERR_FILE_IO;
		} else {
			/* Do a read close. */
			BZ2_bzReadClose(
				&bzError,
				this->PrivBZ2FileDesc
			);
		}

		/* Handle any catastrophic errors. */
		if ( bzError != BZ_OK )
			return TEKERR_UNEXPECTED;

		/* NULLify the stream descriptor. */
		this->PrivBZ2FileDesc = NULL;
	}

	/* Return to caller. */
	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::OpenStream()
    
    Tells the BZIP2 library to open a stream for the internally
    held FILE *
 */
TEKERR TekFileBZ2Binary::OpenStream()
{
	BZFILE * NewStream;
	int bzError;
	int UseSmallMemory;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* How should we open the stream? */
	if ( this->PrivFlags & TekFileBZ2Binary_Mode_WriteToFile )
	{
		//* Open a stream. */
		NewStream = BZ2_bzWriteOpen(
			&bzError,
			this->PrivFileDesc,
			this->PrivBlockSize,
			this->PrivVerbosity,
			this->PrivWorkFactor
		);
	} else {
		/* Determine if SmallMemory is to be used. */
		if ( this->PrivFlags & TekFileBZ2Binary_Mode_UseLessMemory )
			UseSmallMemory = 1;
		else
			UseSmallMemory = 0;

		/* Open a stream. */
		NewStream = BZ2_bzReadOpen(
			&bzError,
			this->PrivFileDesc,
			this->PrivVerbosity,
			UseSmallMemory,
			NULL,
			0
		);
	}

	if ( NewStream == NULL )
		return TEKERR_FAIL;

	this->PrivBZ2FileDesc = NewStream;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_FILE(FILE ** FileDescriptor)
    
    Obtains the handle used to access the open file.
 */
TEKERR TekFileBZ2Binary::get_FILE(FILE ** FileDescriptor)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

#ifndef	NO_SIZE_T_CHECKS
	if ( FileDescriptor == (FILE **)NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	*FileDescriptor = this->PrivFileDesc;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::get_Filename(uchar_t ** Name)
    
    Obtains the filename in the form of a uchar.
 */
TEKERR TekFileBZ2Binary::get_Filename(uchar_t ** Name)
{
	uchar_t * RetVal;

#ifndef	NO_SIZE_T_CHECKS
	if ( Name == (uchar_t **)NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	/* Check to see if the constructor was able to allocate memory. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	if ( this->PrivFilename.uchar_t_ascii[0] == '\0' )
		return TEKERR_ACCESS;

	RetVal = char2uchar(this->PrivFilename.uchar_t_ascii);
	if ( RetVal == NULL )
		return TEKERR_MEMORY;

	*Name = RetVal;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::put_FILE(FILE * FileDescriptor)
    
    Sets the handle used to access a file.\n
    \n
    WARNING!!!!!  DO NOT USE THE FILE DESCRIPTOR YOU PASS IN \n
    TO THIS FUNCTION UNLESS YOU HAVE RE-RETRIEVED IT USING \n
    TekFileBZ2Binary::get_FILE()!  Utilizing the descriptor in a direct fashion after a \n
    successful call to put_FILE() will result in undefined results!!!!!
 */
TEKERR TekFileBZ2Binary::put_FILE(FILE * FileDescriptor)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivBZ2FileDesc != NULL )
		return TEKERR_ACCESS;
	if ( ( this->PrivFileDesc != NULL ) && ( FileDescriptor != NULL ) )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	if ( ( FileDescriptor == NULL ) && ( this->PrivFileDesc != NULL ) )
	{
		if ( this->PrivBZ2FileDesc != NULL )
			this->CloseStream();
		this->Close();
	}

	this->PrivFileDesc = FileDescriptor;

	return TEKERR_OK;
}


/**
    \fn TekFileBZ2Binary::put_Filename(uchar_t * Name)
    
    Sets the filename to access.
 */
TEKERR TekFileBZ2Binary::put_Filename(uchar_t * Name)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

#ifndef	NO_SIZE_T_CHECKS
	if ( Name == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	/* Determine the length. */
	this->PrivFilename.uchar_t_asciilen =
	  strlen(Name->uchar_t_ascii);

	/* Check to see if the constructor was able to allocate memory. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	/* Copy the Filename string. */
	strcpy(this->PrivFilename.uchar_t_ascii, Name->uchar_t_ascii);

	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::get_Type(TekFileType * FileTypePointer)
    
    Gets the file type.
 */
TEKERR TekFileBZ2Binary::get_Type(TekFileType * FileTypePointer)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( FileTypePointer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	*FileTypePointer = TekFileType_BZIP2;

	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::Close()
    
    Closes the file.
 */
TEKERR TekFileBZ2Binary::Close()
{
	struct i64 bzIn, bzOut;
	int bzError;

	/* Determine if there is a BZ2 Descriptor open. */
	if ( this->PrivBZ2FileDesc != NULL )
	{
		/* How should we close the stream? */
		if ( this->PrivFlags & TekFileBZ2Binary_Mode_WriteToFile )
		{
			/* Do a write close. */
			BZ2_bzWriteClose64(
				&bzError,
				this->PrivBZ2FileDesc,
				0,
				&bzIn.i64_a,
				&bzIn.i64_b,
				&bzOut.i64_a,
				&bzOut.i64_b
			);
			/* Convert our data into uint64_t's. */
			memcpy(&this->PrivBytesIn, &bzIn, 8);
			memcpy(&this->PrivBytesOut, &bzOut, 8);

			/* Handle the error, if necessary. */
			if ( bzError == BZ_IO_ERROR )
				return TEKERR_FILE_IO;
		} else {
			/* Do a read close. */
			BZ2_bzReadClose(
				&bzError,
				this->PrivBZ2FileDesc
			);
		}

		/* Handle any catastrophic errors. */
		if ( bzError != BZ_OK )
			return TEKERR_UNEXPECTED;

		/* NULLify the stream descriptor. */
		this->PrivBZ2FileDesc = NULL;

		if ( (this->PrivFlags & TekFileBZ2Binary_Mode_EndOfStream) != 0 )
			this->PrivFlags ^= TekFileBZ2Binary_Mode_EndOfStream;
	}

	/* Determine if there is a FILE descriptor open. */
	if ( this->PrivFileDesc != NULL )
	{
		/* Close the descriptor. */
		if ( fclose(this->PrivFileDesc) != 0 )
		{
			/* Handle the error. */
			if ( errno == ENOSPC )
				return TEKERR_FILE_DISKFULL;
			else
				return TEKERR_FILE_IO;
		}

		/* NULLify the FILE descriptor. */
		this->PrivFileDesc = NULL;
	}

	/* We're all done here. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::get_Mode(TekFileMode * ModeID)
    
    Retrieves the mode currently set.
 */
TEKERR TekFileBZ2Binary::get_Mode(TekFileMode * ModeID)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( ModeID == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Determine if the WriteToFile bit is set. */
	if ( this->PrivFlags && TekFileBZ2Binary_Mode_WriteToFile )
		*ModeID = TekFileMode_WriteOnly;
	else
		*ModeID = TekFileMode_ReadOnly;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::put_Mode(TekFileMode ModeID)
    
    Retrieves the mode currently set.\n
    \n
    <b>NOTE:</b>  This function will fail if a descriptor is open!
 */
TEKERR TekFileBZ2Binary::put_Mode(TekFileMode ModeID)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Determine whether the flag needs to be set. */
	if ( ModeID != TekFileMode_ReadOnly )
	{
		this->PrivFlags |= TekFileBZ2Binary_Mode_WriteToFile;
	}
	else
	{
		if ( this->PrivFlags && TekFileBZ2Binary_Mode_WriteToFile )
			this->PrivFlags ^= TekFileBZ2Binary_Mode_WriteToFile;
	}

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::Flush()
    
    This function is not implemented.
 */
TEKERR TekFileBZ2Binary::Flush()
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2Binary::Open()
    
    Opens the file and a BZIP2 stream.
 */
TEKERR TekFileBZ2Binary::Open()
{
	BZFILE * NewStream;
	FILE * NewDesc;
	int bzError;
	int UseSmallMemory;

	/* Make sure the internal buffer isn't NULL. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	/* How should we open the stream? */
	if ( this->PrivFlags && TekFileBZ2Binary_Mode_WriteToFile )
	{
		/* Open the file for writing. */
		NewDesc = fopen(
			this->PrivFilename.uchar_t_ascii,
			"wb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		/* Open a stream. */
		NewStream = BZ2_bzWriteOpen(
			&bzError,
			NewDesc,
			this->PrivBlockSize,
			this->PrivVerbosity,
			this->PrivWorkFactor
		);
	} else {
		/* Open the file for reading. */
		NewDesc = fopen(
			this->PrivFilename.uchar_t_ascii,
			"rb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		if ( this->PrivFlags & TekFileBZ2Binary_Mode_UseLessMemory )
			UseSmallMemory = 1;
		else
			UseSmallMemory = 0;

		/* Open a stream. */
		NewStream = BZ2_bzReadOpen(
			&bzError,
			NewDesc,
			this->PrivVerbosity,
			UseSmallMemory,
			NULL,
			0
		);
	}

	/* Handle errors. */
	if ( bzError != BZ_OK )
	{
		fclose(NewDesc);
		switch ( bzError )
		{
			case BZ_IO_ERROR:
				return TEKERR_FILE_IO;
			case BZ_MEM_ERROR:
				return TEKERR_MEMORY;
			case BZ_CONFIG_ERROR:
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	/* Set up our internal pointers. */
	this->PrivFileDesc = NewDesc;
	this->PrivBZ2FileDesc = NewStream;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::Open64()
    
    Opens the file and a BZIP2 stream using 64 bit semantics.
 */
TEKERR TekFileBZ2Binary::Open64()
{
	BZFILE * NewStream;
	FILE * NewDesc;
	int bzError;
	int UseSmallMemory;

	/* Make sure the internal buffer isn't NULL. */
	if ( this->PrivFilename.uchar_t_ascii == NULL )
		return TEKERR_MEMORY;

	/* How should we open the stream? */
	if ( this->PrivFlags && TekFileBZ2Binary_Mode_WriteToFile )
	{
		/* Open the file for writing. */
		NewDesc = fopen64(
			this->PrivFilename.uchar_t_ascii,
			"wb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		/* Open a stream. */
		NewStream = BZ2_bzWriteOpen(
			&bzError,
			NewDesc,
			this->PrivBlockSize,
			this->PrivVerbosity,
			this->PrivWorkFactor
		);
	} else {
		/* Open the file for reading. */
		NewDesc = fopen64(
			this->PrivFilename.uchar_t_ascii,
			"rb"
		);
		if ( NewDesc == NULL )
			return TEKERR_FILE_IO;

		if ( this->PrivFlags & TekFileBZ2Binary_Mode_UseLessMemory )
			UseSmallMemory = 1;
		else
			UseSmallMemory = 0;

		/* Open a stream. */
		NewStream = BZ2_bzReadOpen(
			&bzError,
			NewDesc,
			this->PrivVerbosity,
			UseSmallMemory,
			NULL,
			0
		);
	}

	/* Handle errors. */
	if ( bzError != BZ_OK )
	{
		fclose(NewDesc);
		switch ( bzError )
		{
			case BZ_IO_ERROR:
				return TEKERR_FILE_IO;
			case BZ_MEM_ERROR:
				return TEKERR_MEMORY;
			case BZ_CONFIG_ERROR:
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	/* Set up our internal pointers. */
	this->PrivFileDesc = NewDesc;
	this->PrivBZ2FileDesc = NewStream;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
    
    Read from the file using 64 bit semantics.
 */
TEKERR TekFileBZ2Binary::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2Binary::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
    
    Read from the file.
 */
TEKERR TekFileBZ2Binary::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
{
	unsigned long BytesWeRead;
	int bzError;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Determine if we ended up with a buffer situation. */
	if ( this->PrivFlags & TekFileBZ2Binary_Mode_EndOfStream )
	{
		/* Determine if we are at the end. */
		if ( BytesToRead >= (unsigned long)this->PrivUnusedLen )
		{
			memcpy(Buffer, this->PrivUnused, this->PrivUnusedLen);
			if ( BytesRead != NULL )
				*BytesRead = this->PrivUnusedLen;
			return TEKERR_FILE_EOF;
		} else {
			memcpy(Buffer, this->PrivUnused, BytesToRead);
			this->PrivUnused += BytesToRead;
			this->PrivUnusedLen -= BytesToRead;

			if ( BytesRead != NULL )
				*BytesRead = this->PrivUnusedLen;
		}
	} else {
		/* Read next area in the stream. */
		BytesWeRead = (unsigned long)BZ2_bzRead(
			&bzError,
			this->PrivBZ2FileDesc,
			Buffer,
			BytesToRead
		);
		if ( BytesRead != NULL )
			*BytesRead = BytesWeRead;

		/* Handle errors. */
		if ( bzError != BZ_OK )
		{
			switch ( bzError )
			{
			case BZ_STREAM_END:
				/* If we're here, we reached end of stream. */
				this->PrivFlags |= TekFileBZ2Binary_Mode_EndOfStream;
				BZ2_bzReadGetUnused(
					&bzError,
					this->PrivBZ2FileDesc,
					(void**)&this->PrivUnused,
					&this->PrivUnusedLen
				);
				if ( bzError != BZ_OK )
					return TEKERR_UNEXPECTED;
				if ( this->PrivUnusedLen == 0 )
					return TEKERR_FILE_EOF;
				break;
			case BZ_MEM_ERROR:
				return TEKERR_MEMORY;
			case BZ_IO_ERROR:
				return TEKERR_FILE_IO;
			case BZ_DATA_ERROR:
			case BZ_DATA_ERROR_MAGIC:
			case BZ_UNEXPECTED_EOF:
			case BZ_SEQUENCE_ERROR:
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
			}
		}
	}

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
    
    This is not implemented.
 */
TEKERR TekFileBZ2Binary::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2Binary::Seek(TekFileSeek SeekWhere, signed long NewLocation)
    
    This is not implemented.
 */
TEKERR TekFileBZ2Binary::Seek(TekFileSeek SeekWhere, signed long NewLocation)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2Binary::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
    
    Write to the file using 64 bit semantics.
 */
TEKERR TekFileBZ2Binary::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2Binary::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
    
    Write to the file.
 */
TEKERR TekFileBZ2Binary::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
{
	BZFILE * bzdesc;
	int bzError;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	bzdesc = this->PrivBZ2FileDesc;
	/* Write out the buffer. */
	BZ2_bzWrite(&bzError, bzdesc, Buffer, BytesToWrite);
	/* Handle any errors. */
	if ( bzError != BZ_OK )
	{
		switch ( bzError )
		{
			case BZ_SEQUENCE_ERROR:
				return TEKERR_SEQUENCE;
			case BZ_IO_ERROR:
				if ( errno == ENOSPC )
					return TEKERR_FILE_DISKFULL;
				return TEKERR_FILE_IO;
			case BZ_PARAM_ERROR:
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	/* Set *BytesWrote. */
	if ( BytesWrote != NULL )
		*BytesWrote = BytesToWrite;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBZ2Binary::get_Location(unsigned long * Location)

    Retrieves the location.\n
    \n
    <b>CAUTION!</b> This function may not be implemented due to
    optimization settings, so watch out for TEKERR_NOTIMPL
    return value.
 */

TEKERR TekFileBZ2Binary::get_Location(unsigned long * Location)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBZ2Binary::get_Location64(uint64_t * Location)

    Retrieves the location and stores it in a 64-bit variable.\n
    \n
    <b>CAUTION!</b> This function may not be implemented due to
    optimization settings, so watch out for TEKERR_NOTIMPL
    return value.
 */

TEKERR TekFileBZ2Binary::get_Location64(uint64_t * Location)
{
	return TEKERR_NOTIMPL;
}


