/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)

#ifndef	_cplusplus
#define	_cplusplus
#endif	/* _cplusplus */
#include "teklti.h"

#ifdef	USE_386_ASM

#include <limits.h>
#include <errno.h>


#undef	TEKLTI_NULLALL


//#ifdef	TEKLTI_OBSERVE_BINARY_FILE_MODE
const char TekFileBinary_ReadOnly[] = "rb";
const char TekFileBinary_WriteOnly[] = "wb";
const char TekFileBinary_Append[] = "ab";
const char TekFileBinary_ReadWrite[] = "r+b";
const char TekFileBinary_ReadWriteTrunc[] = "w+b";
const char TekFileBinary_ReadAppend[] = "a+b";
//#else
	/* not TEKLTI_OBSERVE_BINARY_FILE_MODE */
/*const char TekFileBinary_ReadOnly[] = "r";
const char TekFileBinary_WriteOnly[] = "w";
const char TekFileBinary_Append[] = "a";
const char TekFileBinary_ReadWrite[] = "r+";
const char TekFileBinary_ReadWriteTrunc[] = "w+";
const char TekFileBinary_ReadAppend[] = "a+";
#endif	*/ /* not TEKLTI_OBSERVE_BINARY_FILE_MODE */


static const void * _tfbtmp = NULL;

/**
Instantizer.
*/
TekFileBinary::TekFileBinary()
{
	TEKSTD_IMPLEMENT_INTERFACE(UTekFile)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFileBinary)
	TEKSTD_IMPLEMENT_ENDBASE

#ifdef	USE_386_ASM_MMX
#ifdef	USE_386_ASM_MMX_RESET_MM7
	/* Copy the nullified quadword to mm0. */
	__asm__ ( " movq %0, %%mm7 " : "=m" (tekzerovar) );
#endif	/* USE_386_ASM_MMX_RESET_MM7 */
	/* Obtain our private section of local variables. */
	__asm__ ( " movl 8(%ebp), %ecx " );

#ifdef	UNICODE
	/* Copy the null quadword to the ascii variables. */
	__asm__ ( " movq %mm7, 76(%ecx) " );
	/* Copy the null quadword to the unicode variables. */
	__asm__ ( " movq %mm7, 84(%ecx) " );
	/* then into PrivFileDesc/PrivFileMode */
	__asm__ ( " movq %mm7, 92(%ecx) " );
	/* and finally into PrivTekFileType. */
	__asm__ ( " movd %mm7, 100(%ecx) " );
#else	/* not UNICODE */
	/* Copy the null quadword to the ascii variables. */
	__asm__ ( " movq %mm7, 76(%ecx) " );
	/* then into PrivFileDesc/PrivFileMode */
	__asm__ ( " movq %mm7, 84(%ecx) " );
	/* and finally into PrivTekFileType. */
	__asm__ ( " movd %mm7, 92(%ecx) " );
#endif	/* not UNICODE */

	/* Be sure to exit the mmx state before leaving. */
	__asm__ ( " emms " );

#else	/* not USE_386_ASM_MMX */
	/* If we're here, we are not using mmx. */
#ifdef	USE_386_ASM_PUSHPOP
	/* Save the current value of edi if necessary. */
	__asm__ ( " pushl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */

	/* Obtain our private section of local variables. */
	__asm__ ( " movl 8(%ebp), %edi " );

	/* Set ax = 0. */
	__asm__ ( " xorl %eax, %eax " );

	/* Add 76 to edi for alignment with our private members. */
	__asm__ ( " addl $76, %edi " );

	/* Set up ecx as appropriate. */
#ifdef	UNICODE
	__asm__ ( " movl $7, %ecx " );
#else	/* not UNICODE */
	__asm__ ( " movl $5, %ecx " );
#endif	/* not UNICODE */

	/* Call the stosl function until the area in ecx is nullified. */
	__asm__ ( " rep stosl " );
#ifdef	USE_386_ASM_PUSHPOP
	/* Be sure to pop edi off the stack if we saved it earlier. */
	__asm__ ( " popl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */
#endif	/* not USE_386_ASM_MMX */
}


/**
Destructor.

FIXME: Convert this function to assembly.
*/
TekFileBinary::~TekFileBinary()
{
	if ( this->PrivFileDesc != NULL )
		fclose(this->PrivFileDesc);
	if ( this->PrivFileName.uchar_t_ascii != NULL )
		free(this->PrivFileName.uchar_t_ascii);
}


/**
    \fn TekFileBinary::ConvertToPipe()
    
    Changes the file type variable to BinaryPipe.\n
    \n
    NOTE: This function should only be called when the contained FILE * is actually a pipe.  This is a one-time change of the file type on any instance of BinaryPipe   called.  Therefore, it is best that this function be called only when passing an instance of TekFileBinary to a library (or to the caller when created in libraries) where you are not sure how the library or caller handles a pipe differently from a regular file.
 */
TEKERR TekFileBinary::ConvertToPipe()
{
	/* Obtain our private section of local variables. */
	__asm__ ( " movl 8(%ebp), %ecx " );
	/* Set this->PrivTekFileType to TekFileType_BinaryPipe. */
	__asm__ ( " movl %1, 92(%%ecx) " :
	 "=m" (_tfbtmp) : 
	 "i" (TekFileType_BinaryPipe) 
	);

	/* Set return value to zero. */
	TEKLTI_RETVAL_TEKERR_OK
}

/**
    \fn TekFileBinary::get_FILE(FILE ** FileDescriptor)
    
    Obtains the handle used to access the open file.
 */
TEKERR TekFileBinary::get_FILE(FILE ** FileDescriptor)
{
#ifndef	NO_SIZE_T_CHECKS
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Get the value of eax. */
	__asm__ ( " movl %0, %%esi " : "=m" (FileDescriptor) );
	/* Test to see if it is null. */
	__asm__ ( " testl %esi, %esi " );
	/* Set the worst-case scenario for return. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_POINTER) );
	/* If NULL, go to PointerFail. */
	__asm__ ( " je .TekFileBinary_get_FILE_PointerFail " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	NO_SIZE_T_CHECKS
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Get the file pointer if we haven't yet retrieved it. */
	__asm__ ( " movl %0, %%esi " : "=m" (FileDescriptor) );
#endif	/* NO_SIZE_T_CHECKS */

	/* Put the PrivTekFileType into the ecx register. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );
	/* And then into *FileTypePointer. */
	__asm__ ( " movl %ecx, (%esi) " );
	/* Set return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK
#ifndef	NO_SIZE_T_CHECKS
	__asm__ ( " .TekFileBinary_get_FILE_PointerFail: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
}


/**
    \fn TekFileBinary::get_Filename(uchar_t ** Name)
    
    Obtains the filename in the form of a uchar.
 */
TEKERR TekFileBinary::get_Filename(uchar_t ** Name)
{
#ifndef	NO_SIZE_T_CHECKS
	/* Put FileDescriptor into the eax register. */
	__asm__ ( " movl %0, %%eax " : "=m" (Name) );
	/* Test to see if Name == NULL */
	__asm__ ( " testl %eax, %eax " );
	/* If Name == NULL, there's nothing to do. */
	__asm__ ( " jne .TekFileBinary_get_Filename_getstr " );

	/* Set retval to TEKERR_POINTER. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_POINTER) );
	/* Jump to fatal return. */
	__asm__ ( " jmp .TekFileBinary_get_Filename_fatal " );

	/* Label for continuing on. */
	__asm__ ( " .TekFileBinary_get_Filename_getstr: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	__asm__ ( " movl %0, %%esi " : "=m" (this->PrivFileName.uchar_t_ascii) );

#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Make sure it isn't NULL. */
	__asm__ ( " testl %esi, %esi " );
	/* If it isn't, start scanning. */
	__asm__ ( " jne .TekFileBinary_get_Filename_scas " );

	/* Set retval to TEKERR_SEQUENCE. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Jump to fatal return. */
	__asm__ ( " jmp .TekFileBinary_get_Filename_fatal2 " );

	/* If uchar_t_ascii was not NULL, we should be here. */
	__asm__ ( " .TekFileBinary_get_Filename_scas: " );
#endif	/* not NO_INTERNAL_NULL_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Copy it to edi. */
	__asm__ ( " movl %esi, %edi " );

	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (PATH_MAX) );
	/* Put null character into al. */
	__asm__ ( " xorb %al, %al " );
	/* Set direction flag to forwards. */
	__asm__ ( " cld " );
	/* And look for the first null character. */
	__asm__ ( " repne scasb " );

	/* Negate ecx. */
	__asm__ ( " negl %ecx " );
	/* Add PATH_MAX to get the final result. */
	__asm__ ( " addl %1, %%ecx " : "=m" (_tfbtmp) : "i" (PATH_MAX) );

	/* Make sure esi is saved. */
	__asm__ ( " pushl %esi " );
#ifdef	USE_386_ASM_MALLOC_FIX1
	/* We are directed by macros to put ecx in eax prior to saving eax, */
	/* probably to work around a malloc() bug. */
	__asm__ ( " movl %ecx, %eax " );
#endif	/* USE_386_ASM_MALLOC_FIX1 */
	/* Push ecx onto the stack. */
	__asm__ ( " pushl %ecx " );
#ifdef	USE_386_ASM_MALLOC_FIX1
	/* To complete the pre-call workaround, set up a blank parameter. */
	__asm__ ( " subl $4, %esp " );
	/* Then copy eax into the parameter. */
	__asm__ ( " movl %eax, (%esp) " );
#endif	/* USE_386_ASM_MALLOC_FIX1 */
	/* Call malloc(). */
	__asm__ ( " call malloc " );
#ifdef	USE_386_ASM_MALLOC_FIX1
	/* Restore the stack if the workaround is compiled by macros. */
	__asm__ ( " addl $4, %esp " );
#endif	/* USE_386_ASM_MALLOC_FIX1 */

	/* Is malloc() == NULL? */
	__asm__ ( " testl %eax, %eax " );
	/* If not, continue on. */
	__asm__ ( " jne .TekFileBinary_get_Filename_mkucharstruct " );

	/* malloc() returned NULL if we're here instead. */
	/* Set the error code. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_MEMORY) );
	/* Pop esi and ecx. */
	__asm__ ( " popl %ecx " );
	__asm__ ( " popl %esi " );
	/* Go to fatal return. */
	__asm__ ( " jmp .TekFileBinary_get_Filename_fatal3 " );

	/* Process string label. */
	__asm__ ( " .TekFileBinary_get_Filename_mkucharstruct: " );
	/* Put malloc's result into edi. */
	__asm__ ( " movl %eax, %edi " );

	/* Put new count into esp. */
	/* Call malloc, while making sure edi is saved. */
	__asm__ ( " pushl %edi " );
	/* Make space for our parameter, or edi will be overwritten. */
	__asm__ ( " subl $4, %esp " );
	/* Set up param 0 with intermediate value 8. */
	__asm__ ( " movl $8, (%esp) " );
	/* Call malloc. */
	__asm__ ( " call malloc " );
	__asm__ ( " addl $4, %esp " );
	__asm__ ( " popl %edi " );
	/* Restore the pointers we saved above. */
	__asm__ ( " popl %ecx " );
	__asm__ ( " popl %esi " );
	/* Is malloc() == NULL? */
	__asm__ ( " testl %eax, %eax " );
	/* If not, continue on. */
	__asm__ ( " jne .TekFileBinary_get_Filename_finish " );

	/* Set up esp for free(). */
	__asm__ ( " movl %edi, (%esp) " );
	/* Call free(). */
	__asm__ ( " call free " );
	/* Set the error code. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_MEMORY) );
	/* Go to fatal return. */
	__asm__ ( " jmp .TekFileBinary_get_Filename_fatal2 " );

	/* Jump that executes all final code. */
	__asm__ ( " .TekFileBinary_get_Filename_finish: " );
	/* Copy edi register to new Name->uchar_t_ascii. */
	__asm__ ( " movl %edi, (%eax) " );
	/* Copy value of ecx to new Name->uchar_t_asciilen. */
	__asm__ ( " movl %ecx, 4(%eax) " );

#ifdef	USE_386_ASM_MALLOC_FIX2
	/* malloc() calls std and leaves direction flag set, so... */
	/* Clear the direction flag again - malloc() may have reset it. */
	__asm__ ( " cld " );
#endif	/* USE_386_ASM_MALLOC_FIX2 */
	/* We can finally copy ecx characters from esi to edi! */
	__asm__ ( " rep movsb " );

	/* Last, but not least, we have to hand over the pointer. */
	/* Get the pointer (again). */
	__asm__ ( " movl %0, %%ecx " : "=m" (Name) );
	/* Copy eax to ecx. */
	__asm__ ( " movl %eax, (%ecx) " );

	/* Set return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for fatal returns. */
	__asm__ ( " .TekFileBinary_get_Filename_fatal3: " );
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " popl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */
	__asm__ ( " .TekFileBinary_get_Filename_fatal2: " );
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	__asm__ ( " .TekFileBinary_get_Filename_fatal: " );
}


/**
    \fn TekFileBinary::put_FILE(FILE * FileDescriptor)
    
    Sets the handle used to access a file.\n
    \n
    WARNING!!!!!  DO NOT USE THE FILE DESCRIPTOR YOU PASS IN TO THIS FUNCTION UNLESS YOU HAVE RE-RETRIEVED IT USING TekFile::get_FILE()!  Utilizing the descriptor in a direct fashion after a successful call to put_FILE() will result in undefined results!!!!!
 */
TEKERR TekFileBinary::put_FILE(FILE * FileDescriptor)
{
	/* Put FileDescriptor into the ecx register. */
	__asm__ ( " movl %0, %%ecx " : "=m" (FileDescriptor) );
	/* And then into the PrivTekFileType. */
	__asm__ ( " movl %%ecx, %0 " : "=m" (this->PrivFileDesc) );

	/* Set return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK
}


/**
    \fn TekFileBinary::put_Filename(uchar_t * Name)
    
    Sets the filename to access.
 */
TEKERR TekFileBinary::put_Filename(uchar_t * Name)
{
	/* Put FileDescriptor into the eax register. */
	__asm__ ( " movl %0, %%eax " : "=m" (Name) );
	/* Test to see if Name == NULL */
	__asm__ ( " testl %eax, %eax " );
	/* If Name == NULL, there's nothing to do. */
	__asm__ ( " je .TekFileBinary_put_Filename_donothing " );

	/* Put file name pointer into esi, pushing if defined to do so. */
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	__asm__ ( " movl (%eax), %esi " );

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Copy esi into edi; scasb uses edi, not esi. */
	__asm__ ( " movl %esi, %edi " );
	/* Put PATH_MAX into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (PATH_MAX) );
	/* Put null character into al. */
	__asm__ ( " xorb %al, %al " );
	/* Set direction flag to forwards. */
	__asm__ ( " cld " );
	/* And look for the first null character. */
	__asm__ ( " repne scasb " );
	/* If found, call malloc. */
	__asm__ ( " je .TekFileBinary_put_Filename_malloc " );

	/* If we get here, there was no null character within PATH_MAX bytes */
	/* Set return value. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Go to fatal error. */
	__asm__ ( " jmp .TekFileBinary_put_Filename_fatal " );

	__asm__ ( " .TekFileBinary_put_Filename_malloc: " );
	/* Negate ecx. */
	__asm__ ( " negl %ecx " );
	/* Add PATH_MAX to get the final result. */
	__asm__ ( " addl %1, %%ecx " : "=m" (_tfbtmp) : "i" (PATH_MAX) );

#ifdef	USE_386_ASM_MALLOC_FIX3
	/* Make sure esi is saved. */
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_MALLOC_FIX3 */

#ifdef	USE_386_ASM_MALLOC_FIX1
	/* We are directed by macros to put ecx in eax prior to saving eax, */
	/* probably to work around a malloc() bug. */
	__asm__ ( " movl %ecx, %eax " );
#endif	/* USE_386_ASM_MALLOC_FIX1 */
	/* Push ecx onto the stack. */
	__asm__ ( " pushl %ecx " );
#ifdef	USE_386_ASM_MALLOC_FIX1
	/* To complete the pre-call workaround, set up a blank parameter. */
	__asm__ ( " subl $4, %esp " );
	/* Then copy eax into the parameter. */
	__asm__ ( " movl %eax, (%esp) " );
#endif	/* USE_386_ASM_MALLOC_FIX1 */
	/* Call malloc(). */
	__asm__ ( " call malloc " );
#ifdef	USE_386_ASM_MALLOC_FIX1
	/* Restore the stack if the workaround is compiled by macros. */
	__asm__ ( " addl $4, %esp " );
#endif	/* USE_386_ASM_MALLOC_FIX1 */

	/* Restore esi and ecx. */
	__asm__ ( " popl %ecx " );
#ifdef	USE_386_ASM_MALLOC_FIX3
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_MALLOC_FIX3 */

	/* Is malloc() == NULL? */
	__asm__ ( " testl %eax, %eax " );
	/* If not, continue on. */
	__asm__ ( " jne .TekFileBinary_put_Filename_proc " );
	/* Set the error code. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_MEMORY) );
	/* Go to fatal return. */
	__asm__ ( " jmp .TekFileBinary_put_Filename_fatal " );

	/* Process string label. */
	__asm__ ( " .TekFileBinary_put_Filename_proc: " );

	/* Move dest string to edi. */
	__asm__ ( " movl %eax, %edi " );
#ifdef	USE_386_ASM_MALLOC_FIX2
	/* malloc() calls std and leaves direction flag set, so... */
	/* Clear the direction flag again - malloc() may have reset it. */
	__asm__ ( " cld " );
#endif	/* USE_386_ASM_MALLOC_FIX2 */
	/* And copy it. */
	__asm__ ( " rep movsb " );
	/* Get the pointer to the instance's variables. */
	__asm__ ( " movl 8(%ebp), %ecx " );
	/* Move result to PrivTekFileName. */
	__asm__ ( " movl %eax, 76(%ecx) " );

	/* Used for do-nothing returns. */
	__asm__ ( " .TekFileBinary_put_Filename_ret: " );
	/* Set the return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for error returns. */
	__asm__ ( " .TekFileBinary_put_Filename_fatal: " );

	/* Clean up - pointer, then stack. */
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " addl $8, %esp " );
#endif	/* USE_386_ASM_PUSHPOP */

	__asm__ ( " .TekFileBinary_put_Filename_donothing: " );
}


/*!
    \fn TekFileBinary::get_Type(TekFileType * FileTypePointer)
    
    Retrieves the filename to access.
 */
TEKERR TekFileBinary::get_Type(TekFileType * FileTypePointer)
{
#ifndef	NO_SIZE_T_CHECKS
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Get the value of eax. */
	__asm__ ( " movl %0, %%esi " : "=m" (FileTypePointer) );
	/* Test to see if it is null. */
	__asm__ ( " testl %esi, %esi " );
	/* Set the worst-case scenario for return. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_POINTER) );
	/* If NULL, go to PointerFail. */
	__asm__ ( " je .TekFileBinary_get_Type_PointerFail " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	NO_SIZE_T_CHECKS
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Get the file pointer if we haven't yet retrieved it. */
	__asm__ ( " movl %0, %%esi " : "=m" (FileTypePointer) );
#endif	/* NO_SIZE_T_CHECKS */

	/* Put the PrivTekFileType into the ecx register. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivTekFileType) );
	/* And then into *FileTypePointer. */
	__asm__ ( " movl %ecx, (%esi) " );
	/* Set return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK
#ifndef	NO_SIZE_T_CHECKS
	__asm__ ( " .TekFileBinary_get_Type_PointerFail: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
}


/*!
    \fn TekFileBinary::Close()
    
    Closes the file.
 */
TEKERR TekFileBinary::Close()
{
	/* First, let's get the value of PrivFileDesc. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );
#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Test to see if it is not zero. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If it isn't, close the descriptor. */
	__asm__ ( " jne .TekFileBinary_Close_close " );
	/* Otherwise, set retval to TEKERR_SEQUENCE. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Goto fatal error. */
	__asm__ ( " jmp .TekFileBinary_Close_fatal " );

	/* Label for closing the descriptor. */
	__asm__ ( " .TekFileBinary_Close_close: " );
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Prepare to close the descriptor. */
	__asm__ ( " pushl %ecx " );
	/* Call fclose(). */
	__asm__ ( " call fclose " );
	/* Throw out the pointer. */
	__asm__ ( " addl $4, %esp " );

	/* Check to see if eax is zero. */
	__asm__ ( " testl %eax, %eax " );
	/* If it is, fclose() was successful. */
	__asm__ ( " je .TekFileBinary_Close_done " );

	/* fclose() was not successful if we're here. */
	/* So, get value of errno. */
	__asm__ ( " movl %0, %%ecx " : "=m" (errno) );
	/* Perform a comparison for ENOSPC (disk full error) */
	__asm__ ( " cmpl %1, %%ecx " : "=m" (_tfbtmp) : "i" (ENOSPC) );
	/* Jump if ENOSPC is the error. */
	__asm__ ( " je .TekFileBinary_Close_DiskFull " );
	/* Perform a comparison for EIO (device input/output error) */
	__asm__ ( " cmpl %1, %%ecx " : "=m" (_tfbtmp) : "i" (EIO) );
	/* Jump if EIO is the error. */
	__asm__ ( " je .TekFileBinary_Close_IO " );
	/* Perform a comparison for EPIPE (pipe broken error) */
	__asm__ ( " cmpl %1, %%ecx " : "=m" (_tfbtmp) : "i" (EPIPE) );
	/* Jump if EIO is the error. */
	__asm__ ( " je .TekFileBinary_Close_Pipe " );
	/* All other errors will be returned as TEKERR_UNEXPECTED. */

	/* For setting TEKERR_UNEXPECTED */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_UNEXPECTED) );
	__asm__ ( " jmp .TekFileBinary_Close_fatal " );
	/* For setting TEKERR_FILE_DISKFULL */
	__asm__ ( " .TekFileBinary_Close_DiskFull: " );
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_DISKFULL) );
	__asm__ ( " jmp .TekFileBinary_Close_fatal " );
	/* For setting TEKERR_FILE_IO */
	__asm__ ( " .TekFileBinary_Close_IO: " );
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_IO) );
	__asm__ ( " jmp .TekFileBinary_Close_fatal " );
	/* For setting TEKERR_FILE_PIPE */
	__asm__ ( " .TekFileBinary_Close_Pipe: " );
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_PIPE) );
	__asm__ ( " jmp .TekFileBinary_Close_fatal " );

	/* We should now be here if fclose() was successful. */
	__asm__ ( " .TekFileBinary_Close_done: " );
	/* NULLify the descriptor; it's no longer valid. */
	__asm__ ( " movl $0, %0 " : "=m" (this->PrivFileDesc) );
	/* Set return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for fatal returns. */
	__asm__ ( " .TekFileBinary_Close_fatal: " );
}


/*!
    \fn TekFileBinary::get_Mode(TekFileMode * ModeID)
    
    Retrieves the mode currently set.
 */
TEKERR TekFileBinary::get_Mode(TekFileMode * ModeID)
{
#ifndef	NO_SIZE_T_CHECKS
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Get the value of eax. */
	__asm__ ( " movl %0, %%esi " : "=m" (ModeID) );
	/* Test to see if it is null. */
	__asm__ ( " testl %esi, %esi " );
	/* Set the worst-case scenario for return. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_POINTER) );
	/* If NULL, go to PointerFail. */
	__asm__ ( " je .TekFileBinary_get_Mode_PointerFail " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	NO_SIZE_T_CHECKS
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Get the file pointer if we haven't yet retrieved it. */
	__asm__ ( " movl %0, %%esi " : "=m" (ModeID) );
#endif	/* NO_SIZE_T_CHECKS */

	/* Put the PrivTekFileType into the ecx register. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileMode) );
	/* And then into *FileTypePointer. */
	__asm__ ( " movl %ecx, (%esi) " );
	/* Set return value to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK
#ifndef	NO_SIZE_T_CHECKS
	__asm__ ( " .TekFileBinary_get_Mode_PointerFail: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
}


/*!
    \fn TekFileBinary::Flush()
    
    Flushes the buffer.
 */
TEKERR TekFileBinary::Flush()
{
	/* First, let's get the value of PrivFileDesc. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );
#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Test to see if it is not zero. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If it isn't, flush the descriptor. */
	__asm__ ( " jne .TekFileBinary_Flush_flush " );
	/* Otherwise, set retval to TEKERR_SEQUENCE. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Goto fatal error. */
	__asm__ ( " jmp .TekFileBinary_Flush_fatal " );
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Label for closing the descriptor. */
	__asm__ ( " .TekFileBinary_Flush_flush: " );
	/* Prepare to close the descriptor. */
	__asm__ ( " pushl %ecx " );
	/* Call fflush(). */
	__asm__ ( " call fflush " );
	/* Throw out the parameter. */
	__asm__ ( " addl $4, %esp " );

	/* Check to see if eax is zero. */
	__asm__ ( " testl %eax, %eax " );
	/* If it is, fflush() was successful. */
	__asm__ ( " je .TekFileBinary_Flush_done " );

	/* fflush() was not successful if we're here. */
	/* So, get value of errno. */
	__asm__ ( " movl %0, %%ecx " : "=m" (errno) );
	/* Perform a comparison for ENOSPC (disk full error) */
	__asm__ ( " cmpl %1, %%ecx " : "=m" (_tfbtmp) : "i" (ENOSPC) );
	/* Jump if ENOSPC is the error. */
	__asm__ ( " je .TekFileBinary_Flush_DiskFull " );
	/* Perform a comparison for EIO (device input/output error) */
	__asm__ ( " cmpl %1, %%ecx " : "=m" (_tfbtmp) : "i" (EIO) );
	/* Jump if EIO is the error. */
	__asm__ ( " je .TekFileBinary_Flush_IO " );
	/* Perform a comparison for EPIPE (pipe broken error) */
	__asm__ ( " cmpl %1, %%ecx " : "=m" (_tfbtmp) : "i" (EPIPE) );
	/* Jump if EIO is the error. */
	__asm__ ( " je .TekFileBinary_Flush_Pipe " );
	/* All other errors will be returned as TEKERR_UNEXPECTED. */

	/* For setting TEKERR_UNEXPECTED */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_UNEXPECTED) );
	__asm__ ( " jmp .TekFileBinary_Flush_fatal " );
	/* For setting TEKERR_FILE_DISKFULL */
	__asm__ ( " .TekFileBinary_Flush_DiskFull: " );
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_DISKFULL) );
	__asm__ ( " jmp .TekFileBinary_Flush_fatal " );
	/* For setting TEKERR_FILE_IO */
	__asm__ ( " .TekFileBinary_Flush_IO: " );
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_IO) );
	__asm__ ( " jmp .TekFileBinary_Flush_fatal " );
	/* For setting TEKERR_FILE_PIPE */
	__asm__ ( " .TekFileBinary_Flush_Pipe: " );
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_PIPE) );
	__asm__ ( " jmp .TekFileBinary_Flush_fatal " );

	/* We should now be here if fclose() was successful. */
	__asm__ ( " .TekFileBinary_Flush_done: " );
#if	TEKERR_OK != 0x00000000
	/* If TEKERR_OK is set to something non-standard, reset eax. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_OK) );
#endif	/* TEKERR_OK != 0x00000000 */

	/* Used for fatal returns. */
	__asm__ ( " .TekFileBinary_Flush_fatal: " );
}


/*!
    \fn TekFileBinary::Open()
    
    Open the file.
 */
TEKERR TekFileBinary::Open()
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Make sure this instance does not have a file already open. */
	/* Grab the PrivFileDesc. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );
	/* Test to see if it is NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If PrivFileDesc != NULL, go to step 2. */
	__asm__ ( " je .TekFileBinary_Open_RetrieveFilename " );

	/* If PrivFileDesc == NULL, report the error to eax. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Jump to the end of this function for cleanup. */
	__asm__ ( " jmp .TekFileBinary_Open_fatal " );

	/* Second, we need a file name to use. */
	__asm__ ( " .TekFileBinary_Open_RetrieveFilename: " );
#endif	/* not NO_INTERNAL_NULL_CHECKS */
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Grab the filename from this->PrivFileName.uchar_t_ascii. */
	__asm__ ( " movl %0, %%esi " : "=m" (this->PrivFileName.uchar_t_ascii) );
	/* Test to see if it is NULL. */
	__asm__ ( " testl %esi, %esi " );
	/* If PrivFileDesc != NULL, go to step 2. */
	__asm__ ( " jne .TekFileBinary_Open_DetermineParam " );

#ifdef	USE_386_ASM_PUSHPOP
	/* We really don't need esi at all. */
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* If PrivFileName.uchar_t_ascii == NULL, report the error to eax. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Jump to the end of this function for cleanup. */
	__asm__ ( " jmp .TekFileBinary_Open_fatal " );

	/* We should be here if the preliminary tests above passed. */
	__asm__ ( " .TekFileBinary_Open_DetermineParam: " );
	/* Figure out what fopen() op are going to do. */
	/* We need to pull the PrivFileMode value first. */
	__asm__ ( " movw %0, %%cx " : "=m" (this->PrivFileMode) );

	/* Third, let's see what mode we will run fopen() on. */
	__asm__ ( " bt $2, %cx " );
	__asm__ ( " jc .TekFileBinary_Open_ReadWrite " );
	__asm__ ( " bt $3, %cx " );
	__asm__ ( " jc .TekFileBinary_Open_ReadWriteTrunc " );
	__asm__ ( " bt $0, %cx " );
	__asm__ ( " jc .TekFileBinary_Open_WriteOnly " );
	__asm__ ( " bt $1, %cx " );
	__asm__ ( " jc .TekFileBinary_Open_Append " );
	__asm__ ( " bt $4, %cx " );
	__asm__ ( " jc .TekFileBinary_Open_ReadAppend " );

	/* If we are here, ReadOnly was set. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadOnly) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open_fopen " );

	/* Now, to insert a string parameter for param 1 of fopen(). */
	/* --- FOR MODE WRITEONLY --- */
	__asm__ ( " .TekFileBinary_Open_WriteOnly: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_WriteOnly) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open_fopen " );

	/* --- FOR MODE APPEND --- */
	__asm__ ( " .TekFileBinary_Open_Append: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_Append) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open_fopen " );

	/* --- FOR MODE READWRITE --- */
	__asm__ ( " .TekFileBinary_Open_ReadWrite: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadWrite) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open_fopen " );

	/* --- FOR MODE READWRITETRUNC --- */
	__asm__ ( " .TekFileBinary_Open_ReadWriteTrunc: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadWriteTrunc) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open_fopen " );

	/* --- FOR MODE APPEND --- */
	__asm__ ( " .TekFileBinary_Open_ReadAppend: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadAppend) );

	__asm__ ( " .TekFileBinary_Open_fopen: " );
	/* Push param 1 onto the stack first. */
	__asm__ ( " pushl %ecx " );
	/* Now push param 0 onto the stack. */
	__asm__ ( " pushl %esi " );
	/* Call fopen(). */
	__asm__ ( " call fopen " );
	/* Restore the stack pointer. */
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " addl $12, %esp " );
#else
	__asm__ ( " addl $8, %esp " );
#endif	/* USE_386_ASM_PUSHPOP */

	/* Test the return value. */
	__asm__ ( " testl %eax, %eax " );
	/* Did fopen() fail? */
	__asm__ ( " jne .TekFileBinary_Open_finish " );

	/* If we get here, fopen() has failed. */
	/* Set our retval to TEKERR_FAIL. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FAIL) );
	/* Do a failure cleanup. */
	__asm__ ( " jmp .TekFileBinary_Open_fatal " );

	/* If fopen() != NULL, we should be here. */
	__asm__ ( " .TekFileBinary_Open_finish: " );
	/* Finally, move the result to our PrivFileDesc. */
	__asm__ ( " movl 8(%ebp), %ecx " );
	/* Move result to PrivFileDesc. */
#ifdef	UNICODE
	__asm__ ( " movl %eax, 92(%ecx) " );
#else	/* not UNICODE */
	__asm__ ( " movl %eax, 84(%ecx) " );
#endif	/* not UNICODE */

	/* Set retval to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for fatal returns. */
	__asm__ ( " .TekFileBinary_Open_fatal: " );
}


/*!
    \fn TekFileBinary::Open64()
    
    Open the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Open64()
{
#ifdef	OPEN64_NOT_SUPPORTED
	return TEKERR_NOTIMPL;
#else	/* OPEN64_NOT_SUPPORTED */
#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Make sure this instance does not have a file already open. */
	/* Grab the PrivFileDesc. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );
	/* Test to see if it is NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If PrivFileDesc != NULL, go to step 2. */
	__asm__ ( " je .TekFileBinary_Open64_RetrieveFilename " );

	/* If PrivFileDesc == NULL, report the error to eax. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Jump to the end of this function for cleanup. */
	__asm__ ( " jmp .TekFileBinary_Open64_fatal " );

	/* Second, we need a file name to use. */
	__asm__ ( " .TekFileBinary_Open64_RetrieveFilename: " );
#endif	/* not NO_INTERNAL_NULL_CHECKS */
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " pushl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */
	/* Grab the filename from this->PrivFileName.uchar_t_ascii. */
	__asm__ ( " movl %0, %%esi " : "=m" (this->PrivFileName.uchar_t_ascii) );
	/* Test to see if it is NULL. */
	__asm__ ( " testl %esi, %esi " );
	/* If PrivFileDesc != NULL, go to step 2. */
	__asm__ ( " jne .TekFileBinary_Open64_DetermineParam " );

	/* If PrivFileName.uchar_t_ascii == NULL, report the error to eax. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Jump to the end of this function for cleanup. */
	__asm__ ( " jmp .TekFileBinary_Open64_fatal " );

	/* We should be here if the preliminary tests above passed. */
	__asm__ ( " .TekFileBinary_Open64_DetermineParam: " );
	/* Figure out what fopen() op are going to do. */
	/* We need to pull the PrivFileMode value first. */
	__asm__ ( " movw %0, %%cx " : "=m" (this->PrivFileMode) );

	/* Third, let's see what mode we will run fopen() on. */
	__asm__ ( " bt $2, %cx " );
	__asm__ ( " jc .TekFileBinary_Open64_ReadWrite " );
	__asm__ ( " bt $3, %cx " );
	__asm__ ( " jc .TekFileBinary_Open64_ReadWriteTrunc " );
	__asm__ ( " bt $0, %cx " );
	__asm__ ( " jc .TekFileBinary_Open64_WriteOnly " );
	__asm__ ( " bt $1, %cx " );
	__asm__ ( " jc .TekFileBinary_Open64_Append " );
	__asm__ ( " bt $4, %cx " );
	__asm__ ( " jc .TekFileBinary_Open64_ReadAppend " );

	/* If we are here, ReadOnly was set. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadOnly) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open64_fopen " );

	/* Now, to insert a string parameter for param 1 of fopen(). */
	/* --- FOR MODE WRITEONLY --- */
	__asm__ ( " .TekFileBinary_Open64_WriteOnly: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_WriteOnly) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open64_fopen " );

	/* --- FOR MODE APPEND --- */
	__asm__ ( " .TekFileBinary_Open64_Append: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_Append) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open64_fopen " );

	/* --- FOR MODE READWRITE --- */
	__asm__ ( " .TekFileBinary_Open64_ReadWrite: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadWrite) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open64_fopen " );

	/* --- FOR MODE READWRITETRUNC --- */
	__asm__ ( " .TekFileBinary_Open64_ReadWriteTrunc: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadWriteTrunc) );
	/* Jump to the fopen calling area. */
	__asm__ ( " jmp .TekFileBinary_Open64_fopen " );

	/* --- FOR MODE APPEND --- */
	__asm__ ( " .TekFileBinary_Open64_ReadAppend: " );
	/* Insert the string into ecx. */
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "X" (TekFileBinary_ReadAppend) );

	__asm__ ( " .TekFileBinary_Open64_fopen: " );
	/* Push param 1 onto the stack first. */
	__asm__ ( " pushl %ecx " );
	/* Now push param 0 onto the stack. */
	__asm__ ( " pushl %esi " );
	/* Call fopen(). */
	__asm__ ( " call fopen64 " );
	/* Restore the stack pointer. */
	__asm__ ( " addl $8, %esp " );

	/* Test the return value. */
	__asm__ ( " testl %eax, %eax " );
	/* Did fopen() fail? */
	__asm__ ( " jne .TekFileBinary_Open64_finish " );

	/* If we get here, fopen() has failed. */
	/* Set our retval to TEKERR_FAIL. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FAIL) );
	/* Do a failure cleanup. */
	__asm__ ( " jmp .TekFileBinary_Open64_fatal2 " );

	/* If fopen() != NULL, we should be here. */
	__asm__ ( " .TekFileBinary_Open64_finish: " );
	/* Finally, move the result to our PrivFileDesc. */
	__asm__ ( " movl 8(%ebp), %ecx " );
	/* Move result to PrivFileDesc. */
#ifdef	UNICODE
	__asm__ ( " movl %eax, 92(%ecx) " );
#else	/* not UNICODE */
	__asm__ ( " movl %eax, 84(%ecx) " );
#endif	/* not UNICODE */

	/* Set retval to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for fatal returns where esi has already been pushed. */
	__asm__ ( " .TekFileBinary_Open64_fatal2: " );
#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " popl %esi " );
#endif	/* USE_386_ASM_PUSHPOP */

	/* Used for fatal returns. */
	__asm__ ( " .TekFileBinary_Open64_fatal: " );
#endif	/* OPEN64_NOT_SUPPORTED */
}


/*!
    \fn TekFileBinary::put_Mode(TekFileMode ModeID)
    
    Sets the mode used to open the file.
 */
TEKERR TekFileBinary::put_Mode(TekFileMode ModeID)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	__asm__ ( " movl 8(%ebp), %eax ");
#ifdef	UNICODE
	__asm__ ( " movl 92(%eax), %ecx " );
#else	/* not UNICODE */
	__asm__ ( " movl 84(%eax), %ecx " );
#endif	/* not UNICODE */

	/* Make sure it isn't NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* Jump to processor if test shows esi != NULL. */
	__asm__ ( " je .TekFileBinary_put_Mode_NullCheckPass " );

	/* If esi == NULL, set retval to TEKERR_SEQUENCE. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_ACCESS) );
	/* Return to caller. */
	__asm__ ( " jmp .TekFileBinary_put_Mode_fatal " );

	/* Used to bypass error 1. */
	__asm__ ( " .TekFileBinary_put_Mode_NullCheckPass: " );

#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Get the value that is stored in ModeID. */
	__asm__ ( " movl %0, %%ecx " : "=m" (ModeID) );

	/* Put ModeID in this->PrivFileDesc. */
#ifdef	NO_INTERNAL_NULL_CHECKS
	__asm__ ( " movl 8(%ebp), %eax ");
#endif	/* NO_INTERNAL_NULL_CHECKS */
#ifdef	UNICODE
	__asm__ ( " movl %ecx, 96(%eax) " );
#else	/* not UNICODE */
	__asm__ ( " movl %ecx, 88(%eax) " );
#endif	/* not UNICODE */

	/* Set retval to TEKERR_OK. */
	TEKLTI_RETVAL_TEKERR_OK

#ifndef	NO_INTERNAL_NULL_CHECKS
	/* If this->PrivFileDesc was NULL, this label is jumped to. */
	__asm__ ( " .TekFileBinary_put_Mode_fatal: " );
#endif	/* not NO_INTERNAL_NULL_CHECKS */
}


/*!
    \fn TekFileBinary::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
    
    Read from the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
{
	return TEKERR_NOTIMPL;
}

/*!
    \fn TekFileBinary::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
    
    Read from the file.
    
    FIXME: Add internal checks for PrivFileDesc.
    FIXME: Correct optimization in most-likely no-error area between ferror and feof.
 */
TEKERR TekFileBinary::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
{
	/* Retrieve the private file descriptor. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );

#ifdef	NO_SIZE_T_CHECKS
	/* Prepare our stack. */
	__asm__ ( " subl $16, %esp " );

	/* and put the descriptor on the stack as parameter 3. */
	__asm__ ( " movl %ecx, 12(%esp) " );

	/* Retrieve the Buffer pointer. */
	__asm__ ( " movl %0, %%eax " : "=m" (Buffer) );
#else	/* not NO_SIZE_T_CHECKS */
	/* Put the file descriptor on the stack as parameter 3. */
	__asm__ ( " pushl %ecx " );

	/* Retrieve the Buffer pointer. */
	__asm__ ( " movl %0, %%ecx " : "=m" (Buffer) );

	/* Test to see if the Buffer pointer is null. */
	__asm__ ( " testl %ecx, %ecx " );
	/* Set the worst-case scenario for return. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_POINTER) );
	/* If NULL, go to PointerFail. */
	__asm__ ( " je .TekFileBinary_Read_done " );

	/* Prepare our stack. */
	__asm__ ( " subl $12, %esp " );

	/* Put the pointer to eax; we'll push it later. */
	__asm__ ( " movl %ecx, %eax " );
#endif	/* not NO_SIZE_T_CHECKS */

	/* Get the amount of bytes to read. */
	__asm__ ( " movl %0, %%ecx " : "=m" (BytesToRead) );
	/* and put it on the stack as parameter 2. */
	__asm__ ( " movl %ecx, 8(%esp) " );

	/* Put a 1 onto the stack as parameter 1. */
	__asm__ ( " movl $1, 4(%esp) " );

	/* Put the Buffer onto the stack. */
	__asm__ ( " movl %eax, (%esp) " );

	/* Finally, call fread. */
	__asm__ ( " call fread " );
	/* Clean up the stack. */
	__asm__ ( " addl $12, %esp " );

	/* Determine what happened. */
	__asm__ ( " testl %eax, %eax " );
	/* If we succeeded, we are ready to post how many characters were read. */
	__asm__ ( " jne .TekFileBinary_Read_postres " );

	/* If we're here, something failed. */
	/* Call feof() to see if we reached the EOF. */
	__asm__ ( " call feof " );
	/* Check the return value. */
	__asm__ ( " testl %eax, %eax " );
	/* Set our return value for this error condition. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_EOF) );
	/* If the eof was reached, we're done. */
	__asm__ ( " jne .TekFileBinary_Read_done " );

	/* If we're here, we have to process the error code. */
	__asm__ ( " movl %0, %%eax " : "=m" (errno) );

	/* Check for "broken pipe" error. */
	__asm__ ( " cmpl %1, %%eax " : "=m" (_tfbtmp) : "i" (EPIPE) );
	/* Jump as appropriate. */
	__asm__ ( " je .TekFileBinary_Read_EPIPE " );

	/* Set our return value. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_IO) );
	/* We're done. */
	__asm__ ( " jmp .TekFileBinary_Read_done " );

	/* Used for setting up retval on EPIPE.. */
	__asm__ ( " .TekFileBinary_Read_EPIPE: " );
	/* Set our return value for this error condition. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_PIPE) );
	/* We're done. */
	__asm__ ( " jmp .TekFileBinary_Read_done " );

	/* Used for setting up BytesRead. */
	__asm__ ( " .TekFileBinary_Read_postres: " );
	/* Retrieve BytesRead. */
	__asm__ ( " movl %0, %%ecx " : "=m" (BytesRead) );
	/* Test to see if it is NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If it is NULL, there's nothing else to do. */
	__asm__ ( " je .TekFileBinary_Read_succeeded " );

	/* Set the memory located by BytesRead to the result. */
	__asm__ ( " movl %eax, (%ecx) " );

	/* Used for returns. */
	__asm__ ( " .TekFileBinary_Read_succeeded: " );
	/* Set our return value for success. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for returns. */
	__asm__ ( " .TekFileBinary_Read_done: " );
	/* Clear the file descriptor off the stack. */
	__asm__ ( " addl $4, %esp " );
}


/*!
    \fn TekFileBinary::Seek64(TekFileSeek SeekWhere, uint64_t NewLocation)
    
    Seek to another position of the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
{
	/* Retrieve the SeekWhere value. */
	__asm__ ( " movl %0, %%ecx " : "=m" (SeekWhere) );

#if	!((TEKFILESEEK_BEGIN == SEEK_SET) && \
(TEKFILESEEK_CURRENT == SEEK_CUR) && \
(TEKFILESEEK_END == SEEK_END))
	/* Do a series of bit tests on SeekWhere. */
	__asm__ ( " bt $0, %cx " );
	__asm__ ( " jc .TekFileBinary_Seek_Cur " );
	__asm__ ( " bt $1, %cx " );
	__asm__ ( " jc .TekFileBinary_Seek_End " );

	/* If we're here, SeekWhere was set to BEGIN of file. */
#if	TEKFILESEEK_BEGIN != SEEK_SET
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (SEEK_SET) );
#endif	/* TEKFILESEEK_BEGIN != SEEK_SET */
	__asm__ ( " jmp .TekFileBinary_Seek_Where_Resume " );

	/* If we're here, SeekWhere was set to CURRENT of file. */
	__asm__ ( " .TekFileBinary_Seek_Cur: " );
#if	TEKFILESEEK_CURRENT != SEEK_CUR
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (SEEK_CUR) );
#endif	/* TEKFILESEEK_CURRENT != SEEK_CUR */
	__asm__ ( " jmp .TekFileBinary_Seek_Where_Resume " );

	/* If we're here, SeekWhere was set to END of file. */
	__asm__ ( " .TekFileBinary_Seek_End: " );
#if	TEKFILESEEK_END != SEEK_END
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (SEEK_END) );
#endif	/* TEKFILESEEK_CURRENT != SEEK_END */

	__asm__ ( " .TekFileBinary_Seek_Where_Resume: " );
#endif	/* !((TEKFILESEEK_BEGIN == SEEK_SET) && \
(TEKFILESEEK_CURRENT == SEEK_CUR) && \
(TEKFILESEEK_END == SEEK_END)) */

	/* Push ecx (param 2 of fseek()) onto the stack. */
	//__asm__ ( " pushl %ecx " );

	/* Move NewLocation to mm1. */
	__asm__ ( " movq %0, %%mm1 " : "=m" (NewLocation) );

	/* Retrieve parameter 1. */
	__asm__ ( " movd %mm1, %ecx " );
	/* Shift some bits. */
	__asm__ ( " psrld $8, %mm1 " );

	/* Clear MMX State. */
	__asm__ ( " emms " );

	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBinary::Seek(TekFileSeek SeekWhere, unsigned long NewLocation)
    
    Seek to another position of the file.
 */
TEKERR TekFileBinary::Seek(TekFileSeek SeekWhere, signed long NewLocation)
{
	/* Retrieve the SeekWhere value. */
	__asm__ ( " movl %0, %%ecx " : "=m" (SeekWhere) );

#if	!((TEKFILESEEK_BEGIN == SEEK_SET) && \
(TEKFILESEEK_CURRENT == SEEK_CUR) && \
(TEKFILESEEK_END == SEEK_END))
	/* Do a series of bit tests on SeekWhere. */
	__asm__ ( " bt $0, %cx " );
	__asm__ ( " jc .TekFileBinary_Seek_Cur " );
	__asm__ ( " bt $1, %cx " );
	__asm__ ( " jc .TekFileBinary_Seek_End " );

	/* If we're here, SeekWhere was set to BEGIN of file. */
#if	TEKFILESEEK_BEGIN != SEEK_SET
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (SEEK_SET) );
#endif	/* TEKFILESEEK_BEGIN != SEEK_SET */
	__asm__ ( " jmp .TekFileBinary_Seek_Where_Resume " );

	/* If we're here, SeekWhere was set to CURRENT of file. */
	__asm__ ( " .TekFileBinary_Seek_Cur: " );
#if	TEKFILESEEK_CURRENT != SEEK_CUR
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (SEEK_CUR) );
#endif	/* TEKFILESEEK_CURRENT != SEEK_CUR */
	__asm__ ( " jmp .TekFileBinary_Seek_Where_Resume " );

	/* If we're here, SeekWhere was set to END of file. */
	__asm__ ( " .TekFileBinary_Seek_End: " );
#if	TEKFILESEEK_END != SEEK_END
	__asm__ ( " movl %1, %%ecx " : "=m" (_tfbtmp) : "i" (SEEK_END) );
#endif	/* TEKFILESEEK_CURRENT != SEEK_END */

	__asm__ ( " .TekFileBinary_Seek_Where_Resume: " );
#endif	/* !((TEKFILESEEK_BEGIN == SEEK_SET) && \
(TEKFILESEEK_CURRENT == SEEK_CUR) && \
(TEKFILESEEK_END == SEEK_END)) */

	/* Push ecx (param 2 of fseek()) onto the stack. */
	__asm__ ( " pushl %ecx " );

	/* Retrieve the NewLocation value. */
	__asm__ ( " movl %0, %%ecx " : "=m" (NewLocation) );
	/* and push it onto the stack as param1 of fseek(). */
	__asm__ ( " pushl %ecx " );

	/* Retrieve the private file descriptor. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );

#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Make sure it isn't NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* Set retval to TEKERR_SEQUENCE. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* Push the file descriptor onto the stack. */
	__asm__ ( " pushl %ecx " );
	/* Jump to fatal return if test shows esi == NULL. */
	__asm__ ( " je .TekFileBinary_Seek_fail " );
#else	/* NO_INTERNAL_NULL_CHECKS */
	/* Push the file descriptor onto the stack. */
	__asm__ ( " pushl %ecx " );
#endif	/* NO_INTERNAL_NULL_CHECKS */

	/* Call fseek(). */
	__asm__ ( " call fseek " );
	/* Test the return value. */
	__asm__ ( " testl %eax, %eax " );
	/* If retval == 0, we succeeded. */
	__asm__ ( " je .TekFileBinary_Seek_ret " );
	/* If the function failed, set the value properly. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FAIL) );

	/* Used for normal returns. */
	__asm__ ( " .TekFileBinary_Seek_ret: " );

	/* Used for fatal returns. */
	__asm__ ( " .TekFileBinary_Seek_fail: " );

	/* Throw params 1, 2, and 3 off the stack and into the trash. */
	__asm__ ( " addl $12, %esp " );
}


/*!
    \fn TekFileBinary::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
    
    Write to the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBinary::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
    
    Write to the file.

    FIXME: Correct optimization in most-likely no-error area between ferror and feof.
 */
TEKERR TekFileBinary::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
{
	/* Retrieve the private file descriptor. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );

#ifdef	NO_SIZE_T_CHECKS
	/* Prepare our stack. */
	__asm__ ( " subl $16, %esp " );

	/* and put the descriptor on the stack as parameter 3. */
	__asm__ ( " movl %ecx, 12(%esp) " );

	/* Retrieve the Buffer pointer. */
	__asm__ ( " movl %0, %%eax " : "=m" (Buffer) );
#else	/* not NO_SIZE_T_CHECKS */
	/* Put the file descriptor on the stack as parameter 3. */
	__asm__ ( " pushl %ecx " );

	/* Retrieve the Buffer pointer. */
	__asm__ ( " movl %0, %%ecx " : "=m" (Buffer) );

	/* Test to see if the Buffer pointer is null. */
	__asm__ ( " testl %ecx, %ecx " );
	/* Set the worst-case scenario for return. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_POINTER) );
	/* If NULL, go to PointerFail. */
	__asm__ ( " je .TekFileBinary_Write_done " );

	/* Prepare our stack. */
	__asm__ ( " subl $12, %esp " );

	/* Put the pointer to eax; we'll push it later. */
	__asm__ ( " movl %ecx, %eax " );
#endif	/* not NO_SIZE_T_CHECKS */

	/* Get the amount of bytes to read. */
	__asm__ ( " movl %0, %%ecx " : "=m" (BytesToWrite) );
	/* and put it on the stack as parameter 2. */
	__asm__ ( " movl %ecx, 8(%esp) " );

	/* Put a 1 onto the stack as parameter 1. */
	__asm__ ( " movl $1, 4(%esp) " );

	/* Put the Buffer onto the stack. */
	__asm__ ( " movl %eax, (%esp) " );

	/* Finally, call fwrite. */
	__asm__ ( " call fwrite " );
	/* Clean up the stack. */
	__asm__ ( " addl $12, %esp " );

	/* Determine what happened. */
	__asm__ ( " testl %eax, %eax " );
	/* If we succeeded, we are ready to post how many characters were written. */
	__asm__ ( " jne .TekFileBinary_Write_postres " );

	/* If we're here, something failed. */
	/* We have to process the error code. */
	__asm__ ( " movl %0, %%eax " : "=m" (errno) );

	/* Check for "broken pipe" error. */
	__asm__ ( " cmpl %1, %%eax " : "=m" (_tfbtmp) : "i" (EPIPE) );
	/* Jump as appropriate. */
	__asm__ ( " je .TekFileBinary_Write_EPIPE " );

	/* Check for "out of space on device" error. */
	__asm__ ( " cmpl %1, %%eax " : "=m" (_tfbtmp) : "i" (ENOSPC) );
	/* Jump as appropriate. */
	__asm__ ( " je .TekFileBinary_Write_EPIPE " );

	/* Check for "read only" error. */
	__asm__ ( " cmpl %1, %%eax " : "=m" (_tfbtmp) : "i" (EBADF) );
	/* Jump as appropriate. */
	__asm__ ( " je .TekFileBinary_Write_EBADF " );

	/* Used for IO errors. */
	__asm__ ( " .TekFileBinary_Write_EIO: " );
	/* Set our return value. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_IO) );
	/* We're done. */
	__asm__ ( " jmp .TekFileBinary_Write_done " );

	/* Used for setting up retval on EPIPE.. */
	__asm__ ( " .TekFileBinary_Write_EPIPE: " );
	/* Set our return value for this error condition. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_PIPE) );
	/* We're done. */
	__asm__ ( " jmp .TekFileBinary_Write_done " );

	/* Used for setting up retval on ENOSPC. */
	__asm__ ( " .TekFileBinary_Write_ENOSPC: " );
	/* Set our return value for this error condition. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_DISKFULL) );
	/* We're done. */
	__asm__ ( " jmp .TekFileBinary_Write_done " );

	/* Used for setting up retval on EBADF. */
	__asm__ ( " .TekFileBinary_Write_EBADF: " );
	/* Test to see if this file was open read-only. */
	/* We need to pull the PrivFileMode value first. */
	__asm__ ( " movw %0, %%cx " : "=m" (this->PrivFileMode) );
	/* Test the PrivFileMode to see if it was open for read-only. */
	__asm__ ( " testw %cx, %cx " );
	/* Error returned will be TEKERR_FILE_IO if test fails. */
	__asm__ ( " jne .TekFileBinary_Write_EIO " );

	/* Set our return value for this error condition. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_READONLY) );
	/* We're done. */
	__asm__ ( " jmp .TekFileBinary_Write_done " );
	/* END OF EBADF PROCESSING */

	/* Used for setting up BytesRead. */
	__asm__ ( " .TekFileBinary_Write_postres: " );
	/* Retrieve BytesRead. */
	__asm__ ( " movl %0, %%ecx " : "=m" (BytesWrote) );
	/* Test to see if it is NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If it is NULL, there's nothing else to do. */
	__asm__ ( " je .TekFileBinary_Write_succeeded " );

	/* Set the memory located by BytesRead to the result. */
	__asm__ ( " movl %eax, (%ecx) " );

	/* Used for returns. */
	__asm__ ( " .TekFileBinary_Write_succeeded: " );
	/* Set our return value for success. */
	TEKLTI_RETVAL_TEKERR_OK

	/* Used for returns. */
	__asm__ ( " .TekFileBinary_Write_done: " );
	/* Clear the file descriptor off the stack. */
	__asm__ ( " addl $4, %esp " );
}


/*!
    \fn TekFileBinary::get_Location(unsigned long * Location)

    Retrieves the location and stores it in a 32-bit variable.
 */

TEKERR TekFileBinary::get_Location(unsigned long * Location)
{
	/* Retrieve the private file descriptor. */
	__asm__ ( " movl %0, %%ecx " : "=m" (this->PrivFileDesc) );

#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Test to see if ecx is NULL. */
	__asm__ ( " testl %ecx, %ecx " );
	/* See retval for worst case scenario. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_SEQUENCE) );
	/* If it is NULL, there's nothing to do. */
	__asm__ ( " je .TekFileBinary_get_Location_ret " );
#endif	/* NO_INTERNAL_NULL_CHECKS */

	/* Put the descriptor on the stack as parameter 0. */
	__asm__ ( " pushl %ecx " );
	/* Call ftell. */
	__asm__ ( " call ftell " );
	/* Reset the stack. */
	__asm__ ( " addl $4, %esp " );

	/* Get the Location pointer. */
	__asm__ ( " movl %0, %%ecx " : "=m" (Location) );
	/* Put result into Location. */
	__asm__ ( " movl %eax, (%ecx) " );
	/* cmp the result. */
	__asm__ ( " cmpl $-1, %eax " );
	/* Jump to end if we succeeded. */
	__asm__ ( " jg .TekFileBinary_get_Location_ret " );

	/* Set retval for failure. */
	__asm__ ( " movl %1, %%eax " : "=m" (_tfbtmp) : "i" (TEKERR_FILE_IO) );
	/* Leave and return to caller. */
	__asm__ ( " leave " );
	__asm__ ( " ret " );

	/* Used for returns. */
	__asm__ ( " .TekFileBinary_get_Location_ret: " );
	/* Set retval for success. */
	TEKLTI_RETVAL_TEKERR_OK
}


/*!
    \fn TekFileBinary::get_Location64(uint64_t * Location)

    Retrieves the location and stores it in a 64-bit variable.
 */

TEKERR TekFileBinary::get_Location64(uint64_t * Location)
{
	return TEKERR_NOTIMPL;
}
#endif	/* USE_386_ASM */


