/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)

#ifndef	_cplusplus
#define	_cplusplus
#endif	/* _cplusplus */
#include "teklti.h"

#include <limits.h>
#include <errno.h>

MKTEKUUID(UTekFileBinary, "d60710c0-e5ea-11d9-8325-000bdbc434d9")
MKTEKDEBUGSTRING(DBTekFileBinary, "TekFileBinary")


#ifndef	USE_386_ASM

/**
Instantizer.
*/
TekFileBinary::TekFileBinary()
{
	//TEKSTD_IMPLEMENT_BEGIN(UTekFileBinary)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFile)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFileBinary)
	TEKSTD_IMPLEMENT_ENDBASE

	this->PrivFileName.uchar_t_ascii = NULL;
	this->PrivFileName.uchar_t_asciilen = 0;
	this->PrivFileDesc = NULL;
	PrivFileMode = TekFileMode_ReadOnly;
	PrivTekFileType = TekFileType_Binary;
}


/**
Destructor.
*/
TekFileBinary::~TekFileBinary()
{
	if ( this->PrivFileDesc != NULL )
		fclose(this->PrivFileDesc);
	if ( this->PrivFileName.uchar_t_ascii != NULL )
		free(this->PrivFileName.uchar_t_ascii);
}


const char TekFileBinary_ReadOnly[] = "rb";
const char TekFileBinary_WriteOnly[] = "wb";
const char TekFileBinary_Append[] = "ab";
const char TekFileBinary_ReadWrite[] = "r+b";
const char TekFileBinary_ReadWriteTrunc[] = "w+b";
const char TekFileBinary_ReadAppend[] = "a+b";

/**
    \fn TekFileBinary::ConvertToPipe()
    
    Changes the file type variable to BinaryPipe.\n
    \n
    NOTE: This function should only be called when the contained FILE * is actually a pipe.  This is a one-time change of the file type on any instance of BinaryPipe   called.  Therefore, it is best that this function be called only when passing an instance of TekFileBinary to a library (or to the caller when created in libraries) where you are not sure how the library or callee handles a pipe differently from a regular file.
 */
TEKERR TekFileBinary::ConvertToPipe()
{
	this->PrivTekFileType = TekFileType_BinaryPipe;
	return TEKERR_OK;
}

/**
    \fn TekFileBinary::get_FILE(FILE ** FileDescriptor)
    
    Obtains the handle used to access the open file.
 */
TEKERR TekFileBinary::get_FILE(FILE ** FileDescriptor)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( FileDescriptor == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	*FileDescriptor = this->PrivFileDesc;
	
	return TEKERR_OK;
}


/**
    \fn TekFileBinary::get_Filename(uchar_t ** Name)
    
    Obtains the filename in the form of a uchar.
 */
TEKERR TekFileBinary::get_Filename(uchar_t ** Name)
{
	uchar_t * newval;
	
#ifndef	NO_SIZE_T_CHECKS
	if ( Name == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileName.uchar_t_ascii == NULL )
		return TEKERR_UNEXPECTED;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Convert our ASCII string to a uchar. */
	newval = char2uchar(this->PrivFileName.uchar_t_ascii);
	if ( newval == NULL )
		return TEKERR_MEMORY;

	/* Set up our uchar. */
	*Name = newval;

	/* Return to caller. */
	return TEKERR_OK;
}


/**
    \fn TekFileBinary::put_FILE(FILE * FileDescriptor)
    
    Sets the handle used to access a file.\n
    \n
    WARNING!!!!!  DO NOT USE THE FILE DESCRIPTOR YOU PASS IN TO THIS FUNCTION UNLESS YOU HAVE RE-RETRIEVED IT USING TekFile::get_FILE()!  Utilizing the descriptor in a direct fashion after a successful call to put_FILE() will result in undefined results!!!!!
 */
TEKERR TekFileBinary::put_FILE(FILE * FileDescriptor)
{
	this->PrivFileDesc = FileDescriptor;
	
	return TEKERR_OK;
}


/**
    \fn TekFileBinary::put_Filename(uchar_t * Name)
    
    Sets the filename to access.
 */
TEKERR TekFileBinary::put_Filename(uchar_t * Name)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( Name == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	this->PrivFileName.uchar_t_ascii = strdup(uchar2char(Name));

	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::get_Type(TekFileType * FileTypePointer)
    
    Retrieves the filename to access.
 */
TEKERR TekFileBinary::get_Type(TekFileType * FileTypePointer)
{
	TekFileType * retval;

	retval = FileTypePointer;

#ifndef	NO_SIZE_T_CHECKS
	if ( FileTypePointer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	retval[0] = this->PrivTekFileType;

	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::Close()
    
    Closes the file.
 */
TEKERR TekFileBinary::Close()
{
	int err;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	err = fclose(this->PrivFileDesc);

	if ( err != 0 )
	{
		switch ( err )
		{
			case ENOSPC:
				return TEKERR_FILE_DISKFULL;
			case EIO:
			case EPIPE:
				return TEKERR_FILE_IO;
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	this->PrivFileDesc = NULL;

	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::get_Mode(TekFileMode * ModeID)
    
    Retrieves the mode currently set.
 */
TEKERR TekFileBinary::get_Mode(TekFileMode * ModeID)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( ModeID == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	*ModeID = this->PrivFileMode;

	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::Flush()
    
    Flushes the buffer.
 */
TEKERR TekFileBinary::Flush()
{
	int err;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	err = fflush(this->PrivFileDesc);

	if ( err != 0 )
	{
		switch ( err )
		{
			case ENOSPC:
				return TEKERR_FILE_DISKFULL;
			case EIO:
			case EPIPE:
				return TEKERR_FILE_IO;
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::Open()
    
    Open the file.
 */
TEKERR TekFileBinary::Open()
{
	const char * tmpopenvar;

	/* Make sure this instance does not have a file already open. */
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Figure out what fopen() op are going to do. */
	switch ( this->PrivFileMode )
	{
		case TekFileMode_WriteOnly:
			tmpopenvar = TekFileBinary_WriteOnly;
			break;
		case TekFileMode_Append:
			tmpopenvar = TekFileBinary_Append;
			break;
		case TekFileMode_ReadWrite:
			tmpopenvar = TekFileBinary_ReadWrite;
			break;
		case TekFileMode_ReadWriteTrunc:
			tmpopenvar = TekFileBinary_ReadWriteTrunc;
			break;
		case TekFileMode_ReadAppend:
			tmpopenvar = TekFileBinary_ReadAppend;
			break;
		default:
			tmpopenvar = TekFileBinary_ReadOnly;
			break;
	}

	/* Open the file. */
	this->PrivFileDesc = fopen(
		this->PrivFileName.uchar_t_ascii,
		tmpopenvar
	);
	if ( this->PrivFileDesc == NULL )
		return TEKERR_FAIL;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::Open64()
    
    Open the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Open64()
{
#ifndef	OPEN64_NOT_SUPPORTED
	const char * tmpopenvar;

	/* Make sure this instance does not have a file already open. */
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Figure out what fopen() op are going to do. */
	switch ( this->PrivFileMode )
	{
		case TekFileMode_ReadOnly:
			tmpopenvar = TekFileBinary_ReadOnly;
			break;
		case TekFileMode_WriteOnly:
			tmpopenvar = TekFileBinary_WriteOnly;
			break;
		case TekFileMode_Append:
			tmpopenvar = TekFileBinary_Append;
			break;
		case TekFileMode_ReadWrite:
			tmpopenvar = TekFileBinary_ReadWrite;
			break;
		case TekFileMode_ReadWriteTrunc:
			tmpopenvar = TekFileBinary_ReadWriteTrunc;
			break;
		case TekFileMode_ReadAppend:
			tmpopenvar = TekFileBinary_ReadAppend;
			break;
		default:
			return TEKERR_UNAVAILABLE;
	}

	/* Open the file. */
	this->PrivFileDesc = fopen64(
		this->PrivFileName.uchar_t_ascii,
		tmpopenvar
	);
	if ( this->PrivFileDesc == NULL )
		return TEKERR_FAIL;

	/* Return to caller. */
	return TEKERR_OK;
#else	/* OPEN64_NOT_SUPPORTED */
	return TEKERR_NOTIMPL;
#endif	/* OPEN64_NOT_SUPPORTED */
}


/*!
    \fn TekFileBinary::put_Mode(TekFileMode ModeID)
    
    Sets the mode used to open the file.
 */
TEKERR TekFileBinary::put_Mode(TekFileMode ModeID)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	this->PrivFileMode = ModeID;
	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
    
    Read from the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBinary::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
    
    Read from the file.
 */
TEKERR TekFileBinary::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
{
	size_t bytesread;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Write the buffer to the file. */
	bytesread = fread(Buffer, sizeof(char), BytesToRead, this->PrivFileDesc);

	/* Put the result into BytesWrote. */
	*BytesRead = bytesread;

	/* Check for success or failure. */
	if ( bytesread > 0 )
		return TEKERR_OK;

	/* Check for EOF error. */
	if ( feof(this->PrivFileDesc) != 0 )
		return TEKERR_FILE_EOF;

	/* Check for EPIPE error. */
	if ( errno == EPIPE )
		return TEKERR_FILE_PIPE;

	/* All other errors are considered IO errors. */
	return TEKERR_FILE_IO;
}


/*!
    \fn TekFileBinary::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
    
    Seek to another position of the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBinary::Seek(TekFileSeek SeekWhere, signed long NewLocation)
    
    Seek to another position of the file.
 */
TEKERR TekFileBinary::Seek(TekFileSeek SeekWhere, signed long NewLocation)
{
	int SeekWhence;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
  		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	switch ( SeekWhere )
	{
		case TekFileSeek_Begin:
			SeekWhence = SEEK_SET;
			break;
		case TekFileSeek_Current:
			SeekWhence = SEEK_CUR;
			break;
		case TekFileSeek_End:
			SeekWhence = SEEK_END;
			break;
		default:
			return TEKERR_UNAVAILABLE;
	}

	if ( fseek(this->PrivFileDesc, NewLocation, SeekWhence) )
		return TEKERR_FAIL;

	return TEKERR_OK;
}


/*!
    \fn TekFileBinary::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
    
    Write to the file using 64 bit semantics.
 */
TEKERR TekFileBinary::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileBinary::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
    
    Write to the file.
 */
TEKERR TekFileBinary::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
{
	size_t written;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Write the buffer to the file. */
	written = fwrite(Buffer, sizeof(char), BytesToWrite, this->PrivFileDesc);

	/* Put the result into BytesWrote. */
	*BytesWrote = written;

	/* Check for success or failure. */
	if ( written == BytesToWrite )
		return TEKERR_OK;

	/* We have to find out what the error is. */
	switch ( errno )
	{
		case EPIPE:
			return TEKERR_FILE_PIPE;
		case ENOSPC:
			return TEKERR_FILE_DISKFULL;
		case EBADF:
			/* Check for read-only status. */
			if ( this->PrivFileMode == TekFileMode_ReadOnly )
				return TEKERR_FILE_READONLY;
		case EIO:
		default:
			return TEKERR_FILE_IO;
	}
}


/*!
    \fn TekFileBinary::get_Location(unsigned long * Location)

    Retrieves the location and stores it in a 32-bit variable.
 */

TEKERR TekFileBinary::get_Location(unsigned long * Location)
{
	long loc;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Get the current file position. */
	loc = ftell(this->PrivFileDesc);

	/* Check for error. */
	if ( loc != -1 )
	{
		/* Move location to memory referenced by Location. */
		*Location = (unsigned long)loc;

		/* Indicate success. */
		return TEKERR_OK;
	}

	/* See if this is a pipe error. */
	if ( errno != EPIPE )
		return TEKERR_FILE_IO;

	return TEKERR_FILE_PIPE;
}


/*!
    \fn TekFileBinary::get_Location64(uint64_t * Location)

    Retrieves the location and stores it in a 64-bit variable.
 */

TEKERR TekFileBinary::get_Location64(uint64_t * Location)
{
	return TEKERR_NOTIMPL;
}

#endif	/* not USE_386_ASM */

