/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)

#ifndef	_cplusplus
#define	_cplusplus
#endif	/* _cplusplus */
#include "teklti.h"


#ifdef	NULL
#undef	NULL
#define	NULL	(0)
#endif	/* NULL */


MKTEKUUID(UTekStatFile, "E369BDA-3789-11DA-8436-000BDBC434D9");
MKTEKDEBUGSTRING(DBTekStatFile, "DBTekStatFile");


#if	NULL != 0
/**
This interface defines the standard for viewing file information as a file stands on a device.\n
\n
This interface is <b>INHERITABLE</b>.\n
\n
<b>NOTE:</b> Most interfaces implementing TekStatFile might give out a TEKERR_ACCESS if
you do not put_Filename() or put_FILE().

@author Kevin McBride
*/
class TekStatFile : public TekStd
{
public:
  TekStatFile();
  ~TekStatFile();

  virtual TEKERR put_Filename(const uchar_t * Filename) = 0;
  virtual TEKERR put_FILE(FILE * FileDescriptor) = 0;
  virtual TEKERR put_UserInfoToAccess(uuid_t * UUIDOfUserInfo) = 0;

  virtual TEKERR MakeTekFile(TekFileType TekType, TekFile ** RetVal) = 0;

  virtual TEKERR get_Dev(dev_t * Dev) = 0;
  virtual TEKERR get_DevNo(dev_t * DevNo) = 0;
  virtual TEKERR get_FileSerialNo(ino_t * SerialNo) = 0;
  virtual TEKERR get_Mode(mode_t * FileMode) = 0;
  virtual TEKERR get_ModeString(uchar_t ** FileModeString) = 0;
  virtual TEKERR put_Mode(mode_t NewFileMode) = 0;
  virtual TEKERR put_ModeString(const uchar_t * NewFileModeString) = 0;
  virtual TEKERR get_LinkCount(nlink_t * LinkCount) = 0;
  virtual TEKERR get_UID(uid_t * UID) = 0;
  virtual TEKERR put_UID(uid_t UID) = 0;
  virtual TEKERR get_GID(gid_t * GID) = 0;
  virtual TEKERR put_GID(gid_t GID) = 0;
  virtual TEKERR get_UIDString(uchar_t ** UIDString) = 0;
  virtual TEKERR put_UIDString(const uchar_t * UIDString) = 0;
  virtual TEKERR get_GIDString(uchar_t ** GIDString) = 0;
  virtual TEKERR put_GIDString(const uchar_t * GIDString) = 0;
  virtual TEKERR get_Size(unsigned long * FileSize) = 0;
  virtual TEKERR get_Size64(uint64_t * FileSize) = 0;
  virtual TEKERR get_BlockSize(blksize_t * FileBlockSize) = 0;
  virtual TEKERR get_BlockCount(blkcnt64_t * FileBlockCount) = 0;
  virtual TEKERR get_TimeOfLastAccessRaw(unsigned long ** RetVal) = 0;
  virtual TEKERR get_TimeOfLastModificationRaw(unsigned long ** RetVal) = 0;
  virtual TEKERR get_TimeOfLastStatChangeRaw(unsigned long ** RetVal) = 0;
  virtual TEKERR get_TimeOfLastAccess(uchar_t ** RetVal) = 0;
  virtual TEKERR get_TimeOfLastModification(uchar_t ** RetVal) = 0;
  virtual TEKERR get_TimeOfLastStatChange(uchar_t ** RetVal) = 0;

/**
This contains the pure virtual defines.
*/
  TEKSTD_PUREVIRTUAL_FUNC_V1
};
#endif


/**
Instantizer.
*/
TekStatFile::TekStatFile()
{
	;
}


/**
\fn TekStatFile::~TekStatFile()

Destructor.
*/
TekStatFile::~TekStatFile()
{
	;
}


#if	NULL != 0
/**
    \fn TekStatFile::put_Filename(const uchar_t * Filename)
    
    Retrieves information from named file.
 */
TEKERR TekStatFile::put_Filename(const uchar_t * Filename)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_FILE(FILE * FileDescriptor)
    
    Retrieves information from file descriptor.
 */
TEKERR TekStatFile::put_FILE(FILE * FileDescriptor)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_UserInfoToAccess(uuid_t * UUIDOfUserInfo)
    
    Sets the userinfo access interface to read usernames from.\n
    \n
    <b>NOTE:</b> If this is not called, the inheriting interface will
    choose the scheme on its own, which could vary from operating system
    to operating system.  So, unless you don't mind of the inheriting
    interface choosing the scheme for you, you should utilize setting
    this parameter.
 */
TEKERR TekStatFile::put_UserInfoToAccess(uuid_t * UUIDOfUserInfo)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::MakeTekFile(TekFileType TekType, TekFile ** RetVal)
    
    Makes an unopened instance of a TekFile, type TekType.  This function
    is included here as a timesaver inside consitional blocks.
 */
TEKERR MakeTekFile(TekFileType TekType, TekFile ** RetVal)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_Dev(dev_t * Dev)
    
    Retrieves the device.
 */
TEKERR TekStatFile::get_Dev(dev_t * Dev)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_DevNo(dev_t * DevNo)
    
    Retrieves the device number.
 */
TEKERR TekStatFile::get_DevNo(dev_t * DevNo)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_FileSerialNo(ino_t * SerialNo)
    
    Retrieves the file serial number.
 */
TEKERR TekStatFile::get_FileSerialNo(ino_t * SerialNo)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_Mode(mode_t * FileMode)
    
    Retrieves the raw mode.
 */
TEKERR TekStatFile::get_Mode(mode_t * FileMode)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_ModeString(uchar_t ** FileModeString)
    
    Retrieves the mode string.
 */
TEKERR TekStatFile::get_ModeString(uchar_t ** FileModeString)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_Mode(mode_t NewFileMode)
    
    Sets the mode in raw form.
 */
TEKERR TekStatFile::put_Mode(mode_t NewFileMode)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_ModeString(const uchar_t * NewFileModeString)
    
    Sets the mode using string format.  The format to follow will
    be documented by the inheriting interfaces.
 */
TEKERR TekStatFile::put_ModeString(const uchar_t * NewFileModeString)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_LinkCount(nlink_t * LinkCount)
    
    Retrieves the mode string.
 */
TEKERR TekStatFile::get_LinkCount(nlink_t * LinkCount)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_UID(uid_t * UID)
    
    Retrieves the user ID in raw form.
 */
TEKERR TekStatFile::get_UID(uid_t * UID)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_UID(uid_t UID)
    
    Sets the user ID in raw form.
 */
TEKERR TekStatFile::put_UID(uid_t UID)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_GID(gid_t * GID)
    
    Retrieves the group ID in raw form.
 */
TEKERR TekStatFile::get_GID(gid_t * GID)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_GID(gid_t GID)
    
    Sets the group ID in raw form.
 */
TEKERR TekStatFile::put_GID(gid_t GID)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_UIDString(uchar_t ** UIDString)
    
    Retrieves the Username behind the UID.
 */
TEKERR TekStatFile::get_UIDString(uchar_t ** UIDString)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_UIDString(const uchar_t * UIDString)
    
    Sets the Username.
 */
TEKERR TekStatFile::put_UIDString(const uchar_t * UIDString)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_GIDString(uchar_t ** GIDString)
    
    Retrieves the Groupname behind the GID.
 */
TEKERR TekStatFile::get_GIDString(uchar_t ** GIDString)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::put_GIDString(const uchar_t * GIDString)
    
    Sets the Groupname.
 */
TEKERR TekStatFile::put_GIDString(const uchar_t * GIDString)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_Size(unsigned long * FileSize)
    
    Retrieves the file size.
 */
TEKERR TekStatFile::get_Size(unsigned long * FileSize)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_Size64(uint64_t * FileSize)
    
    Retrieves the file size, cast to a uint64_t.
 */
TEKERR TekStatFile::get_Size64(uint64_t * FileSize)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_BlockSize(blksize_t * FileBlockSize)
    
    Retrieves the block size to use for optimum performance.
 */
TEKERR TekStatFile::get_BlockSize(blksize_t * FileBlockSize)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_BlockCount(blkcnt64_t * FileBlockCount)
    
    Retrieves the amount of blocks the file is taking.
 */
TEKERR TekStatFile::get_BlockCount(blkcnt64_t * FileBlockCount)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_TimeOfLastAccessRaw(unsigned long ** RetVal)
    
    Retrieves the time of last access to file, in raw form.
 */
TEKERR TekStatFile::get_TimeOfLastAccessRaw(unsigned long ** RetVal)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_TimeOfLastModificationRaw(unsigned long ** RetVal)
    
    Retrieves the time of last modification to file, in raw form.
 */
TEKERR TekStatFile::get_TimeOfLastModificationRaw(unsigned long ** RetVal)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_TimeOfLastStatChangeRaw(unsigned long ** RetVal)
    
    Retrieves the time of last stat change to file, in raw form.
 */
TEKERR TekStatFile::get_TimeOfLastStatChangeRaw(unsigned long ** RetVal)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_TimeOfLastAccess(uchar_t ** RetVal)
    
    Retrieves the time of last access to file.
 */
TEKERR TekStatFile::get_TimeOfLastAccess(uchar_t ** RetVal)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_TimeOfLastModification(uchar_t ** RetVal)
    
    Retrieves the time of last modification to file.
 */
TEKERR TekStatFile::get_TimeOfLastModification(uchar_t ** RetVal)
{
	return TEKERR_NOTIMPL;
}

/**
    \fn TekStatFile::get_TimeOfLastStatChange(uchar_t ** RetVal)
    
    Retrieves the time of last stat change to file.
 */
TEKERR TekStatFile::get_TimeOfLastStatChange(uchar_t ** RetVal)
{
	return TEKERR_NOTIMPL;
}
#endif	/* NULL != 0 */

