/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)
//#define	TEKSTD_INTERNAL 	(1)

#ifndef	_cplusplus
#define	_cplusplus	(0)
#endif	/* _cplusplus */

#include "teklti.h"

#ifndef	NULLTEST
#define	NULLTEST	(0)
#endif	/* not NULLTEST */


MKTEKUUID(UTekStd, "567de9ac-d51c-11d9-871d-000bdbc434d9")
MKTEKDEBUGSTRING(DBTekStd, "TekStd")


/**
Constructor.
*/
TekStd::TekStd()
{
	TEKSTD_IMPLEMENT_BEGIN
	TEKSTD_IMPLEMENT_ENDVARS
}

/**
Destructor.
*/
TekStd::~TekStd()
{
	TEKSTD_IMPLEMENT_FREE
}

//#if	NULLTEST != 0

//TEKSTD_SOURCE_FUNC(TekStd)

/**
Adds a reference to the instance.
*/
unsigned int TekStd::AddRef()
{
	/* Add a reference to the instance. */
	__instancecount++;
	/* Return reference count to the caller. */
	return __instancecount;
}

/**
Checks the instance for support of a particular set of interfaces.\n
\n
If RetVal != NULL and Flags are set to 1, then CheckFor() will create a pointer to the instance and set it in the memory reference provided by RetVal.\n
\n
Unlike most functions, CheckFor() will return a TEKERR value based on the return condition:\n
\n
- On sucessful completion, a flat TEKERR_OK is returned in the TEKERR value.\n
\n
- If RetVal is NULL when Flags is 1, a flat TEKERR_POINTER may be returned in the TEKERR value.\n
\n
- If an element in UUIDArray is not found in <b>__uuidarray</b>, TEKERR will be set as follows:\n
  <b>0x00020001</b>\n
  Where <b>0x0001</b> is set to TEKERR_NOTIMPL, cast to an unsigned short.\n
  Where <b>0x0002</b> will be set to the array element causing the error.\n
  So, in the above example, it reads "UUIDArray[1] did not match anything in __uuidarray." \n
  <i>TEKLTI_CHECKFOR_FAILING_ELEMENT(i)</i> can be used to retrieve the element number,
but only works when the TEKERR_NOTIMPL error occurs.
*/
TEKERR TekStd::CheckFor(
	const uuid_t * UUIDArray,
	size_t UUIDArraySize,
	unsigned short Flags,
	void ** RetVal,
	unsigned int ** RefCountPtrRet
)
{
	TEKERR result; /* Used to prepare return value. */
	size_t i; /* Used for enumerating UUIDArray. */
	size_t j; /* Used for enumerating this->__uuidarray. */

#ifndef	NO_SIZE_T_CHECKS
	if ( UUIDArray == NULL )
	{
#ifdef	DEBUG_USE_FERROR
		fprintf(
			stderr,
			"POTENTIAL SEGFAULT in interface %s, function CheckFor(): UUIDArray is null!\n",
			TEKDEBUGSTRING(DBTekStd)
		);
#endif	/* DEBUG_USE_FERROR */
		return TEKERR_POINTER;
	}
	
	if ( Flags )
	{
		if ( RetVal == NULL )
		{
#ifdef	DEBUG_USE_FERROR
		fprintf(
			stderr,
			"POTENTIAL SEGFAULT in interface %s, function CheckFor(): RetVal is null!\n",
			TEKDEBUGSTRING(DBTekStd)
		);
#endif	/* DEBUG_USE_FERROR */
		return TEKERR_POINTER;
		}
	}
#endif	/* NO_SIZE_T_CHECKS */

	for ( i=0; i < UUIDArraySize; i++ )
	{
		/*
		  Check our private UUID array to see if UUIDArray[i]
		  is supported by this instance.
		*/
		for ( j=0; j < this->__uuidarraycount; j++ )
			if ( uuid_compare(this->__uuidarray[j], UUIDArray[i]) == 0 )
				goto FoundUUID; /* UUIDArray[i] is supported. */

		/* We get here if UUIDArray[i] was not supported. */
		/* Increment 'i' to help caller make debugging output. */
		i++;
		/* Set upper 16 bits to failing UUIDArray element. */
		result = i << (8 * sizeof(short));
		/* Set lower 16 bits to TEKERR_NOTIMPL. */
		result |= TEKERR_NOTIMPL;
		/* Return the 'result' value to caller. */
		return result;
FoundUUID:
		;
	}

	/* We get here if the checks passed. */
	/* If Flags is set to 1, we are to add a reference and set RetVal. */
	if ( Flags )
	{
		/* Add a reference to ourselves. */
		this->AddRef();
		/* Set RetVal to ourselves. */
		*RetVal = this;
	}

	/* Return to caller with TEKERR_OK status.  That's all folks. */
	return TEKERR_OK;
}

/**
Removes a reference to the instance.
*/
unsigned int TekStd::Release()
{
	/* Decrement reference count. */
	__instancecount--;

	/* If instancecount is now zero, we need to delete ourselves. */
	if ( __instancecount == 0 )
	{
		/* Delete ourselves. */
		delete this;
		/* Return zero to indicate "we no longer exist!" */
		return 0;
	}

	/* Return reference count to caller. */
	return __instancecount;
}

TEKERR TekStd::CallFunc(
	const uchar_t * FuncName,
	void * RetVal,
	unsigned short ParamsCount,
	void ** Params
)
{
	return TEKERR_NOTIMPL;
}

//#endif	/* NULL != 0 */

