/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#define LIBTEKLTI_EXPORT (1)

#ifndef _cplusplus
#define _cplusplus
#endif /* _cplusplus */

#include "teklti.h"

MKTEKUUID(UTekUserUnixRO, "EF449EEC-3E85-11DA-80BC-000BDBC434D9")
MKTEKDEBUGSTRING(DBTekUserUnixRO, "TekUserUnixRO")


/**
Instantizer.  SUInfo is the input.\n
\n
<b>WARNING!!!!!</b> All uchar_t's will be taken over by the TekUserUnixRO
instance, so make sure you create the uchar_t's with char2uchar()!
Failure to do so results in unpredictable results, especially since
ucharempty() is called!
*/
TekUserUnixRO::TekUserUnixRO()
{
	TEKSTD_IMPLEMENT_INTERFACE(UTekUser)
	TEKSTD_IMPLEMENT_INTERFACE(UTekUserUnixRO)
	TEKSTD_IMPLEMENT_ENDBASE
}

/**
Destructor.
*/
TekUserUnixRO::~TekUserUnixRO()
{
	if ( this->Priv_UserInfo.TekUserInfo_Description != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_Description);
	if ( this->Priv_UserInfo.TekUserInfo_Username != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_Username);
	if ( this->Priv_UserInfo.TekUserInfo_HomeDir != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_HomeDir);
}


/**
\fn TekUserUnixRO::put_TekUserInfo(struct TekUserInfo * SUInfo)

Sets a new set of user information into the instance.
*/
TEKERR TekUserUnixRO::put_TekUserInfo(struct TekUserInfo * SUInfo)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( SUInfo == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	memcpy(&this->Priv_UserInfo, SUInfo, sizeof(struct TekUserInfo));

	return TEKERR_OK;
}

/**
\fn TekUserUnixRO::get_Description(uchar_t ** RetVal)

Retrieves a description or a real name, of the user in question.
*/
TEKERR TekUserUnixRO::get_Description(uchar_t** RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Duplicate the uchar. */
	*RetVal = uchardup(this->Priv_UserInfo.TekUserInfo_Description);

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRO::get_GroupID(gid_t* RetVal)

Retrieves the primary Group ID of the user in question.
*/
TEKERR TekUserUnixRO::get_GroupID(gid_t* RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Set the group ID. */
	*RetVal = this->Priv_UserInfo.TekUserInfo_GroupID;

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRO::get_HomeDir(uchar_t ** RetVal)

Retrieves the home directory of the user in question.
*/
TEKERR TekUserUnixRO::get_HomeDir(uchar_t** RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Duplicate the uchar. */
	*RetVal = uchardup(this->Priv_UserInfo.TekUserInfo_HomeDir);

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRO::get_UserID(uid_t * RetVal)

Retrieves the User ID of the user in question.
*/
TEKERR TekUserUnixRO::get_UserID(uid_t* RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */
}

/**
\fn TekUserUnixRO::get_Username(uchar_t ** RetVal)

Retrieves the username of the user in question.
*/
TEKERR TekUserUnixRO::get_Username(uchar_t** RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Duplicate the uchar. */
	*RetVal = uchardup(this->Priv_UserInfo.TekUserInfo_Username);

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRO::put_Description(const uchar_t * NewDescription)

This function is not implemented.
*/
TEKERR TekUserUnixRO::put_Description(const uchar_t* NewDescription)
{
	return TEKERR_NOTIMPL;
}

/**
\fn TekUserUnixRO::put_GroupID(gid_t NewGroupID)

This function is not implemented.
*/
TEKERR TekUserUnixRO::put_GroupID(gid_t NewGroupID)

{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRO::put_HomeDir(const uchar_t* NewHomeDir)

This function is not implemented.
*/
TEKERR TekUserUnixRO::put_HomeDir(const uchar_t* NewHomeDir)

{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRO::put_Password(uchar_t* NewPassword)

This function is not implemented.
*/
TEKERR TekUserUnixRO::put_Password(uchar_t* NewPassword)

{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRO::put_UserID(uid_t NewID)

This function is not implemented.
*/
TEKERR TekUserUnixRO::put_UserID(uid_t NewID)

{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRO::put_Username(const uchar_t* NewUsername)

This function is not implemented.
*/
TEKERR TekUserUnixRO::put_Username(const uchar_t* NewUsername)
{
	return TEKERR_NOTIMPL;
}

/**
\fn TekUserUnixRO::Save()

This function is not implemented.
*/
TEKERR TekUserUnixRO::Save()
{
	return TEKERR_NOTIMPL;
}

