/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#define LIBTEKLTI_EXPORT (1)

#ifndef _cplusplus
#define _cplusplus
#endif /* _cplusplus */

#include "teklti.h"

MKTEKUUID(UTekUserUnixRW, "76184080-3E87-11DA-9E50-000BDBC434D9")
MKTEKDEBUGSTRING(DBTekUserUnixRW, "TekUserUnixRW")


/**
Instantizer.  SUInfo is the input.\n
\n
<b>WARNING!!!!!</b> All uchar_t's will be taken over by the TekUserUnixRW
instance, so make sure you create the uchar_t's with char2uchar()!
Failure to do so results in unpredictable results, especially since
ucharempty() is called!  This warning applies only to this instantizer.
*/
TekUserUnixRW::TekUserUnixRW()
{
	TEKSTD_IMPLEMENT_INTERFACE(UTekUser)
	TEKSTD_IMPLEMENT_INTERFACE(UTekUserUnixRW)
	TEKSTD_IMPLEMENT_ENDBASE

	memset(&this->Priv_UserInfo, '\0', sizeof(struct TekUserInfo));
}


/**
Destructor.
*/
TekUserUnixRW::~TekUserUnixRW()
{
	if ( this->Priv_UserInfo.TekUserInfo_Description != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_Description);
	if ( this->Priv_UserInfo.TekUserInfo_Username != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_Username);
	if ( this->Priv_UserInfo.TekUserInfo_HomeDir != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_HomeDir);
}


/**
\fn TekUserUnixRW::put_TekUserInfo(struct TekUserInfo * SUInfo)

Sets a new set of user information into the instance.
*/
TEKERR TekUserUnixRW::put_TekUserInfo(struct TekUserInfo * SUInfo)
{
	if ( SUInfo == NULL )
		memset(&this->Priv_UserInfo, '\0', sizeof(struct TekUserInfo));
	else
		memcpy(&this->Priv_UserInfo, SUInfo, sizeof(struct TekUserInfo));

	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::get_Description(uchar_t** RetVal)

Retrieves a description or a real name, of the user in question.
*/
TEKERR TekUserUnixRW::get_Description(uchar_t** RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Duplicate the uchar. */
	*RetVal = uchardup(this->Priv_UserInfo.TekUserInfo_Description);

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::get_GroupID(gid_t * RetVal)

Retrieves the primary Group ID of the user in question.
*/
TEKERR TekUserUnixRW::get_GroupID(gid_t * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Set the group ID. */
	*RetVal = this->Priv_UserInfo.TekUserInfo_GroupID;

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::get_HomeDir(uchar_t ** RetVal)

Retrieves the home directory of the user in question.
*/
TEKERR TekUserUnixRW::get_HomeDir(uchar_t ** RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Duplicate the uchar. */
	*RetVal = uchardup(this->Priv_UserInfo.TekUserInfo_HomeDir);

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::get_UserID(uid_t * RetVal)

Retrieves the User ID of the user in question.
*/
TEKERR TekUserUnixRW::get_UserID(uid_t * RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Set the group ID. */
	*RetVal = this->Priv_UserInfo.TekUserInfo_UserID;

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::get_Username(uchar_t ** RetVal)

Retrieves the username of the user in question.
*/
TEKERR TekUserUnixRW::get_Username(uchar_t ** RetVal)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Duplicate the uchar. */
	*RetVal = uchardup(this->Priv_UserInfo.TekUserInfo_Username);

	/* Return to caller. */
	return TEKERR_OK;
}


/**
\fn TekUserUnixRW::put_Description(const uchar_t * NewDescription)

Sets a new description of the user in question.  This should be
set to the real name of the person to make identification easier.
*/
TEKERR TekUserUnixRW::put_Description(const uchar_t * NewDescription)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( NewDescription == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Make sure our uchar_t is destroyed. */
	if ( this->Priv_UserInfo.TekUserInfo_Description != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_Description);

	/* Duplicate the user's uchar_t. */
	this->Priv_UserInfo.TekUserInfo_Description =
		uchardup(NewDescription);
	/* Test the result. */
	if ( this->Priv_UserInfo.TekUserInfo_Description == NULL )
		return TEKERR_MEMORY;

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::put_GroupID(gid_t NewGroupID)

Sets a new Group ID of the user in question.
*/
TEKERR TekUserUnixRW::put_GroupID(gid_t NewGroupID)
{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRW::put_HomeDir(const uchar_t * NewHomeDir)

Sets a new home directory of the user in question.
*/
TEKERR TekUserUnixRW::put_HomeDir(const uchar_t * NewHomeDir)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( NewHomeDir == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Make sure our uchar_t is destroyed. */
	if ( this->Priv_UserInfo.TekUserInfo_HomeDir != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_HomeDir);

	/* Duplicate the user's uchar_t. */
	this->Priv_UserInfo.TekUserInfo_HomeDir =
		uchardup(NewHomeDir);
	/* Test the result. */
	if ( this->Priv_UserInfo.TekUserInfo_HomeDir == NULL )
		return TEKERR_MEMORY;

	/* Return to caller. */
	return TEKERR_OK;
}


/**
\fn TekUserUnixRW::put_Password(uchar_t * NewPassword)

Sets a new password of the user in question.

<b>WARNING!</b>  This parameter will fry the entire uchar_t!
This is for privacy reasons!!!!!
*/
TEKERR TekUserUnixRW::put_Password(uchar_t * NewPassword)
{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRW::put_UserID(uid_t NewID)

Sets a new User ID of the user in question.
*/
TEKERR TekUserUnixRW::put_UserID(uid_t NewID)
{
	return TEKERR_NOTIMPL;
}


/**
\fn TekUserUnixRW::put_Username(const uchar_t * NewUsername)

Sets a new username of the user in question.
*/
TEKERR TekUserUnixRW::put_Username(const uchar_t * NewUsername)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( NewUsername == NULL )
		return TEKERR_POINTER;
#endif	/* NO_SIZE_T_CHECKS */

	/* Make sure our uchar_t is destroyed. */
	if ( this->Priv_UserInfo.TekUserInfo_Username != NULL )
		ucharempty(this->Priv_UserInfo.TekUserInfo_Username);

	/* Duplicate the user's uchar_t. */
	this->Priv_UserInfo.TekUserInfo_Username =
		uchardup(NewUsername);
	/* Test the result. */
	if ( this->Priv_UserInfo.TekUserInfo_Username == NULL )
		return TEKERR_MEMORY;

	/* Return to caller. */
	return TEKERR_OK;
}

/**
\fn TekUserUnixRW::Save()

Saves the user information into the provided instance of TekUserDB.
*/
TEKERR TekUserUnixRW::Save()
{
	TekUserDB * PrivDB;  /* Holds the DB instance pointer. */

	/* Set up the DB. */
	PrivDB = (TekUserDB *)this->Priv_UserInfo.TekUserInfo_DB;

#ifndef	NO_INTERNAL_NULL_CHECKS
	/* Check to see if the parameter was NULL. */
	if ( PrivDB == NULL )
		return TEKERR_FAIL;
#endif	/* NO_INTERNAL_NULL_CHECKS */

	/* Write outselves out. */
	PrivDB->WriteUser(this);

	/* Return to caller. */
	return TEKERR_OK;
}

