/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#define	LIBTEKLTI_EXPORT	(0)

#include "teklti.h"
#include <string.h>

#ifdef	DEBUG_USE_FERROR
#include <stdio.h>
#endif	/* DEBUG_USE_FERROR */


#ifdef	DEBUG_USE_FERROR
static const char _mkuuid_t_uuidstr_null[] = 
"POTENTIAL SEGFAULT in _mkuuid_t: uuid_str is NULL!\n";
static const char _mkuuid_t_uuidstr_overrun[] = 
"POTENTIAL OVERRUN in _mkuuid_t: uuid_str will cause a buffer overrun!\n";
#endif	/* DEBUG_USE_FERROR */


#ifndef	USE_386_ASM
TEKLTI_DECLSPEC void _mkuuid_t ( const char * uuid_str, uuid_t uuid_raw )
{	
#ifndef	NO_SIZE_T_CHECKS
	if ( uuid_str == NULL )
	{
#ifdef	DEBUG_USE_FERROR
		fputs(_mkuuid_t_uuidstr_null, stderr);
#endif	/* DEBUG_USE_FERROR */
		return;
	}
	if ( strlen(uuid_str) >= 40 * sizeof(char) )
	{
#ifdef	DEBUG_USE_FERROR
		fputs(_mkuuid_t_uuidstr_overrun, stderr);
#endif	/* DEBUG_USE_FERROR */
		return;
	}
#endif	/* not NO_SIZE_T_CHECKS */
	(void)uuid_parse(uuid_str, uuid_raw);

	/* Return to caller. */
	return;
}
#endif	/* not USE_386_ASM */


/*
NOTE: TekStdGlobalVariables will NEVER be programmed in assembly due to
its nature of grabbing global variables and passing them to the caller.
The caller can then determine, without advanced asm causing undesirable
problems, whether or not libTekLTI is compiled in a manner suitable
enough to the caller to run in a "smooth, yet safe" manner.
*/

TEKLTI_DECLSPEC TEKERR TekStdGetGlobalVariables ( 
	struct TekStdGlobalVars * RetVal
)
{
#ifndef	NO_SIZE_T_CHECKS
	/* Check for a valid RetVal. */
	if ( RetVal == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	/* Make sure size is set enough for us to work with. */
	if ( RetVal->TekStdGlobalVarsSize < sizeof(struct TekStdGlobalVars) )
		return TEKERR_UNAVAILABLE;

	/* Reduce GlobalVarsSize as needed. */
	if ( RetVal->TekStdGlobalVarsSize > sizeof(struct TekStdGlobalVars) )
		RetVal->TekStdGlobalVarsSize = sizeof(struct TekStdGlobalVars);

	/* Reduce VersionSize as needed. */
	if ( RetVal->TekStdGlobalVarsVersion.TekStdVerSize
	     > sizeof(struct TekStdVersion) )
	{
		RetVal->TekStdGlobalVarsVersion.TekStdVerSize = 
			sizeof(struct TekStdVersion);
	}

	/* Set up the versioning structures. */
	RetVal->TekStdGlobalVarsVersion.TekStdVerMajor = TEKLTI_VER_MAJOR;
	RetVal->TekStdGlobalVarsVersion.TekStdVerMinor = TEKLTI_VER_MINOR;
	RetVal->TekStdGlobalVarsVersion.TekStdVerRev = TEKLTI_VER_REVISION;
	RetVal->TekStdGlobalVarsVersion.TekStdVerPatchNo = TEKLTI_VER_PATCHNO;
	strcpy(
		RetVal->TekStdGlobalVarsVersion.TekStdVerMisc,
		TEKLTI_VER_MISCINFO
	);

	/* Now set up the maximum implementations variable. */
	RetVal->TekStdGlobalVarsMaxImpl = TEKSTD_MAXIMPL;
	
	/* Set up the Unicode variable. */
#ifdef	UNICODE
	RetVal->TekStdGlobalVarsUnicode = 1;
#else	/* not UNICODE */
	RetVal->TekStdGlobalVarsUnicode = 0;
#endif	/* not UNICODE */

#ifdef	USE_386_ASM_MMX
	RetVal->TekStdGlobalVarsMMX = 1;
#else	/* not USE_386_ASM_MMX */
	RetVal->TekStdGlobalVarsMMX = 0;
#endif	/* not USE_386_ASM_MMX */

#ifdef	USE_386_ASM_MMX_RESET_MM7
	RetVal->TekStdGlobalVarsMMXResetMM7 = 0;
#else	/* not USE_386_ASM_MMX_RESET_MM7 */
	RetVal->TekStdGlobalVarsMMXResetMM7 = 1;
#endif	/* not USE_386_ASM_MMX_RESET_MM7 */

	/* Set up the internal checks variable. */
#ifdef	NO_INTERNAL_NULL_CHECKS
	RetVal->TekStdGlobalVarsInternalChecks = 0;
#else	/* not NO_INTERNAL_NULL_CHECKS */
	RetVal->TekStdGlobalVarsInternalChecks = 1;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Set up the pass in pointer checks variable. */
#ifdef	NO_SIZE_T_CHECKS	
	RetVal->TekStdGlobalVarsPassInPointerChecks = 0;
#else	/* not NO_SIZE_T_CHECKS */
	RetVal->TekStdGlobalVarsPassInPointerChecks = 1;
#endif	/* not NO_SIZE_T_CHECKS */

	/* Set up the push pop enforced variable. */
#ifdef	USE_386_ASM_PUSHPOP
	RetVal->TekStdGlobalVarsPushPopEnforced = 1;
#else	/* not USE_386_ASM_PUSHPOP */
	RetVal->TekStdGlobalVarsPushPopEnforced = 0;
#endif	/* not USE_386_ASM_PUSHPOP */

#ifdef	USE_386_ASM_ENFORCE_PRIVACY
	RetVal->TekStdGlobalVarsPrivacyEnforced = 1;
#else	/* not USE_386_ASM_ENFORCE_PRIVACY */
	RetVal->TekStdGlobalVarsPrivacyEnforced = 0;
#endif	/* not USE_386_ASM_ENFORCE_PRIVACY */

	return TEKERR_OK;
}


TEKLTI_DECLSPEC TEKERR TekStdCreateInstance (
	CPTekStdCIInput InterfaceStructs,
	size_t InterfaceStructCount,
	void ** InstanceArray,
	size_t InstanceArrayCount,
	const uuid_t * UUIDArray,
	size_t UUIDArrayCount,
	unsigned int Flags,
	PTekStdVersion VersionStruct
)
{
	return TEKERR_NOTIMPL;
}


TEKLTI_DECLSPEC TEKERR TekStdGetLibraryVersion (
	PTekStdVersion * VersionStruct,
	size_t VersionStructCount
)
{
	return TEKERR_NOTIMPL;
}

/*int main()
{
	unsigned char * tmp;
	char chrout[64];
	uuid_t uuidmake;
	int i;

	uuid_generate_time(uuidmake);
	tmp = uuidmake;
	for ( i=0; i < 16; i++ )
		printf("0x%.2X ", (unsigned int)tmp[i]);
	printf("\n");

	uuid_unparse(uuidmake, chrout);

	printf("%s\n", chrout);
}*/


