/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TEKFILE_H_AF7D74D2_DB02_11D9_9782_000BDBC434D9
#define TEKFILE_H_AF7D74D2_DB02_11D9_9782_000BDBC434D9

#ifndef	TEKLTI_H_C35A2A4C_D475_11D9_90BC_000BDBC434D9
#error	This file must be included from <teklti.h>
#endif	/* TEKLTI_H_C35A2A4C_D475_11D9_90BC_000BDBC434D9 */


TEKUUIDIMPORT(UTekFile);


enum TekFileType
{
	TekFileType_Binary = 0x0000,
	TekFileType_ASCII = 0x0001,
	TekFileType_ConfigASCII = 0x0002,
	TekFileType_UNICODE = 0x0004,
	TekFileType_ConfigUNICODE = 0x0008,
	TekFileType_BinaryPipe = 0x0010,
	TekFileType_ASCIIPipe = 0x0020,
	TekFileType_UnicodePipe = 0x0040,
	TekFileType_BZIP2 = 0x0100,
	TekFileType_BZIP2_ASCII = 0x0200
	
};

#define TekFileType_UserDefined	TekFileType_Binary

enum TekFileMode
{
	TekFileMode_ReadOnly = 0x0000,
	TekFileMode_WriteOnly = 0x0001,
	TekFileMode_Append = 0x0002,
	TekFileMode_ReadWrite = 0x0004,
	TekFileMode_ReadWriteTrunc = 0x0008,
	TekFileMode_ReadAppend = 0x0010
};

/*
For those compiling on a platform utilizing Intel processors:

WARNING!!!  If you decide to change values starting with
TekFileSeek (in any case), TEKFILESEEK_ must contain a
numeric value, and TekFileSeek_ must contain the equivalent
value in hexadecimal format.  The reason for this is because
these values are also used in preprocessing #if...#endif
statements to further optimize the assembly code, not just
the ordinary code.

ex.: If TEKFILESEEK_END = 2, then TekFileSeek_End = 0x0002
*/

#define	TEKFILESEEK_BEGIN	0
#define	TEKFILESEEK_CURRENT	1
#define	TEKFILESEEK_END 	2

enum TekFileSeek
{
	TekFileSeek_Begin = 0x0000,
	TekFileSeek_Current = 0x0001,
	TekFileSeek_End = 0x0002
};

/*
   The below external variables are inserted for the
   sole purpose of satisfying the GNU Assembler.
*/

#ifdef	USE_386_ASM
extern const char TekFileBinary_ReadOnly[];
extern const char TekFileBinary_WriteOnly[];
extern const char TekFileBinary_Append[];
extern const char TekFileBinary_ReadWrite[];
extern const char TekFileBinary_ReadWriteTrunc[];
extern const char TekFileBinary_ReadAppend[];
#endif	/* USE_386_ASM */



/**
This interface defines the standard routines for any interface performing I/O 
on files, pipes, and sockets.

This interface is <b>INHERITABLE</b>.

@author Kevin McBride
*/
class TekFile : public TekStd
{
public:
/**
Instantizer.
*/
TekFile();

/**
Destructor.
*/
~TekFile();

#ifndef	TEKFILE_INTERNAL
virtual TEKERR get_FILE(FILE ** FileDescriptor) = 0;
virtual TEKERR get_Filename(uchar_t ** Name) = 0;
virtual TEKERR get_Mode(TekFileMode * ModeID) = 0;
virtual TEKERR get_Type(TekFileType * FileTypePointer) = 0;
virtual TEKERR put_FILE(FILE * FileDescriptor) = 0;
virtual TEKERR put_Filename(uchar_t * Name) = 0;
virtual TEKERR put_Mode(TekFileMode ModeID) = 0;

virtual TEKERR Close() = 0;
virtual TEKERR Flush() = 0;
virtual TEKERR Open() = 0;
virtual TEKERR Open64() = 0;
virtual TEKERR Read64(
	void * Buffer,
	uint64_t BytesToRead,
	uint64_t * BytesRead
) = 0;
virtual TEKERR Read(
	void * Buffer,
	unsigned long BytesToRead,
	unsigned long * BytesRead
) = 0;
virtual TEKERR Seek64(TekFileSeek SeekWhere, int64_t NewLocation) = 0;
virtual TEKERR Seek(TekFileSeek SeekWhere, signed long NewLocation) = 0;
virtual TEKERR Write64(
	void * Buffer,
	uint64_t BytesToWrite,
	uint64_t * BytesWrote
) = 0;
virtual TEKERR Write(
	void * Buffer,
	unsigned long BytesToWrite,
	unsigned long * BytesWrote
) = 0;
virtual TEKERR get_Location(unsigned long * Location) = 0;
virtual TEKERR get_Location64(uint64_t * Location) = 0;
#endif	/* not TEKFILE_INTERNAL */

/**
This macro contains the generic implementation for TekStd.\n
It is much easier to use as opposed to creating the\n
implementation by hand.
*/
//TEKSTD_HEADER_VARSANDFUNC(TekFile);
TEKSTD_HEADER_VARS;
};


/*
    \fn TEKLTI_DECLSPEC TEKERR TekFileCreate(TekFileType FileType, TekFile ** RetVal, uchar_t * FileName = (uchar_t *)NULL)
    
    Creates an instance to an appropriate TekFileType.
 */

TEKLTI_DECLSPEC TEKERR TekFileCreate(TekFileType FileType, TekFile ** RetVal, uchar_t * FileName = (uchar_t *)NULL);

#endif	/* not TEKFILE_H_AF7D74D2_DB02_11D9_9782_000BDBC434D9 */


