/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TEKGLOBALVARS_H
#define TEKGLOBALVARS_H


#error This file can not be included in any project.


/**
Contains definitions for the Global Functions of libTekLTI.\n
\n
<b>NOTE</b>: These functions are global functions, and as such, are not
called from interfacing with nor creating instances of TekGlobalVars. As a
result, this class serves simply to force doxygen to document the global
functions in libTekLTI.

@author Kevin McBride
*/
class TekGlobalVars{
public:
    TekGlobalVars();

/*!
    \fn TekGlobalVars::TekFileCreate(TekFileType FileType, TekFile ** RetVal, uchar_t * FileName = (uchar_t *)NULL);
    
    \n
    This function creates a new instance to an interface specified by
    FileType.\n
    \n
    <b>FileType</b> - Identifies the type of interface to create a new instance out of.\n
    \n
    <b>RetVal</b> - Pointer to a TekFile pointer to return the result in.\n
    \n
    <b>FileName: </b><i>(Optional, default = NULL)</i> Pointer to a FileName to
    associate with the new TekFile instance.  If NULL, no Filename will be associated
    with the new instance.
 */
    ~TekGlobalVars();
    TEKLTI_DECLSPEC TEKERR TekFileCreate(
    	TekFileType FileType,
	TekFile ** RetVal,
	uchar_t * FileName = (uchar_t *)NULL
    );

/*!
    \fn TekGlobalVars::TekUserInfoCreate(struct TekUserInfo * UserInfo, TekUser ** RetVal, TekUserType UserType);
    
    \n
    This function creates a new instance to an interface specified by
    the UserType enumeration.\n
    \n
    <b>UserInfo</b> - The structure of user information to pass in to the new
    instance. \n
    \n
    <b>RetVal</b> - Pointer to a TekFile pointer to return the result in.\n
    \n
    <b>UserType: </b><i>(Optional, default = TekUserInfo_Any)</i> - The TekUser
    interface from the TekUserType enumeration to make an instance out of.
*/
    TEKLTI_DECLSPEC TEKERR TekUserInfoCreate(
	struct TekUserInfo * UserInfo,
	TekUser ** RetVal,
	TekUserType UserType = TekUserType_Any
    );

/*!
\fn TekGlobalVars::TekStdCheckAndRecieveProcessorInfoIntel(struct TekStdProcessorInfoIntel * ProcessorInfoArray,
    size_t ProcessorInfoArrayCount,
    TEKSTDPROCESSORINFO_FAIL_CALLBACK * CallbackFunction)
    
    \n
    This function gets the processor info of the
    Intel processor.  This function is compiled in libTekLTI only when
    TEKLTI_USE_386_ASM is defined during compilation. \n
    \n
    <b>ProcessorInfoArray</b> - Arrays to fill in with processor info.\n
    \n
    <b>ProcessorInfoArrayCount</b> - Array length of ProcessorInfoArray \n
    \n
    <b>CallbackFunction</b> - Function to call to determine whether or not
    to call exit().  If NULL, and an error occurs, exit() will be called.\n
    The format of the CallbackFunction is:
    <b>int</b> func(<b>void</b>)
  */
        TEKERR TekStdCheckAndRecieveProcessorInfoIntel(struct TekStdProcessorInfoIntel * ProcessorInfoArray,
	size_t ProcessorInfoArrayCount,
	TEKSTDPROCESSORINFO_FAIL_CALLBACK * CallbackFunction);

/*!
    \n
    <b><i>This function is still under construction.</i></b>
*/
TEKLTI_DECLSPEC TEKERR TekStdCreateInstance (
	CPTekStdCIInput InterfaceStructs,
	size_t InterfaceStructCount,
	void ** InstanceArray,
	size_t InstanceArrayCount,
	const uuid_t * UUIDArray,
	size_t UUIDArrayCount,
	unsigned int Flags,
	PTekStdVersion VersionStruct
);

/*!
    \fn TekGlobalVars::TekStdGetGlobalVariables(struct TekStdGlobalVars * RetVal)

    \n
    This function obtains the global variables set in libTekLTI.  You can use
    the variables to determine whether or not the library can be used by your
    program. \n
    \n
    <b>RetVal</b> - Pointer to a structure to fill with the global variables.
*/

TEKLTI_DECLSPEC TEKERR TekStdGetGlobalVariables ( 
	struct TekStdGlobalVars * RetVal
);

/*!
    \n
    <b><i>This function is still under construction.</i></b>
*/

TEKLTI_DECLSPEC TEKERR TekStdGetLibraryVersion (
	PTekStdVersion * VersionStruct,
	size_t VersionStructCount
)
{
	return TEKERR_NOTIMPL;
}

/*!
    \fn TekGlobalVars::char2uchar ( const char * src )

    \n
    This function converts an ASCII string into a uchar_t. \n
    \n
    <b>src</b> - Pointer to an ASCII string.\n
    \n
    Return value is uchar_t; will be NULL on failure.
*/

TEKLTI_DECLSPEC uchar_t * char2uchar ( const char * src );

/*!
    \fn TekGlobalVars::char2ucharlen ( const char * src, size_t len )

    \n
    This function converts len characters of an ASCII string into a uchar_t.\n
    \n
    <b>src</b> - Pointer to an ASCII string.\n
    \n
    <b>len</b> - Length of the ASCII string in sizeof(char)'s.\n
    \n
    Return value is uchar_t; will be NULL on failure.
*/

TEKLTI_DECLSPEC uchar_t * char2ucharlen ( const char * src, size_t len );

/*!
    \fn TekGlobalVars::uchardup ( const uchar_t * src )

    \n
    This function duplicates a uchar_t. \n
    \n
    <b>src</b> - Pointer to a uchar_t.\n
    \n
    Return value is uchar_t; will be NULL on failure.
*/

TEKLTI_DECLSPEC uchar_t * uchardup ( const uchar_t * src );

/*!
    \fn TekGlobalVars::ucharempty ( uchar_t * src )

    \n
    This function frees a uchar_t from memory. \n
    \n
    <b>uchrtofree</b> - Pointer to a uchar_t to free from memory.\n
    \n
    This function returns <b>nothing.</b>\n
    \n
    Under debugging scenarios, if uchrtofree == NULL, a SIGTRAP can be raised.
*/

TEKLTI_DECLSPEC void ucharempty ( uchar_t * uchrtofree );

/**
\fn TekGlobalVars::uchar2char ( const uchar_t * uchr )

This macro retrieves the C-style parameter of a uchar_t.
*/
TEKLTI_DECLSPEC char * uchar2char ( const uchar_t * uchr );

/**
\fn TekGlobalVars::mkucharnull ( uchar_t * uchr, uint32_t extradw )

This macro fries a uchar_t structure.\n
\n
extradw is currently unused. \n
\n
<b>CAUTION!!!!!</b> This should only be used in situations where you
are making the uchar_t without the help of the functions on this page!
*/
TEKLTI_DECLSPEC void mkucharnull ( uchar_t * uchr, uint32_t extradw );

};

#endif


