/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "testsuite.h"


int runtekstdtests ( struct sigaction * oldactsegv, struct sigaction * oldactill );
int runtekfiletests ( void );
int runtekfileasciitests ( void );

#ifdef	TEKLTI_USE_BZIP2
int runtekfilebz2asciitests ( void );
int runtekfilebz2binarytests ( void );
#endif	/* TEKLTI_USE_BZIP2 */

int checkbz2 ( void );


int checkbz2()
{
#ifdef	TEKLTI_USE_BZIP2
	if ( runtekfilebz2binarytests() != EXIT_SUCCESS )
		return EXIT_FAILURE;

	return runtekfilebz2asciitests();
#else	/* not TEKLTI_USE_BZIP2 */
	return 0;
#endif	/* TEKLTI_USE_BZIP2 */
}



int main(int argc, char *argv[])
{
	struct sigaction newact, oldactsegv, oldactill;
	sigset_t signos;
	uchar_t * uchr;
	uchar_t * duchr;
	int i;

	/* Set up our signal handlers for the worst. */
	sigemptyset(&signos);
	sigaddset(&signos, SIGSEGV);
	newact.sa_handler = &uchar_sigsegv;
	newact.sa_mask = signos;
	newact.sa_flags = SA_RESTART;
	sigaction(SIGSEGV, &newact, &oldactsegv);
	sigaction(SIGILL, &newact, &oldactill);

	/* Check the processor. */
	checkprocessor();

	/*struct i64 s64;
	i = 0x80808080;
	__asm__ ( " movd %0, %%mm6 " : "=m" ( i ) );
	s64.i64_a = 0x00000000;
	s64.i64_b = 0x74747474;
	__asm__ ( " movq %0, %%mm5 " : "=m" ( s64 ) );
	__asm__ ( " paddusw %mm6, %mm5 " );
	__asm__ ( " movq %%mm5, %0 " : "=m" ( s64 ) );
	__asm__ ( " emms " );*/

	/* Make sure uchar_t processing works. */
	uchr = char2uchar("This is a test");
	if ( strcmp("This is a test", uchar2char(uchr)) )
	{
		printucharerror();
		return EXIT_FAILURE;
	}
	ucharempty(uchr);
	uchr = char2ucharlen("This is a test", 14);
	if ( strcmp("This is a test", uchar2char(uchr)) )
	{
		printucharerror();
		return EXIT_FAILURE;
	}
	duchr = uchardup(uchr);
	if ( strcmp("This is a test", uchar2char(duchr)) )
	{
		printucharerror();
		return EXIT_FAILURE;
	}
	ucharempty(uchr);


	/* We're in: begin the testsuite. */

	fprintf(stdout, "Welcome to the libTekLTI Testsuite.\n\n");

	/* Run the TekStd tests. */
	i = runtekstdtests(&oldactsegv, &oldactill);
	if ( i != EXIT_SUCCESS )
		return EXIT_FAILURE;

	/* Run the TekFile/TekFileBinary tests. */
	i = runtekfiletests();
	if ( i != EXIT_SUCCESS )
		return EXIT_FAILURE;

	/* Run the TekFileASCII/TekFileASCIIBare tests. */
	i = runtekfileasciitests();
	if ( i != EXIT_SUCCESS )
		return EXIT_FAILURE;

	i = checkbz2();
	if ( i != EXIT_SUCCESS )
		return EXIT_FAILURE;

	/* Print out the ending message. */
	fprintf(stdout, "End of libTekLTI Testsuite.\n");

	/* Sleep for a few seconds. */
	sleep(5);

	/* That's all folks. */
	return EXIT_SUCCESS;
}

