/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef TEKUSERDB_H_8E4AC624_3D1E_11DA_83AF_000BDBC434D9
#define TEKUSERDB_H_8E4AC624_3D1E_11DA_83AF_000BDBC434D9

#ifndef	TEKLTI_H_C35A2A4C_D475_11D9_90BC_000BDBC434D9
#error	This file must be included from <teklti.h>
#endif	/* TEKLTI_H_C35A2A4C_D475_11D9_90BC_000BDBC434D9 */


TEKUUIDIMPORT(UTekUserDB);


typedef int(TekUserDBFunc)(TekUser * TekUserEnum);


/**
This provides the generic interface for accessing a system's user database.

This interface is <b>INHERITABLE</b>.

@author Kevin McBride
*/
class TekUserDB : public TekEnum
{
public:
    TekUserDB();
    ~TekUserDB();

    virtual TEKERR Clone(TekEnum ** RetVal) = 0;
    virtual TEKERR Back(void ** RetVal) = 0;
    virtual TEKERR EnumNo(unsigned long ItemNo) = 0;
    virtual TEKERR Next(void ** RetVal) = 0;
    virtual TEKERR Reset() = 0;

    virtual TEKERR Open() = 0;
    virtual TEKERR Close() = 0;
    virtual TEKERR Flush() = 0;
    virtual TEKERR OpenAndLoadAll() = 0;
    virtual TEKERR FindUser(const uchar_t * Username, TekUser * RetVal) = 0;
    virtual TEKERR FindUsers(
        const uchar_t * SearchString,
	TekUserDBFunc * FuncToCall,
	struct TekArray ** RetVal = NULL
    ) = 0;
    virtual TEKERR FindUsersFree(TekUser ** UserInfoArray) = 0;
    virtual TEKERR WriteUsers(TekUser ** UserInfoArray, unsigned long Count) = 0;
    virtual TEKERR WriteUser(TekUser * UserInfo) = 0;
    virtual TEKERR DeleteUser(TekUser * UserInfo) = 0;
    virtual TEKERR DeleteUsers(
        const uchar_t * SearchString,
	TekUserDBFunc * FuncToCall,
	struct TekArray ** RetVal = NULL
    ) = 0;

    virtual TEKERR put_Filename(const uchar_t * NewFilename) = 0;
    virtual TEKERR get_Filename(uchar_t ** Filename) = 0;

    virtual TEKERR get_ReadOnly(bool * ReadOnlyStat) = 0;

    virtual TEKERR get_FileInterfaceUUID(uuid_t * FileUUID) = 0;
    virtual TEKERR put_FileInterfaceUUID(const uuid_t * FileUUID) = 0;
    virtual TEKERR get_GenericFileType(TekFileType * CurrentFileType) = 0;
    virtual TEKERR put_GenericFileType(TekFileType CurrentFileType) = 0;
};


TEKLTI_DECLSPEC TEKERR TekOpenUserDB(TekUserDB ** RetVal);

#endif	/* TEKUSERDB_H_8E4AC624_3D1E_11DA_83AF_000BDBC434D9 */

