/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "testsuite.h"



/* Generics. */
const char tekerr_ok[] = "TEKERR_OK";
const char tekerr_notimpl[] = "TEKERR_NOTIMPL";
const char tekerr_fail[] = "TEKERR_FAIL";
const char tekerr_unexpected[] = "TEKERR_UNEXPECTED";
const char tekerr_memory[] = "TEKERR_MEMORY";
const char tekerr_access[] = "TEKERR_ACCESS";
const char tekerr_pointer[] = "TEKERR_POINTER";
const char tekerr_unavailable[] = "TEKERR_UNAVAILABLE";
const char tekerr_sequence[] = "TEKERR_SEQUENCE";

/* DB Errors. */
const char tekerr_db_cantconnect[] = "TEKERR_DB_CANTCONNECT";
const char tekerr_db_accessdenied[] = "TEKERR_DB_ACCESSDENIED";
const char tekerr_db_notallowed[] = "TEKERR_DB_NOTALLOWED";
const char tekerr_db_corrupt[] = "TEKERR_DB_CORRUPT";

/* File/Socket Errors. */
const char tekerr_file_diskfull[] = "TEKERR_FILE_DISKFULL";
const char tekerr_file_readonly[] = "TEKERR_FILE_READONLY / TEKERR_FILE_EOF";
const char tekerr_file_io[] = "TEKERR_FILE_IO";
const char tekerr_file_pipe[] = "TEKERR_FILE_PIPE";



const char * gettekerrstr ( TEKERR ErrorCode )
{
	switch ( ErrorCode )
	{
		case TEKERR_OK:
			return tekerr_ok;
		case TEKERR_FAIL:
			return tekerr_fail;
		case TEKERR_UNEXPECTED:
			return tekerr_unexpected;
		case TEKERR_MEMORY:
			return tekerr_memory;
		case TEKERR_ACCESS:
			return tekerr_access;
		case TEKERR_POINTER:
			return tekerr_pointer;
		case TEKERR_UNAVAILABLE:
			return tekerr_unavailable;
		case TEKERR_SEQUENCE:
			return tekerr_sequence;
		case TEKERR_DB_CANTCONNECT:
			return tekerr_db_cantconnect;
		case TEKERR_DB_ACCESSDENIED:
			return tekerr_db_accessdenied;
		case TEKERR_DB_NOTALLOWED:
			return tekerr_db_notallowed;
		case TEKERR_DB_CORRUPT:
			return tekerr_db_corrupt;
		case TEKERR_FILE_DISKFULL:
			return tekerr_file_diskfull;
		case TEKERR_FILE_READONLY:
			return tekerr_file_readonly;
		case TEKERR_FILE_IO:
			return tekerr_file_io;
		case TEKERR_FILE_PIPE:
			return tekerr_file_pipe;
		case TEKERR_NOTIMPL:
		default:
			return tekerr_notimpl;
	}
}



void uchar_sigsegv ( int signum )
{
	if ( signum == SIGSEGV )
	{
		fprintf(stderr, "ERROR: Segmentation fault in uchar functions.\n");
	} else {
		fprintf(stderr, "ERROR: Your processor complained about an instruction being\n");
		fprintf(stderr, "ran illegal in the uchar processing functions.\n");
	}
	printucharerror();
	exit(EXIT_FAILURE);
}

void verstructfatal ( void )
{
	fprintf(stderr, "\nERROR: TekStdGetGlobalVariables() failed!!\n");
}

void incorrectverfatal ( void )
{
	fprintf(stderr, "\nERROR: Incorrect version of libTekLTI installed!!\n");
}

void implfatal ( void )
{
	fprintf(stderr, "\nERROR: Maximum Implementations is less than 10!\n");
	fprintf(stderr, "Re-compilation of libTekLTI is required.\n\n");
	fprintf(stdout, "Be sure to change the TEKSTD_MAXIMPL macro to something\n");
	fprintf(stdout, "greater than or equal to 10 (ten).\n\n");
}

unsigned int checkverstruct ( void )
{
	struct TekStdGlobalVars tekstdgvar;
	struct TekStdGlobalVars * tekstdvvar;
	TEKERR res;

	/* Set up the pointer. */
	tekstdvvar = &tekstdgvar;

	/* Clear out tekstdgvar. */
	memset(tekstdvvar, '\0', sizeof(struct TekStdGlobalVars));

	tekstdgvar.TekStdGlobalVarsSize = sizeof(struct TekStdGlobalVars);

	res = TekStdGetGlobalVariables(&tekstdgvar);
	if ( res != TEKERR_OK )
		return 1;

	if ( !TEKLTI_VER_MATCH(tekstdvvar, 1, 0, 0, 0) )
 		return 2;

	if ( tekstdvvar->TekStdGlobalVarsMaxImpl < 10 )
	{
		return 3;
	}

	return 0;
}

void printucharerror()
{
fprintf(stderr, "Unexpected failure in uchar functions.\n");
fprintf(stdout, "TekLTI Testsuite cannot run if uchar_t's can not be produced.\n");
#ifdef	USE_386_ASM
fprintf(stdout, "You may want to disable USE_386_ASM and recompile libTekLTI\n");
#endif	/* USE_386_ASM */
return;
}


int processorcheckfail ( void )
{
	fprintf(stderr, "Your processor has failed the CPUID feature check.\n\n");
	fprintf(stdout, "libTekLTI Testsuite can not run with this error.\n\n");

	return 1;
}

void checkprocessor ( void )
{
#ifdef	TEKLTI_USE_386_ASM
	struct TekStdProcessorInfoIntel tekgen[3];
	TEKERR tekerr;

	tekerr = TekStdCheckAndRecieveProcessorInfoIntel(tekgen, 3, &processorcheckfail);
	if ( tekerr != TEKERR_OK )
		exit(EXIT_FAILURE);
#endif	/* TEKLTI_USE_386_ASM */

	return;
}


