/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "testsuite.h"


#define	TESTFILE_TESTSUITE	"./test.txt"


int runtekfiletests ( void )
{
	bool bi[NUM_OF_TEKFILEBINARY_HUNKS];
	TekFileMode tekmode;
	TekFileType tektype;
	uchar_t * uchrt;
	uuid_t uuidval;
	FILE * fdesc;
	TEKERR hres;
	unsigned long lt;
	unsigned short st;
	unsigned long l;
	char cbuf[7];
	TekFileBinary * tekbin;
	TekFile * tekfil;

	unlink(TESTFILE_TESTSUITE);

	/* NULLify the bi array first. */
	for ( l=0; l != NUM_OF_TEKFILEBINARY_HUNKS; l++ )
		bi[l] = false;

	/* Tell the user we are running some tests. */
	fprintf(stdout, "Running test: TekFileBinary tests.\n");
	fprintf(stdout, "Number of hunks to test: %d\n\n", NUM_OF_TEKFILEBINARY_HUNKS);

	/* -------------------- */
	/* --- HUNK 0 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(0, "Create an instance of TekFileBinary")

	/* NULLify tekbin. */
	tekbin = NULL;
	/* Run TekFileCreate(). */
	hres = TekFileCreate(TekFileType_Binary, (TekFile **)&tekbin);
	/* Check the result. */
	if ( hres == TEKERR_OK )
	{
		/* Make sure tekbin is a valid pointer. */
		if ( tekbin == NULL )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFileCreate() did not give valid pointer on retval of TEKERR_OK.\n"
			);
			goto TekFileBinaryHunk0Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 0)
	}
	else
	{
TekFileBinaryHunk0Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 0, hres)
		fprintf(stderr, "Can not produce instances of TekFileBinary!!!!!\n");
		goto TekFileBinaryFatal;
	}


	/* -------------------- */
	/* --- HUNK 1 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(1, "Check that the instance is derived from TekFile")

	/* Get the binary UUID of UTekFile. */
	GETTEKUUID(UTekFile, uuidval);
	/* NULLify tekfil. */
	tekfil = NULL;
	/* Run the test; call CheckFor(). */
	hres = tekbin->CheckFor(&uuidval, 1, 1, (void **)&tekfil);
	/* Check the result. */
	if ( hres == TEKERR_OK )
	{
		/* Make sure tekfil is a valid pointer. */
		if ( tekfil == NULL )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekStd::CheckFor() did not give valid pointer on retval of TEKERR_OK.\n"
			);
			goto TekFileBinaryHunk1Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 1)
	}
	else
	{
TekFileBinaryHunk1Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 1, hres)
		fprintf(stderr, "Can not produce instances of TekFileBinary!!!!!\n");
		goto TekFileBinaryFatal;
	}


	/* -------------------- */
	/* --- HUNK 2 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(2, "Check that get_Type currently reports TekFileType_Binary")

	/* Get the typeof tekfil. */
	hres = tekfil->get_Type(&tektype);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Check the tektype. */
		if ( tektype != TekFileType_Binary )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Type() returned value %d instead of %d to &tektype.\n",
			  tektype,
			  TekFileType_Binary
			);
			goto TekFileBinaryHunk2Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 2)
	}
	else
	{
TekFileBinaryHunk2Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 2, hres)
	}


	/* -------------------- */
	/* --- HUNK 3 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(3, "Check that put_FILE and get_FILE works")

	/* Set FILE param to stderr. */
	hres = tekfil->put_FILE(stderr);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Check the FILE param. */
		hres = tekfil->get_FILE(&fdesc);
		if ( hres != TEKERR_OK )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_FILE() failed!.\n"
			);
			goto TekFileBinaryHunk3Fail;
		}

		if ( fdesc != stderr )
		{
			/* The parameter has invalid information; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_FILE() returned value other than stderr.\n"
			);
			goto TekFileBinaryHunk3Fail;
		}

		hres = tekfil->put_FILE(NULL);
		if ( hres != TEKERR_OK )
		{
			/* The parameter has invalid information; do fatal cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::put_FILE() unexpectedly failed.\n"
			);
			PRINTFAIL(bi, 3, hres)
			goto TekFileBinaryFatal;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 3)
	}
	else
	{
TekFileBinaryHunk3Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 3, hres)
	}


	/* -------------------- */
	/* --- HUNK 4 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(4, "Check that get_Filename and put_Filename works")

	/* Make a uchar out of TESTFILE_TESTSUITE. */
	uchrt = char2uchar(TESTFILE_TESTSUITE);
	if ( uchrt == NULL )
		goto TekFileBinaryHunk4Fail;

	/* Set Filename param to stderr. */
	hres = tekfil->put_Filename(uchrt);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Free the uchar. */
		ucharempty(uchrt);
		uchrt = NULL;

		/* Check the Filename param. */
		hres = tekfil->get_Filename(&uchrt);
		if ( hres != TEKERR_OK )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Filename() failed!.\n"
			);
			goto TekFileBinaryHunk4Fail;
		}

		if ( uchrt == NULL )
		{
			/* The parameter was NULL; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Filename() returned NULL.\n"
			);
			goto TekFileBinaryHunk4Fail;
		}

		if ( strcmp(uchar2char(uchrt), TESTFILE_TESTSUITE) != 0 )
		{
			/* The parameter has invalid information; go to cleanup. */
			ucharempty(uchrt);
			fprintf(stderr, 
			  "XFAIL: uchrt from TekFile::get_Filename() does not match provided filename: %s.\n",
			  TESTFILE_TESTSUITE
			);
			goto TekFileBinaryHunk4Fail;
		}

		/* It is valid; hunk passed. */
		ucharempty(uchrt);
		PRINTPASS(bi, 4)
	}
	else
	{
		/* Empty out the uchrt. */
		ucharempty(uchrt);

TekFileBinaryHunk4Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 4, hres)
	}

	/*tekmode = TekFileMode_ReadAppend;
	hres = tekfil->put_Mode(tekmode);
	tekmode = TekFileMode_ReadOnly;
	hres = tekfil->get_Mode(&tekmode);*/

	/* -------------------- */
	/* --- HUNK 5 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(5, "Check that get_Mode and put_Mode works")

	/* Set tekmode to TekFileMode_Append. */
	hres = tekfil->put_Mode(TekFileMode_WriteOnly);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Check the Mode param. */
		hres = tekfil->get_Mode(&tekmode);
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Mode() failed!.\n"
			);
			goto TekFileBinaryHunk5Fail;
		}

		if ( tekmode != TekFileMode_WriteOnly )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Mode() returned incorrect mode.\n"
			);
			goto TekFileBinaryHunk5Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 5)
	}
	else
	{
TekFileBinaryHunk5Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 5, hres)
	}


	/* -------------------- */
	/* --- HUNK 6 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(6, "Open the test file in TekFileMode_WriteOnly mode")

	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 6)
	}
	else
	{
		PRINTFAIL(bi, 6, hres)
		goto TekFileBinaryFatal;
	}


	/* -------------------- */
	/* --- HUNK 7 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(7, "Close the file")

	hres = tekfil->Close();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 7)
	}
	else
	{
		PRINTFAIL(bi, 7, hres)
		goto TekFileBinaryFatal;
	}


	/* -------------------- */
	/* --- HUNK 8 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(8, "Open the test file in TekFileMode_WriteOnly mode 64-bit")

	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 8)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 8, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
/* TekFileBinaryHunk8Fail: */
		PRINTFAIL(bi, 8, hres)
	}

	/* -------------------- */
	/* --- HUNK 9 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(9, "Open the test file in TekFileMode_ReadOnly mode")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadOnly);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 9, hres)
		goto TekFileBinaryFatal;
	}

	/* Try 32-bit mode first. */
	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 9, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 9, hres)
	}

	/* Try 64-bit mode second. */
	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 9)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 9, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 9, hres)
	}


	/* --------------------- */
	/* --- HUNK 10 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(10, "Open the test file in TekFileMode_Append mode")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_Append);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 10, hres)
		goto TekFileBinaryFatal;
	}

	/* Try 32-bit mode first. */
	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 10, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 10, hres)
	}

	/* Try 64-bit mode second. */
	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 10)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 10, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 10, hres)
	}


	/* --------------------- */
	/* --- HUNK 11 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(11, "Open the test file in TekFileMode_ReadWrite mode")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadWrite);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 11, hres)
		goto TekFileBinaryFatal;
	}

	/* Try 32-bit mode first. */
	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 11, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 11, hres)
	}

	/* Try 64-bit mode second. */
	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 11)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 11, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 11, hres)
	}


	/* --------------------- */
	/* --- HUNK 12 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(12, "Open the test file in TekFileMode_ReadWriteTrunc mode")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadWriteTrunc);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 12, hres)
		goto TekFileBinaryFatal;
	}

	/* Try 32-bit mode first. */
	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 12, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 12, hres)
	}

	/* Try 64-bit mode second. */
	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 12)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 12, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 12, hres)
	}


	/* --------------------- */
	/* --- HUNK 13 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(13, "Open the test file in TekFileMode_ReadAppend mode")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadAppend);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 13, hres)
		goto TekFileBinaryFatal;
	}

	/* Try 32-bit mode first. */
	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 13, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 13, hres)
	}

	/* Try 64-bit mode second. */
	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 13)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 13, hres)
			goto TekFileBinaryFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 13, hres)
	}


	/* --------------------- */
	/* --- HUNK 14 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(14, "Checking to see if TekFileBinary::Write() works")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadWriteTrunc);
	if ( hres != TEKERR_OK )
	{
		fprintf(stderr, 
		  "XFAIL: TekFile::put_Mode() failed!.\n"
		);
		PRINTFAIL(bi, 14, hres)
		goto TekFileBinaryFatal;
	}

	/* Re-open the file. */
	hres = tekfil->Open();
	if ( hres != TEKERR_OK )
	{
		fprintf(stderr, 
		  "XFAIL: TekFile::Open() failed!.\n"
		);
		PRINTFAIL(bi, 14, hres)
		goto TekFileBinaryFatal;
	}

	/* Write to the file. */
	hres = tekfil->Write((void *)"test&c", 6, &l);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 14, hres)
		goto TekFileBinaryFatal;
	}
	if ( l == 6 )
	{
		PRINTPASS(bi, 14);
	}
	else
	{
		fprintf(stderr, 
		  "XFAIL: Total bytes written < sizeof(char) * 6 !!!!!\n"
		);
		PRINTFAIL(bi, 14, hres)
		goto TekFileBinaryFatal;
	}


	/* --------------------- */
	/* --- HUNK 15 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(15, "Checking to see if TekFileBinary::Flush() works")
	hres = tekfil->Flush();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 15)
	}
	else
	{
		PRINTFAIL(bi, 15, hres)
	}


	/* --------------------- */
	/* --- HUNK 16 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(16, "Checking to see if we can seek -1 from current position")
	hres = tekfil->Seek(TekFileSeek_Current, -1);
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 16)
	}
	else
	{
		PRINTFAIL(bi, 16, hres)
	}


	/* --------------------- */
	/* --- HUNK 17 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(17, "Checking to see if get_Location() works")
	hres = tekfil->get_Location(&l);
	if ( hres == TEKERR_OK )
	{
		if ( l == 5 )
		{
			PRINTPASS(bi, 17)
		}
		else
		{
			fprintf(stderr, 
			  "XFAIL: get_Location returned %d instead of 5.\n",
			  (int)l
			);
			PRINTFAIL(bi, 17, hres)
		}
	}
	else
	{
		PRINTFAIL(bi, 17, hres)
	}


	/* --------------------- */
	/* --- HUNK 18 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(18, "Checking to see if we can seek to the end of the file")
	hres = tekfil->Seek(TekFileSeek_End, 0);
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 18)
	}
	else
	{
		PRINTFAIL(bi, 18, hres)
	}


	/* --------------------- */
	/* --- HUNK 19 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(19, "Checking to see if we can seek to the beginning of the file")
	hres = tekfil->Seek(TekFileSeek_Begin, 0);
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 19)
	}
	else
	{
		PRINTFAIL(bi, 19, hres)
	}


	/* --------------------- */
	/* --- HUNK 20 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(20, "Checking to see if TekFileBinary::Read() works")

	/* We must clear out the memory before executing cbuf. */
	memset(cbuf, '\0', sizeof(cbuf) * sizeof(char));

	/* Now execute tekfil->Read(). */
	hres = tekfil->Read(cbuf, sizeof(cbuf) * sizeof(char), &l);
	if ( hres == TEKERR_OK )
	{
		/* Do a strcmp() on the result. */
		if ( strcmp("test&c", cbuf) == 0 )
		{
			PRINTPASS(bi, 20)
		}
		else
		{
			fprintf(stderr, 
			  "XFAIL: Read() gave inaccurate result.\n  ours: %s\n read:%s\n",
			  "test&c",
			  cbuf
			);
			PRINTFAIL(bi, 20, hres)
		}
	}
	else
	{
		PRINTFAIL(bi, 20, hres)
	}


	/* --------------------- */
	/* --- HUNK 21 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(21, "Checking to see if we can close the file successfully")
	hres = tekfil->Close();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 21)
	}
	else
	{
		PRINTFAIL(bi, 21, hres)
	}

	/* --------------------- */
	/* --- HUNK 22 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(22, "Checking to see if TekFileBinary::ConvertToPipe() works")
	hres = tekbin->ConvertToPipe();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->get_Type(&tektype);
		if ( hres == TEKERR_OK )
		{
			if ( tektype == TekFileType_BinaryPipe )
			{
				PRINTPASS(bi, 22)
			} else {
				fprintf(stderr, 
				  "XFAIL: tektype != BinaryPipe (current val: %d)\n",
				  (int)tektype
				);
				PRINTFAIL(bi, 22, hres)
			}
		}
		else
		{
			fprintf(stderr, 
			  "XFAIL: get_Type() returned an error."
			);
			PRINTFAIL(bi, 22, hres)
		}
	}
	else
	{
		PRINTFAIL(bi, 22, hres)
	}

	/*
	hres = tekfil->Close();
	*/

	st = 0;
	for ( lt=0; lt < NUM_OF_TEKFILEBINARY_HUNKS; lt++ )
		if ( bi[lt] == true )
			st += 1;

	/* Release the instances from memory. */
	tekfil->Release();
	tekbin->Release();

	/* Print the results. */
	fprintf(stdout, "\n%d out of %d Hunks Passed.\n\n", st, sizeof(bi));

	return EXIT_SUCCESS;
	
TekFileBinaryFatal:

	fprintf(stderr, "Error is fatal.  Please report this issue.\n\n");
	fprintf(stdout, "It is highly suggested that you DO NOT install this library\n");
	fprintf(stdout, "until the problem listed above is fixed and/or patched.\n\n");
	fprintf(stdout, "A SIGTRAP is being raised to force a core dump.\n\n");

	raise(SIGTRAP);

	return EXIT_FAILURE;
}


