/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "testsuite.h"


#define	TESTFILE_TESTSUITE	"./test.txt"


int runtekfilebz2asciitests ( void )
{
	bool bi[NUM_OF_TEKFILEBZ2ASCII_HUNKS];
	TekFileBZ2ASCII * tekbin;
	TekFile * tekfil;
	TekFileMode tekmode;
	TekFileType tektype;
	uchar_t * uchrt;
	uchar_t luchrt;
	char * chrt;
	uuid_t uuidval;
	FILE * fdesc;
	TEKERR hres;
	unsigned long lt;
	unsigned short st;
	unsigned long l;
	char cbuf[7];

	unlink(TESTFILE_TESTSUITE);

	/* NULLify the bi array first. */
	for ( l=0; l != NUM_OF_TEKFILEBZ2ASCII_HUNKS; l++ )
		bi[l] = false;

	/* Tell the user we are running some tests. */
	fprintf(stdout, "Running test: TekFileBZ2ASCII tests.\n");
	fprintf(stdout, "Number of hunks to test: %d\n\n", NUM_OF_TEKFILEBZ2ASCII_HUNKS);

	/* -------------------- */
	/* --- HUNK 0 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(0, "Create an instance of TekFileBZ2ASCII")

	/* NULLify tekbin. */
	tekbin = NULL;
	/* Run TekFileCreate(). */
	hres = TekFileCreate(TekFileType_BZIP2_ASCII, (TekFile **)&tekbin);
	/* Check the result. */
	if ( hres == TEKERR_OK )
	{
		/* Make sure tekbin is a valid pointer. */
		if ( tekbin == NULL )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFileCreate() did not give valid pointer on retval of TEKERR_OK.\n"
			);
			goto TekFileASCIIBareHunk0Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 0)
	}
	else
	{
TekFileASCIIBareHunk0Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 0, hres)
		fprintf(stderr, "Can not produce instances of TekFileBZ2ASCII!!!!!\n");
		goto TekFileASCIIBareFatal;
	}


	/* -------------------- */
	/* --- HUNK 1 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(1, "Check that the instance is derived from TekFile\n, TekFileASCII, TekFileBZ2, and TekFileBZ2ASCII")

	/* Get the binary UUID of UTekFileASCII. */
	GETTEKUUID(UTekFileASCII, uuidval);
	/* Run phase 1 of the test; call CheckFor(). */
	hres = tekbin->CheckFor(&uuidval, 1);
	if ( hres != TEKERR_OK )
		goto TekFileASCIIBareHunk1Fail;

	/* Get the binary UUID of UTekFileASCII. */
	GETTEKUUID(UTekFileBZ2, uuidval);
	/* Run phase 1 of the test; call CheckFor(). */
	hres = tekbin->CheckFor(&uuidval, 1);
	if ( hres != TEKERR_OK )
		goto TekFileASCIIBareHunk1Fail;

	/* Get the binary UUID of UTekFileASCII. */
	GETTEKUUID(UTekFileBZ2ASCII, uuidval);
	/* Run phase 1 of the test; call CheckFor(). */
	hres = tekbin->CheckFor(&uuidval, 1);
	if ( hres != TEKERR_OK )
		goto TekFileASCIIBareHunk1Fail;

	/* Get the binary UUID of UTekFile. */
	GETTEKUUID(UTekFile, uuidval);
	/* NULLify tekfil. */
	tekfil = NULL;
	/* Run phase 2 of the test; call CheckFor(). */
	hres = tekbin->CheckFor(&uuidval, 1, 1, (void **)&tekfil);
	/* Check the result. */
	if ( hres == TEKERR_OK )
	{
		/* Make sure tekfil is a valid pointer. */
		if ( tekfil == NULL )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekStd::CheckFor() did not give valid pointer on retval of TEKERR_OK.\n"
			);
			goto TekFileASCIIBareHunk1Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 1)
	}
	else
	{
TekFileASCIIBareHunk1Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 1, hres)
		fprintf(stderr, "Can not produce instances of TekFileBZ2ASCII!!!!!\n");
		goto TekFileASCIIBareFatal;
	}


	/* -------------------- */
	/* --- HUNK 2 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(2, "Check that get_Type currently reports TekFileType_BZIP2_ASCII")

	/* Get the typeof tekfil. */
	hres = tekfil->get_Type(&tektype);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Check the tektype. */
		if ( tektype != TekFileType_BZIP2_ASCII )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Type() returned value %d instead of %d to &tektype.\n",
			  tektype,
			  TekFileType_Binary
			);
			goto TekFileASCIIBareHunk2Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 2)
	}
	else
	{
TekFileASCIIBareHunk2Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 2, hres)
	}


	/* -------------------- */
	/* --- HUNK 3 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(3, "Check that put_FILE and get_FILE works")

	/* Set FILE param to stderr. */
	hres = tekfil->put_FILE(stdin);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Check the FILE param. */
		hres = tekfil->get_FILE(&fdesc);
		if ( hres != TEKERR_OK )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_FILE() failed!.\n"
			);
			goto TekFileASCIIBareHunk3Fail;
		}

		if ( fdesc != stdin )
		{
			/* The parameter has invalid information; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_FILE() returned value other than stderr.\n"
			);
			goto TekFileASCIIBareHunk3Fail;
		}

		hres = tekfil->put_FILE(NULL);
		if ( hres != TEKERR_OK )
		{
			/* The parameter has invalid information; do fatal cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::put_FILE() unexpectedly failed.\n"
			);
			PRINTFAIL(bi, 3, hres)
			goto TekFileASCIIBareFatal;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 3)
	}
	else
	{
TekFileASCIIBareHunk3Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 3, hres)
	}


	/* -------------------- */
	/* --- HUNK 4 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(4, "Check that get_Filename and put_Filename works")

	/* Make a uchar out of TESTFILE_TESTSUITE. */
	uchrt = char2uchar(TESTFILE_TESTSUITE);
	if ( uchrt == NULL )
		goto TekFileASCIIBareHunk4Fail;

	/* Set Filename param to stderr. */
	hres = tekfil->put_Filename(uchrt);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Free the uchar. */
		ucharempty(uchrt);
		uchrt = NULL;

		/* Check the Filename param. */
		hres = tekfil->get_Filename(&uchrt);
		if ( hres != TEKERR_OK )
		{
			/* It is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Filename() failed!.\n"
			);
			goto TekFileASCIIBareHunk4Fail;
		}

		if ( uchrt == NULL )
		{
			/* The parameter was NULL; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Filename() returned NULL.\n"
			);
			goto TekFileASCIIBareHunk4Fail;
		}

		if ( strcmp(uchar2char(uchrt), TESTFILE_TESTSUITE) != 0 )
		{
			/* The parameter has invalid information; go to cleanup. */
			ucharempty(uchrt);
			fprintf(stderr, 
			  "XFAIL: uchrt from TekFile::get_Filename() does not match provided filename: %s.\n",
			  TESTFILE_TESTSUITE
			);
			goto TekFileASCIIBareHunk4Fail;
		}

		/* It is valid; hunk passed. */
		ucharempty(uchrt);
		PRINTPASS(bi, 4)
	}
	else
	{
		/* Empty out the uchrt. */
		ucharempty(uchrt);

TekFileASCIIBareHunk4Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 4, hres)
	}


	/* -------------------- */
	/* --- HUNK 5 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(5, "Check that get_Mode and put_Mode works")

	/* Set tekmode to TekFileMode_Append. */
	hres = tekfil->put_Mode(TekFileMode_WriteOnly);
	/* Check return value. */
	if ( hres == TEKERR_OK )
	{
		/* Check the Mode param. */
		hres = tekfil->get_Mode(&tekmode);
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Mode() failed!.\n"
			);
			goto TekFileASCIIBareHunk5Fail;
		}

		if ( tekmode != TekFileMode_WriteOnly )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::get_Mode() returned incorrect mode.\n"
			);
			goto TekFileASCIIBareHunk5Fail;
		}

		/* It is valid; hunk passed. */
		PRINTPASS(bi, 5)
	}
	else
	{
TekFileASCIIBareHunk5Fail:
		/* Hunk failed; report the error. */
		PRINTFAIL(bi, 5, hres)
	}


	/* -------------------- */
	/* --- HUNK 6 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(6, "Open the test file in TekFileMode_WriteOnly mode")

	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 6)
	}
	else
	{
		PRINTFAIL(bi, 6, hres)
		goto TekFileASCIIBareFatal;
	}


	/* -------------------- */
	/* --- HUNK 7 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(7, "Close the file and stream")

	hres = tekbin->CloseStream();
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 7, hres)
		goto TekFileASCIIBareFatal;
	}
	hres = tekfil->Close();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 7)
	}
	else
	{
		PRINTFAIL(bi, 7, hres)
		goto TekFileASCIIBareFatal;
	}


	/* -------------------- */
	/* --- HUNK 8 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(8, "Open the test file in TekFileMode_WriteOnly mode 64-bit")

	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 8)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 8, hres)
			goto TekFileASCIIBareFatal;
		}
	}
	else
	{
/* TekFileASCIIBareHunk8Fail: */
		PRINTFAIL(bi, 8, hres)
	}

	/* -------------------- */
	/* --- HUNK 9 BEGIN --- */
	/* -------------------- */
	PRINTHUNK(9, "Open the test file in TekFileMode_ReadOnly mode")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadOnly);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 9, hres)
		goto TekFileASCIIBareFatal;
	}

	/* Try 32-bit mode first. */
	hres = tekfil->Open();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres != TEKERR_OK )
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 9, hres)
			goto TekFileASCIIBareFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 9, hres)
	}

	/* Try 64-bit mode second. */
	hres = tekfil->Open64();
	if ( hres == TEKERR_OK )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 9)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 9, hres)
			goto TekFileASCIIBareFatal;
		}
	}
	else
	{
		PRINTFAIL(bi, 9, hres)
	}


	/* --------------------- */
	/* --- HUNK 10 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(10, "Checking to see if TekFileBZ2ASCII::Write() works")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_WriteOnly);
	if ( hres != TEKERR_OK )
	{
		fprintf(stderr, 
		  "XFAIL: TekFile::put_Mode() failed!.\n"
		);
		PRINTFAIL(bi, 10, hres)
		goto TekFileASCIIBareFatal;
	}

	/* Re-open the file. */
	hres = tekfil->Open();
	if ( hres != TEKERR_OK )
	{
		fprintf(stderr, 
		  "XFAIL: TekFile::Open() failed!.\n"
		);
		PRINTFAIL(bi, 10, hres)
		goto TekFileASCIIBareFatal;
	}

	/* Write to the file. */
	hres = tekfil->Write((void *)"test&c", 6, &l);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 10, hres)
		goto TekFileASCIIBareFatal;
	}
	if ( l == 6 )
	{
		hres = tekfil->Close();
		if ( hres == TEKERR_OK )
		{
			PRINTPASS(bi, 10)
		}
		else
		{
			/* Mode is invalid; go to cleanup. */
			fprintf(stderr, 
			  "XFAIL: TekFile::Close() failed!.\n"
			);
			PRINTFAIL(bi, 10, hres)
			goto TekFileASCIIBareFatal;
		}
	}
	else
	{
		fprintf(stderr, 
		  "XFAIL: Total bytes written < sizeof(char) * 6 !!!!!\n"
		);
		PRINTFAIL(bi, 10, hres)
		goto TekFileASCIIBareFatal;
	}


	/* --------------------- */
	/* --- HUNK 11 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(11, "Checking to see if TekFileBZ2ASCII::Read() works")

	/* Set mode. */
	hres = tekfil->put_Mode(TekFileMode_ReadOnly);
	if ( hres != TEKERR_OK )
	{
		fprintf(stderr, 
		  "XFAIL: TekFile::put_Mode() failed!.\n"
		);
		PRINTFAIL(bi, 11, hres)
		goto TekFileASCIIBareFatal;
	}

	/* Re-open the file. */
	hres = tekfil->Open();
	if ( hres != TEKERR_OK )
	{
		fprintf(stderr, 
		  "XFAIL: TekFile::Open() failed!.\n"
		);
		PRINTFAIL(bi, 11, hres)
		goto TekFileASCIIBareFatal;
	}

	/* We must clear out the memory before executing cbuf. */
	memset(cbuf, '\0', sizeof(cbuf) * sizeof(char));

	/* Now execute tekfil->Read(). */
	hres = tekfil->Read(cbuf, sizeof(cbuf) * sizeof(char), &l);
	if ( ( hres == TEKERR_OK ) || ( hres == TEKERR_FILE_EOF ) )
	{
		/* Do a strcmp() on the result. */
		if ( strcmp("test&c", cbuf) == 0 )
		{
			PRINTPASS(bi, 11)
		}
		else
		{
			fprintf(stderr, 
			  "XFAIL: Read() gave inaccurate result.\n  ours: %s\n read:%s\n",
			  "test&c",
			  cbuf
			);
			PRINTFAIL(bi, 11, hres)
		}
	}
	else
	{
		PRINTFAIL(bi, 11, hres)
	}


	/* --------------------- */
	/* --- HUNK 12 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(12, "Checking to see if we can close the file successfully")
	hres = tekfil->Close();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 12)
	}
	else
	{
		PRINTFAIL(bi, 12, hres)
	}

	/* --------------------- */
	/* --- HUNK 22 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(13, "Checking to see if TekFileBZ2ASCII::WriteLine() works")

	/* Allocate an ascii string. */
	luchrt.uchar_t_ascii = (char *)calloc(257, sizeof(char));
	if ( luchrt.uchar_t_ascii == NULL )
	{
		PRINTFAIL(bi, 13, TEKERR_MEMORY);
		fprintf(stderr, "XFAIL: Unexpected failure in calloc().\n");
		goto TekFileASCIIBareFatal;
	}
	luchrt.uchar_t_asciilen = 256 * sizeof(char);

	/* Copy the string "test" 256 times. */
	/* NOTE: This method is more efficient than strcat(). */
	/* strcat() always has to find the end of the string. */
	chrt = luchrt.uchar_t_ascii;
	for ( l=0; l < sizeof(char) * 64; l++ )
	{
		memcpy(chrt, "test", sizeof(char) * 4);
		chrt += 4;
	}
	chrt[0] = '\0';

	/* Re-open the file. */
	hres = tekfil->put_Mode(TekFileMode_WriteOnly);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 13, hres)
		fprintf(stderr, "XFAIL: Unexpected failure in tekfil->put_Mode().\n");
		free(luchrt.uchar_t_ascii);
		goto TekFileASCIIBareFatal;
	}
	hres = tekfil->Open();
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 13, hres)
		fprintf(stderr, "XFAIL: Unexpected failure in tekfil->Open().\n");
		free(luchrt.uchar_t_ascii);
		goto TekFileASCIIBareFatal;
	}

	for ( l=0; l < 4; l++ )
	{
		/* Write out the line. */
		hres = tekbin->WriteLine(&luchrt);
		if ( hres != TEKERR_OK )
		{
			PRINTFAIL(bi, 13, hres);
			goto WriteLineFreeuchr;
		}
	}

	/* Write line 5. */
	luchrt.uchar_t_ascii[4] = '\0';
	luchrt.uchar_t_asciilen = 4;

	/* Write out the line. */
	hres = tekbin->WriteLine(&luchrt);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 13, hres);
		goto WriteLineFreeuchr;
	}

	hres = tekfil->Close();
	if ( hres == TEKERR_OK )
	{
		PRINTPASS(bi, 13)
	}
	else
	{
		PRINTFAIL(bi, 13, hres)
	}

WriteLineFreeuchr:
	free(luchrt.uchar_t_ascii);

	/* --------------------- */
	/* --- HUNK 14 BEGIN --- */
	/* --------------------- */
	PRINTHUNK(14, "Checking to see if TekFileBZ2ASCII::ReadLine() works")

	/* Re-open the file. */
	hres = tekfil->put_Mode(TekFileMode_ReadOnly);
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 14, hres)
		fprintf(stderr, "XFAIL: Unexpected failure in tekfil->put_Mode().\n");
		free(luchrt.uchar_t_ascii);
		goto TekFileASCIIBareFatal;
	}
	hres = tekfil->Open();
	if ( hres != TEKERR_OK )
	{
		PRINTFAIL(bi, 14, hres)
		fprintf(stderr, "XFAIL: Unexpected failure in tekfil->Open().\n");
		free(luchrt.uchar_t_ascii);
		goto TekFileASCIIBareFatal;
	}

	/* Enter loop. */
	uchrt = NULL;
	while ( hres == TEKERR_OK )
	{
		/* Read four characters from file. */
		hres = tekbin->ReadLine(&uchrt, 4);

		/* If hres == TEKERR_OK, check the string. */
		if ( hres == TEKERR_OK )
		{
ReadFileCheckString:
			if ( strcmp(uchrt->uchar_t_ascii, "test") != 0 )
			{
				PRINTFAIL(bi, 14, TEKERR_FAIL)
				fprintf(stderr, "XFAIL: strcmp() failed.\n");
				goto ReadLineSkip;
			}

			continue;
		}

		/* If hres == TEKERR_FILE_EOF, check if we only reached the
		   end of the line. */
		if ( hres == TEKERR_FILE_EOF )
		{
			hres = tekbin->ReadLine(&uchrt, 4);
			if ( hres == TEKERR_OK )
				goto ReadFileCheckString;
			if ( hres == TEKERR_FILE_EOF )
				break;

			/* Some error occured; nothing else to do. */
			PRINTFAIL(bi, 14, hres)
			goto ReadLineSkip;
		} else {
			/* Some error occured; nothing else to do. */
			PRINTFAIL(bi, 14, hres)
			goto ReadLineSkip;
		}
	}

	PRINTPASS(bi, 14)

ReadLineSkip:

	/* Empty out the uchar. */
	if ( uchrt != NULL )
		ucharempty(uchrt);


	st = 0;
	for ( lt=0; lt < NUM_OF_TEKFILEBZ2ASCII_HUNKS; lt++ )
		if ( bi[lt] == true )
			st += 1;

	/* Release the instances from memory. */
	tekfil->Release();
	tekbin->Release();

	/* Print the results. */
	fprintf(stdout, "\n%d out of %d Hunks Passed.\n\n", st, sizeof(bi));

	return EXIT_SUCCESS;
	
TekFileASCIIBareFatal:

	fprintf(stderr, "Error is fatal.  Please report this issue.\n\n");
	fprintf(stdout, "It is highly suggested that you DO NOT install this library\n");
	fprintf(stdout, "until the problem listed above is fixed and/or patched.\n\n");
	fprintf(stdout, "A SIGTRAP is being raised to force a core dump.\n\n");

#ifdef	USE_386_ASM
	__asm__ ( " int $3 " );
#else	/* USE_386_ASM */
	raise(SIGTRAP);
#endif	/* USE_386_ASM */

	return EXIT_FAILURE;
}


