/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "testsuite.h"


int runtekstdtests ( struct sigaction * oldactsegv, struct sigaction * oldactill )
{
	TekFileBinary * ct = NULL;
	TekFile * cv = NULL;
	unsigned long lt;
	unsigned short st;
	unsigned int ui;
	bool bi[NUM_OF_MAIN_HUNKS] = {false, false, false, false, false, false};
	TEKERR err;
	uuid_t uuidar;
	freext * fxt;

	fprintf(stdout, "Running test: TekStd tests.\n");
	fprintf(stdout, "Number of hunks to test: %d\n\n", NUM_OF_MAIN_HUNKS);
	fxt = &free;

	PRINTHUNK(0, "Checking libTekLTI's Global Variables");
	ui = checkverstruct();
	switch ( ui )
	{
		case 1:
			PRINTFAIL(bi, 0, TEKERR_UNAVAILABLE);
			verstructfatal();
			return EXIT_FAILURE;
		case 2:
			PRINTFAIL(bi, 0, TEKERR_UNAVAILABLE);
			incorrectverfatal();
			return EXIT_FAILURE;
		case 3:
			PRINTFAIL(bi, 0, TEKERR_UNAVAILABLE);
			implfatal();
			return EXIT_FAILURE;
		default:
			PRINTPASS(bi, 0)
			break;
	}

	PRINTHUNK(1, "Create an instance of a TekStd-derived interface")
	
	/* Restore the sigactions back to what they were */
	sigaction(SIGSEGV, oldactsegv, NULL);
	sigaction(SIGILL, oldactill, NULL);
	ct = new TekFileBinary;
	PRINTPASS(bi, 1);

	PRINTHUNK(2, "Add a reference to an instance")
	ui = ct->AddRef();
	if ( ui == 2 )
	{ PRINTPASS(bi, 2) }
	else
	{ PRINTFAIL(bi, 2, TEKERR_UNAVAILABLE) }

	PRINTHUNK(3, "Remove a reference to an instance")
	ui = ct->Release();
	if ( ui == 1 )
	{ PRINTPASS(bi, 3) }
	else
	{ PRINTFAIL(bi, 3, TEKERR_UNAVAILABLE) }

	GETTEKUUID(UTekFile, uuidar);
	PRINTHUNK(4, "Check to see if an instance supports an interface")
	err = ct->CheckFor(&uuidar, 1, 1, (void **)&cv);
	if ( err )
	{
		PRINTFAIL(bi, 4, err)
	} else {
		PRINTPASS(bi, 4)
	}

	PRINTHUNK(5, "Destroy all references to an instance")
	if ( cv != NULL )
		cv->Release();
	ui = ct->Release();
	if ( ui == 0 )
	{ PRINTPASS(bi, 5) }
	else
	{ PRINTFAIL(bi, 5, TEKERR_UNAVAILABLE) }

	st = 0;
	for ( lt=0; lt < sizeof(bi); lt++ )
		if ( bi[lt] == true )
			st += 1;

	/* Print the results. */
	fprintf(stdout, "\n%d out of %d Hunks Passed.\n\n", st, sizeof(bi));

	return EXIT_SUCCESS;
}

