/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)

#include "teklti.h"

#ifdef	USE_386_ASM

#include <string.h>

#ifdef	DEBUG_USE_FERROR
#include <stdio.h>
#endif	/* DEBUG_USE_FERROR */


#ifdef	DEBUG_USE_FERROR
static const char _uchardup_srcnull[] = "POTENTIAL SEGFAULT in uchardup: src is NULL!\n";
static const char _char2ucharlen_srcnull[] = "POTENTIAL SEGFAULT in char2ucharlen: src is NULL!\n";
static const char _char2uchar_srcnull[] = "POTENTIAL SEGFAULT in char2uchar: src is NULL!\n";
#endif	/* DEBUG_USE_FERROR */


TEKLTI_DECLSPEC uchar_t * uchardup ( const uchar_t * src )
{
	/* Set up our retval pointer. */
	__asm__ ( " uchardup_passpushpop: " );

	/* Copy src pointer to eax. */
	__asm__ ( " movl %0, %%eax " : "=m" (src) );

#ifndef	NO_SIZE_T_CHECKS
	/* Test if ebx is NULL. */
	__asm__ ( " testl %eax, %eax " );
	/* If ebx is not NULL, check the length.fwrite */
	__asm__ ( " jne .uchardupproc " );
#ifdef	DEBUG_USE_FERROR
	/* Set up the stack. */
	__asm__ ( " subl $8, %esp " );
	/* Copy the ptr of the error. */
	__asm__ ( " movl $_uchardup_srcnull, (%esp) " );
	/* Copy the stderr address to eax. */
	__asm__ ( " movl stderr, %eax " );
	/* Then to param 1. */
	__asm__ ( " movl %eax, 4(%esp) " );
	/* Call fputs to output the error to stderr. */
	__asm__ ( " call fputs " );
#endif	/* DEBUG_USE_FERROR */
	/* Be sure eax is zero for retval. */
	__asm__ ( " xorl %eax, %eax " );
	/* Reset the stack. */
	__asm__ ( " addl $8, %esp " );
	/* We must now punt. */
	__asm__ ( " jmp .uchardupnullret " );

	/* Resume normal functions. */
	__asm__ ( " .uchardupproc: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " subl $16, %esp " );
	/* Push esi onto the stack */
	__asm__ ( " movl %esi, 12(%esp) " );
	/* Save edi. */
	__asm__ ( " movl %edi, 4(%esp) " );
#else	/* not USE_386_ASM_PUSHPOP */
	__asm__ ( " subl $12, %esp " );
#endif	/* not USE_386_ASM_PUSHPOP */

	/* FIRST - Let's get the length of string src. */
	/* Copy src pointer to esi. */
#ifndef	NO_SIZE_T_CHECKS
	__asm__ ( " movl %eax, %esi " );
#else	/* NO_SIZE_T_CHECKS */
	__asm__ ( " movl %0, %%esi " : "=m" (src) );
#endif	/* NO_SIZE_T_CHECKS */

	/* FIRST, let's allocate the new string. */
	/* Set the count register (ecx) to (len). */
	__asm__ ( " movl 4(%esi), %ecx " );
	/* Set up the malloc() param. */
	__asm__ ( " movl %ecx, (%esp) " );
	/* Call malloc(). */
	__asm__ ( " call malloc " );
	/* Check to see if malloc() failed. */
	__asm__ ( " testl %eax, %eax " );
	/* If it did, nothing to do. */
	__asm__ ( " je .uchardupmallocfail " );

	/* Save the string somewhere. */
	__asm__ ( " movl %eax, %edi " );
	/* Put count into param 0. */
	__asm__ ( " movl $8, (%esp) " );
	/* Call malloc(). */
	__asm__ ( " call malloc " );
	/* Did malloc fail? */
	__asm__ ( " testl %eax, %eax " );
	/* If it did, nothing to do. */
	__asm__ ( " je .uchardupmallocfail " );

	/* Set up the uchar_t. */
	/* Set the count register (ecx) to (len). */
	__asm__ ( " movl 4(%esi), %ecx " );
	/* Start with dest.uchar_t_ascii */
	__asm__ ( " movl %edi, (%eax) " );
	/* End with dest.uchar_t_asciilen. */
	__asm__ ( " movl %ecx, 4(%eax) " );

	/* Copy the string. */
	/* Set up %esi. */
	__asm__ ( " movl (%esi), %esi " );
	/* Clear direction flag. */
	__asm__ ( " cld " );
	/* Copy the string! */
	__asm__ ( " rep movsb " );
	/* Null-terminate the destination! */
	__asm__ ( " movb $0, (%edi) " );

	__asm__ ( " .uchardupmallocfail: " );
#ifdef	USE_386_ASM_PUSHPOP
	/* Restore esi. */
	__asm__ ( " movl 12(%esp), %esi " );
	__asm__ ( " movl 4(%esp), %edi " );
	__asm__ ( " addl $16, %esp " );
#else	/* not USE_386_ASM_PUSHPOP */
	__asm__ ( " addl $12, %esp " );
#endif	/* not USE_386_ASM_PUSHPOP */	

#ifndef	NO_SIZE_T_CHECKS
	/* Final cleanup. */
	__asm__ ( " .uchardupnullret: " );
#endif	/* not NO_SIZE_T_CHECKS */

	/* Return to caller - added as a last resort exit. */
	return;
}


TEKLTI_DECLSPEC uchar_t * char2ucharlen ( const char * src, size_t len )
{
	/* Set up our retval pointer. */
	__asm__ ( " char2ucharlen_passpushpop: " );

	/* Copy src pointer to eax. */
	__asm__ ( " movl %0, %%eax " : "=m" (src) );

#ifndef	NO_SIZE_T_CHECKS
	/* Test if ebx is NULL. */
	__asm__ ( " testl %eax, %eax " );
	/* If ebx is not NULL, check the length.fwrite */
	__asm__ ( " jne .char2ucharlenproc " );
#ifdef	DEBUG_USE_FERROR
	/* Set up the stack. */
	__asm__ ( " subl $8, %esp " );
	/* Copy the ptr of the error. */
	__asm__ ( " movl $_char2ucharlen_srcnull, (%esp) " );
	/* Copy the stderr address to eax. */
	__asm__ ( " movl stderr, %eax " );
	/* Then to param 1. */
	__asm__ ( " movl %eax, 4(%esp) " );
	/* Call fputs to output the error to stderr. */
	__asm__ ( " call fputs " );
#endif	/* DEBUG_USE_FERROR */
	/* Be sure eax is zero for retval. */
	__asm__ ( " xorl %eax, %eax " );
	/* Reset the stack. */
	__asm__ ( " addl $8, %esp " );
	/* We must now punt. */
	__asm__ ( " jmp .char2ucharlennullret " );

	/* Resume normal functions. */
	__asm__ ( " .char2ucharlenproc: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " subl $16, %esp " );
	/* Push esi onto the stack */
	__asm__ ( " movl %esi, 12(%esp) " );
	/* Save edi. */
	__asm__ ( " movl %edi, 4(%esp) " );
#else	/* not USE_386_ASM_PUSHPOP */
	__asm__ ( " subl $12, %esp " );
#endif	/* not USE_386_ASM_PUSHPOP */

	/* FIRST - Let's get the length of string src. */
	/* Copy src pointer to esi. */
#ifndef	NO_SIZE_T_CHECKS
	__asm__ ( " movl %eax, %esi " );
#else	/* NO_SIZE_T_CHECKS */
	__asm__ ( " movl %0, %%esi " : "=m" (src) );
#endif	/* NO_SIZE_T_CHECKS */

	/* Set the count register (ecx) to (len). */
	__asm__ ( " movl %0, %%ecx " : "=m" (len) );
	/* Set up the malloc() param. */
	__asm__ ( " movl %ecx, (%esp) " );
	/* Call malloc(). */
	__asm__ ( " call malloc " );
	/* Check to see if malloc() failed. */
	__asm__ ( " testl %eax, %eax " );
	/* If it did, nothing to do. */
	__asm__ ( " je .char2ucharlenmallocfail " );

	/* Save the string somewhere. */
	__asm__ ( " movl %eax, %edi " );
	/* Put count into param 0. */
	__asm__ ( " movl $8, (%esp) " );
	/* Call malloc(). */
	__asm__ ( " call malloc " );
	/* Did malloc fail? */
	__asm__ ( " testl %eax, %eax " );
	/* If it did, nothing to do. */
	__asm__ ( " je .char2ucharlenmallocfail " );

	/* Set up the uchar_t. */
	/* Set the count register (ecx) to (len). */
	__asm__ ( " movl %0, %%ecx " : "=m" (len) );
	/* Start with dest.uchar_t_ascii */
	__asm__ ( " movl %edi, (%eax) " );
	/* End with dest.uchar_t_asciilen. */
	__asm__ ( " movl %ecx, 4(%eax) " );

	/* Copy the string. */
	/* Clear direction flag. */
	__asm__ ( " cld " );
	/* Copy the string! */
	__asm__ ( " rep movsb " );
	/* Null-terminate the destination! */
	__asm__ ( " movb $0, (%edi) " );

	__asm__ ( " .char2ucharlenmallocfail: " );
#ifdef	USE_386_ASM_PUSHPOP
	/* Restore esi. */
	__asm__ ( " movl 12(%esp), %esi " );
	__asm__ ( " movl 4(%esp), %edi " );
	__asm__ ( " addl $16, %esp " );
#else	/* not USE_386_ASM_PUSHPOP */
	__asm__ ( " addl $12, %esp " );
#endif	/* not USE_386_ASM_PUSHPOP */	

#ifndef	NO_SIZE_T_CHECKS
	/* Final cleanup. */
	__asm__ ( " .char2ucharlennullret: " );
#endif	/* not NO_SIZE_T_CHECKS */

	/* Return to caller - added as a last resort exit. */
	return;
}


TEKLTI_DECLSPEC uchar_t * char2uchar ( const char * src )
{
	/* Set up our retval pointer. */
	__asm__ ( " char2uchar_passpushpop: " );

	/* Copy src pointer to eax. */
	__asm__ ( " movl %0, %%eax " : "=m" (src) );

#ifndef	NO_SIZE_T_CHECKS
	/* Test if ebx is NULL. */
	__asm__ ( " testl %eax, %eax " );
	/* If ebx is not NULL, check the length.fwrite */
	__asm__ ( " jne .char2ucharproc " );
#ifdef	DEBUG_USE_FERROR
	/* Set up the stack. */
	__asm__ ( " subl $8, %esp " );
	/* Copy the ptr of the error. */
	__asm__ ( " movl $_char2uchar_srcnull, (%esp) " );
	/* Copy the stderr address to eax. */
	__asm__ ( " movl stderr, %eax " );
	/* Then to param 1. */
	__asm__ ( " movl %eax, 4(%esp) " );
	/* Call fputs to output the error to stderr. */
	__asm__ ( " call fputs " );
#endif	/* DEBUG_USE_FERROR */
	/* Be sure eax is zero for retval. */
	__asm__ ( " xorl %eax, %eax " );
	/* Reset the stack. */
	__asm__ ( " addl $8, %esp " );
	/* We must now punt. */
	__asm__ ( " jmp .char2ucharnullret " );

	/* Resume normal functions. */
	__asm__ ( " .char2ucharproc: " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_PUSHPOP
	__asm__ ( " subl $16, %esp " );
	/* Push esi onto the stack */
	__asm__ ( " movl %esi, 12(%esp) " );
	/* Save edi. */
	__asm__ ( " movl %edi, 4(%esp) " );
#else	/* not USE_386_ASM_PUSHPOP */
	__asm__ ( " subl $12, %esp " );
#endif	/* not USE_386_ASM_PUSHPOP */

	/* FIRST - Let's get the length of string src. */
	/* Copy src pointer to esi. */
#ifndef	NO_SIZE_T_CHECKS
	__asm__ ( " movl %eax, %esi " );
#else	/* NO_SIZE_T_CHECKS */
	__asm__ ( " movl %0, %%esi " : "=m" (src) );
#endif	/* NO_SIZE_T_CHECKS */
	/* Set the count register (ecx) to zero. */
	__asm__ ( " xorl %ecx, %ecx " );

	/* Clear direction flag. */
	__asm__ ( " cld " );
	/* Create a label for a small conditional jump. */
	__asm__ ( " .char2ucharnext1: " );
	/* Get next char in sequence. */
	__asm__ ( " lodsb " );
	/* Increment count variable. */
	__asm__ ( " incl %ecx " );
	/* Test for the null character. */
	__asm__ ( " testb %al, %al " );
	/* If null character not reached yet, get next character. */
	__asm__ ( " jne .char2ucharnext1 " );
	/* Restore esi back to what it originally was. */
	__asm__ ( " subl %ecx, %esi " );
	/* Make the count true. */
	__asm__ ( " decl %ecx " );

	/* Push ecx onto the stack. */
	__asm__ ( " movl %ecx, 8(%esp) " );
	/* Set up the malloc() param. */
	__asm__ ( " movl %ecx, (%esp) " );
	/* Call malloc(). */
	__asm__ ( " call malloc " );
	/* Check to see if malloc() failed. */
	__asm__ ( " testl %eax, %eax " );
	/* If it did, nothing to do. */
	__asm__ ( " je .char2ucharmallocfail " );

	/* Save the string somewhere. */
	__asm__ ( " movl %eax, %edi " );
	/* Put count into param 0. */
	__asm__ ( " movl $8, (%esp) " );
	/* Call malloc(). */
	__asm__ ( " call malloc " );
	/* Did malloc fail? */
	__asm__ ( " testl %eax, %eax " );
	/* If it did, nothing to do. */
	__asm__ ( " je .char2ucharmallocfail " );

	/* Set up the uchar_t. */
	/* Restore ecx. */
	__asm__ ( " movl 8(%esp), %ecx " );
	/* Start with dest.uchar_t_ascii */
	__asm__ ( " movl %edi, (%eax) " );
	/* End with dest.uchar_t_asciilen. */
	__asm__ ( " movl %ecx, 4(%eax) " );

	/* Copy the string. */
	/* Clear direction flag. */
	__asm__ ( " cld " );
	/* Copy the string! */
	__asm__ ( " rep movsb " );
	/* Null-terminate the destination! */
	__asm__ ( " movb $0, (%edi) " );

	__asm__ ( " .char2ucharmallocfail: " );
#ifdef	USE_386_ASM_PUSHPOP
	/* Restore esi. */
	__asm__ ( " movl 12(%esp), %esi " );
	__asm__ ( " movl 4(%esp), %edi " );
	__asm__ ( " addl $16, %esp " );
#else	/* not USE_386_ASM_PUSHPOP */
	__asm__ ( " addl $12, %esp " );
#endif	/* not USE_386_ASM_PUSHPOP */	

#ifndef	NO_SIZE_T_CHECKS
	/* Final cleanup. */
	__asm__ ( " .char2ucharnullret: " );
#endif	/* not NO_SIZE_T_CHECKS */

	/* Return to caller - added as a last resort exit. */
	return;
}


TEKLTI_DECLSPEC void ucharempty ( uchar_t * uchrtofree )
{
	/* Prepare for the free() calls. */
	__asm__ ( " ucharempty_passpushpop: " );

	/* Get the value to uchrtofree. */
	__asm__ ( " movl %0, %%ecx " : "=m" (uchrtofree) );

#ifndef	NO_SIZE_T_CHECKS
	/* Check %ecx for NULLs. */
	__asm__ ( " testl %ecx, %ecx " );
	/* If it is NULL, jump to the error area to raise a SIGTRAP. */
	__asm__ ( " je .ucharempty_sigtrap " );
#endif	/* not NO_SIZE_T_CHECKS */

#ifdef	USE_386_ASM_ENFORCE_PRIVACY
#ifdef	USE_386_ASM_PUSHPOP
	/* Save current values of edi. */
	__asm__ ( " pushl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */

	/* Get uchrtofree->uchar_t_ascii */
	__asm__ ( " movl (%ecx), %edi " );

#ifdef	USE_386_ASM_MMX
	/* Put the pointer from (%ecx) into param 0. */
	__asm__ ( " pushl %edi " );

	/* Save the value of dx for now. */
	__asm__ ( " movw %dx, %di " );

#ifdef	USE_386_ASM_MMX_RESET_MM7
	/* Copy the nullified quadword to mm0. */
	__asm__ ( " movq %0, %%mm7 " : "=m" (tekzerovar) );
#endif	/* USE_386_ASM_MMX_RESET_MM7 */

	/* Set dx to upper four digits of uchar_t_asciilen. */
	__asm__ ( " movw 6(%ecx), %dx " );
	/* Set ax to lower four digits. */
	__asm__ ( " movw 4(%ecx), %ax " );
	/* Set cx to 8 as the divisor. */
	__asm__ ( " movw $8, %cx " );
	/* Divide dx:ax by 8. */
	__asm__ ( " divw %cx " );

	/* NULLify ecx, the count register. */
	__asm__ ( " xorl %ecx, %ecx " );

	/* Get ready for a loop.  Put %ax into %cx. */
	__asm__ ( " movw %ax, %cx " );
	/* Move %di to %ax. */
	__asm__ ( " movw %di, %ax " );
	/* Restore %edi. */
	__asm__ ( " movl (%esp), %edi " );
	/* Increment %cx. */
	__asm__ ( " incw %cx " );
	/* Determine if we need to enter the loop. */
	__asm__ ( " loopw .ucharempty_mmx_loop " );
	/* We don't, so let's skip the loop. */
	__asm__ ( " jmp .ucharempty_mmx_loop_end " );

	/* While %ecx != 0.... */
	__asm__ ( " .ucharempty_mmx_loop: " );
	/* NULLify the current memory location. */
	__asm__ ( " movq %mm7, (%edi) " );
	/* Add eight to edi. */
	__asm__ ( " addl $8, %edi " );
	/* See if we will have to execute more nullifications. */
	__asm__ ( " loopw .ucharempty_mmx_loop " );

	/* End of MMX loop. */
	__asm__ ( " .ucharempty_mmx_loop_end: " );

	/* Exit the mmx state. */
	__asm__ ( " emms " );

	/* Put edx into ecx for rep stosb to do its work. */
	__asm__ ( " movw %dx, %cx " );
	/* Restore %dx. */
	__asm__ ( " movw %ax, %dx " );

#else	/* not USE_386_ASM_MMX */
	/* Put the pointer from (%ecx) into param 0. */
	__asm__ ( " pushl %edi " );

	/* Get uchrtofree->uchar_t_asciilen */
	__asm__ ( " movl 4(%ecx), %ecx " );
#endif	/* not USE_386_ASM_MMX */

	/* Put $0 into al. */
	__asm__ ( " xorb %al, %al " );
	/* and nullify the rest of the string. */
	__asm__ ( " rep stosb " );

	/* Free the ascii string. */
	__asm__ ( " call free " );

	/* Clean up the stack. */
	__asm__ ( " addl $4, %esp " );
#ifdef	USE_386_ASM_PUSHPOP
	/* Restore %edi as necessary. */
	__asm__ ( " popl %edi " );
#endif	/* USE_386_ASM_PUSHPOP */


#else	/* not USE_386_ASM_ENFORCE_PRIVACY */

	/*
	   If we're here, the user(s) does their own privacy,
	   so we only need to release the memory.
	*/

	/* Get uchrtofree->uchar_t_ascii */
	__asm__ ( " movl (%ecx), %eax " );
	/* Put the pointer into param 0. */
	__asm__ ( " pushl %eax " );
	/* Free the ascii string. */
	__asm__ ( " call free " );
#endif	/* not USE_386_ASM_ENFORCE_PRIVACY */

	/* Cleanup. */
	__asm__ ( " movl %ebp, %esp " );
	__asm__ ( " popl %ebp " );
	/* free() has similiar syntax as this function,
	   so a jump is all we need to free the final
	   structure from memory. */
	__asm__ ( " jmp free " );

#ifndef	NO_SIZE_T_CHECKS
	/* Used for raising SIGTRAP in the program. */
	__asm__ ( " .ucharempty_sigtrap: " );
#ifndef	USE_386_ASM_NO_INT_3
	/* Raise the SIGTRAP. */
	__asm__ ( " int $3 " );
#endif	/* USE_386_ASM_INT_3 */
#endif	/* not NO_SIZE_T_CHECKS */
}

#endif	/* USE_386_ASM */

