/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride                                   *
 *   kevin@planetsaphire.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)

#include "teklti.h"

#ifndef	USE_386_ASM

#include <string.h>
#include <signal.h>

#include <stdio.h>


#ifdef	DEBUG_USE_FERROR
static const char uchardup_srcnull[] = "POTENTIAL SEGFAULT in uchardup: src is NULL!\n";
static const char char2uchar_srcnull[] = "POTENTIAL SEGFAULT in char2uchar: src is NULL!\n";
static const char ucharempty_uchrtofreenull[] = "POTENTIAL SEGFAULT in ucharfree: uchrtofree is NULL!\n";
#endif	/* DEBUG_USE_FERROR */


TEKLTI_DECLSPEC uchar_t * uchardup ( const uchar_t * src )
{
	/* Set up our retval pointer. */
	uchar_t * retval;

#ifndef	NO_SIZE_T_CHECKS
	if ( src == NULL )
	{
#ifdef	DEBUG_USE_FERROR
		fputs(char2uchar_srcnull, stderr);
#endif	/* DEBUG_USE_FERROR */
		return NULL;
	}
#endif	/* not NO_SIZE_T_CHECKS */

	retval = (uchar_t *)malloc(sizeof(uchar_t));
	if ( retval == NULL )
		return NULL;

	retval->uchar_t_asciilen = src->uchar_t_asciilen;

	retval->uchar_t_ascii = (char *)malloc(
		sizeof(char) * retval->uchar_t_asciilen
	);
	if ( retval->uchar_t_ascii == NULL )
	{
		free(retval);

		return NULL;
	}

	(void)strncpy(
		retval->uchar_t_ascii,
		src->uchar_t_ascii,
		retval->uchar_t_asciilen
	);

	return retval;
}


TEKLTI_DECLSPEC uchar_t * char2ucharlen ( const char * src, size_t len )
{
	uchar_t * retval;

#ifndef	NO_SIZE_T_CHECKS
	if ( src == NULL )
	{
#ifdef	DEBUG_USE_FERROR
		fputs(char2uchar_srcnull, stderr);
#endif	/* DEBUG_USE_FERROR */
		return NULL;
	}
#endif	/* not NO_SIZE_T_CHECKS */

	retval = (uchar_t *)malloc(sizeof(uchar_t));
	if ( retval == NULL )
		return NULL;

	retval->uchar_t_asciilen = len + 1;
	
	retval->uchar_t_ascii = (char *)malloc(
		sizeof(char) * retval->uchar_t_asciilen
	);
	if ( retval->uchar_t_ascii == NULL )
	{
		free(retval);

		return NULL;
	}

	(void)strncpy(retval->uchar_t_ascii, src, retval->uchar_t_asciilen);

	return retval;
}


TEKLTI_DECLSPEC uchar_t * char2uchar ( const char * src )
{
	/* Set up our retval pointer. */
	uchar_t * retval;

#ifndef	NO_SIZE_T_CHECKS
	if ( src == NULL )
	{
#ifdef	DEBUG_USE_FERROR
		fputs(char2uchar_srcnull, stderr);
#endif	/* DEBUG_USE_FERROR */
		return NULL;
	}
#endif	/* not NO_SIZE_T_CHECKS */


	retval = (uchar_t *)malloc(sizeof(uchar_t));
	if ( retval == NULL )
		return NULL;

	retval->uchar_t_asciilen = strlen(src) + 1;
	
	retval->uchar_t_ascii = (char *)malloc(
		sizeof(char) * retval->uchar_t_asciilen
	);
	if ( retval->uchar_t_ascii == NULL )
	{
		free(retval);

		return NULL;
	}

	(void)strncpy(retval->uchar_t_ascii, src, retval->uchar_t_asciilen);

	return retval;
}


TEKLTI_DECLSPEC void ucharempty ( uchar_t * uchrtofree )
{
	/* Perform all pointer checks as necessary. */
#ifndef	NO_SIZE_T_CHECKS
	if ( uchrtofree == NULL )
	{
#ifdef	DEBUG_USE_FERROR
		fputs(ucharempty_uchrtofreenull, stderr);
#endif	/* DEBUG_USE_FERROR */
		raise(1);
		return;
	}
#endif	/* not NO_SIZE_T_CHECKS */

	/* Free the contained string first, and enforce privacy as necessary. */
#ifdef	TEKLTI_ENFORCE_PRIVACY
	free(
		memset(
			uchrtofree->uchar_t_ascii,
			'\0',
			sizeof(char) * uchrtofree->uchar_t_asciilen
		)
	);
#else	/* not USE_386_ASM_ENFORCE_PRIVACY */
	free(uchrtofree->uchar_t_ascii);
#endif	/* not USE_386_ASM_ENFORCE_PRIVACY */

	free(uchrtofree);
}

#endif	/* not USE_386_ASM */


